@@ -140,7 +140,8 @@ const int KateIndentConfigTab::flags[] = {
     KateDocument::cfTabIndents,
     KateDocument::cfBackspaceIndents,
     KateDocumentConfig::cfDoxygenAutoTyping,
-    KateDocumentConfig::cfMixedIndent
+    KateDocumentConfig::cfMixedIndent,
+    KateDocumentConfig::cfIndentPastedText
 };
 
 KateIndentConfigTab::KateIndentConfigTab(QWidget *parent)
@@ -160,6 +161,7 @@ KateIndentConfigTab::KateIndentConfigTab(QWidget *parent)
   m_configPage = new QPushButton(SmallIconSet("configure"), i18n("Configure..."), indentLayout);
 
   opt[5] = new QCheckBox(i18n("Insert leading Doxygen \"*\" when typing"), gbAuto);
+  opt[7] = new QCheckBox(i18n("Adjust indentation of code pasted from the clipboard"), gbAuto);
 
   QVGroupBox *gbSpaces = new QVGroupBox(i18n("Indentation with Spaces"), this);
   QVBox *spaceLayout = new QVBox(gbSpaces);
@@ -191,6 +193,7 @@ KateIndentConfigTab::KateIndentConfigTab(QWidget *parent)
   opt[4]->setChecked(configFlags & flags[4]);
   opt[5]->setChecked(configFlags & flags[5]);
   opt[6]->setChecked(configFlags & flags[6]);
+  opt[7]->setChecked(configFlags & flags[7]);
 
   layout->addWidget(gbAuto);
   layout->addWidget(gbSpaces);
@@ -218,6 +221,9 @@ KateIndentConfigTab::KateIndentConfigTab(QWidget *parent)
         "style comment."));
   QWhatsThis::add( opt[6], i18n(
       "Use a mix of tab and space characters for indentation.") );
+  QWhatsThis::add( opt[7], i18n(
+      "If this option is selected, pasted code from the clipboard is indented. "
+      "Triggering the <b>undo</b>-action removes the indentation.") );
   QWhatsThis::add(indentationWidth, i18n("The number of spaces to indent with."));
 
   QWhatsThis::add(m_configPage, i18n(
@@ -242,6 +248,7 @@ KateIndentConfigTab::KateIndentConfigTab(QWidget *parent)
   connect( opt[4], SIGNAL( toggled(bool) ), this, SLOT( slotChanged() ) );
   connect( opt[5], SIGNAL( toggled(bool) ), this, SLOT( slotChanged() ) );
   connect( opt[6], SIGNAL( toggled(bool) ), this, SLOT( slotChanged() ) );
+  connect( opt[7], SIGNAL( toggled(bool) ), this, SLOT( slotChanged() ) );
 
   connect(indentationWidth, SIGNAL(valueChanged(int)), this, SLOT(slotChanged()));
 
@@ -349,7 +356,7 @@ KateSelectConfigTab::KateSelectConfigTab(QWidget *parent)
 
   QVGroupBox *gbCursor = new QVGroupBox(i18n("Text Cursor Movement"), this);
 
-  opt[0] = new QCheckBox(i18n("Smart ho&me"), gbCursor);
+  opt[0] = new QCheckBox(i18n("Smart ho&me and smart end"), gbCursor);
   opt[0]->setChecked(configFlags & KateDocumentConfig::cfSmartHome);
   connect(opt[0], SIGNAL(toggled(bool)), this, SLOT(slotChanged()));
 
@@ -391,7 +398,8 @@ KateSelectConfigTab::KateSelectConfigTab(QWidget *parent)
 
   QWhatsThis::add(opt[0], i18n(
         "When selected, pressing the home key will cause the cursor to skip "
-        "whitespace and go to the start of a line's text."));
+        "whitespace and go to the start of a line's text. "
+        "The same applies for the end key."));
 
     QWhatsThis::add(opt[1], i18n(
         "When on, moving the insertion cursor using the <b>Left</b> and "
@@ -433,7 +441,7 @@ void KateSelectConfigTab::apply ()
 
   KateDocumentConfig::global()->setConfigFlags(configFlags);
 
-  KateViewConfig::global()->setAutoCenterLines(QMAX(0, e4->value()));
+  KateViewConfig::global()->setAutoCenterLines(kMax(0, e4->value()));
   KateDocumentConfig::global()->setPageUpDownMovesCursor(e6->isChecked());
 
   KateViewConfig::global()->setPersistentSelection (m_tabs->id (m_tabs->selected()) == 1);
@@ -1244,9 +1252,10 @@ void KatePartPluginConfigPage::slotConfigure()
 //END KatePartPluginConfigPage
 
 //BEGIN KateHlConfigPage
-KateHlConfigPage::KateHlConfigPage (QWidget *parent)
+KateHlConfigPage::KateHlConfigPage (QWidget *parent, KateDocument *doc)
  : KateConfigPage (parent, "")
  , hlData (0)
+ , m_doc (doc)
 {
   QVBoxLayout *layout = new QVBoxLayout(this, 0, KDialog::spacingHint() );
 
@@ -1267,7 +1276,6 @@ KateHlConfigPage::KateHlConfigPage (QWidget *parent)
     else
       hlCombo->insertItem(KateHlManager::self()->hlNameTranslated(i));
   }
-  hlCombo->setCurrentItem(0);
 
   QGroupBox *gbInfo = new QGroupBox( 1, Qt::Horizontal, i18n("Information"), this );
   layout->add (gbInfo);
@@ -1316,8 +1324,9 @@ KateHlConfigPage::KateHlConfigPage (QWidget *parent)
   QPushButton *btnDl = new QPushButton(i18n("Do&wnload..."), hbBtns);
   connect( btnDl, SIGNAL(clicked()), this, SLOT(hlDownload()) );
 
-  hlCombo->setCurrentItem( 0 );
-  hlChanged(0);
+  int currentHl = m_doc ? m_doc->hlMode() : 0;
+  hlCombo->setCurrentItem( currentHl );
+  hlChanged( currentHl );
 
   QWhatsThis::add( hlCombo, i18n(
         "Choose a <em>Syntax Highlight mode</em> from this list to view its "
@@ -1634,7 +1643,7 @@ void KateModOnHdPrompt::slotDiff()
   // Start a KProcess that creates a diff
   KProcIO *p = new KProcIO();
   p->setComm( KProcess::All );
-  *p << "diff" << "-ub" << "-" <<  m_doc->url().path();
+  *p << "diff" << "-u" << "-" <<  m_doc->url().path();
   connect( p, SIGNAL(processExited(KProcess*)), this, SLOT(slotPDone(KProcess*)) );
   connect( p, SIGNAL(readReady(KProcIO*)), this, SLOT(slotPRead(KProcIO*)) );
 
@@ -1644,7 +1653,7 @@ void KateModOnHdPrompt::slotDiff()
 
   uint lastln =  m_doc->numLines();
   for ( uint l = 0; l <  lastln; l++ )
-    p->writeStdin(  m_doc->textLine( l ), l < lastln );
+    p->writeStdin( m_doc->textLine( l ) );
 
   p->closeWhenDone();
 }
@@ -1656,15 +1665,32 @@ void KateModOnHdPrompt::slotPRead( KProcIO *p)
     m_tmpfile = new KTempFile();
   // put all the data we have in it
   QString stmp;
+  bool dataRead = false;
   while ( p->readln( stmp, false ) > -1 )
+  {
     *m_tmpfile->textStream() << stmp << endl;
+    dataRead = true;
+  }
 
-  p->ackRead();
+  // dominik: only ackRead(), when we *really* read data, otherwise, this slot
+  // is called initity times, which leads to a crash
+  if( dataRead )
+    p->ackRead();
 }
 
 void KateModOnHdPrompt::slotPDone( KProcess *p )
 {
   setCursor( ArrowCursor );
+  if( ! m_tmpfile )
+  {
+    // dominik: there were only whitespace changes, so that the diff returned by
+    // diff(1) has 0 bytes. So slotPRead() is never called, as there is
+    // no data, so that m_tmpfile was never created and thus is NULL.
+    // NOTE: would be nice, if we could produce a fake-diff, so that kompare
+    //       tells us "The files are identical". Right now, we get an ugly
+    //       "Could not parse diff output".
+    m_tmpfile = new KTempFile();
+  }
   m_tmpfile->close();
 
   if ( ! p->normalExit() /*|| p->exitStatus()*/ )
@@ -1673,6 +1699,8 @@ void KateModOnHdPrompt::slotPDone( KProcess *p )
                         i18n("The diff command failed. Please make sure that "
                              "diff(1) is installed and in your PATH."),
                         i18n("Error Creating Diff") );
+    delete m_tmpfile;
+    m_tmpfile = 0;
     return;
   }
 
