@@ -64,6 +64,7 @@ class KNetwork::KSocketDevicePrivate
 {
 public:
   mutable QSocketNotifier *input, *output, *exception;
+  KSocketAddress local, peer;
   int af;
 
   inline KSocketDevicePrivate()
@@ -187,6 +188,9 @@ void KSocketDevice::close()
 
       d->input = d->output = d->exception = 0L;
 
+      d->local.setFamily(AF_UNSPEC);
+      d->peer.setFamily(AF_UNSPEC);
+
       ::close(m_sockfd);
     }
   setState(0);
@@ -216,6 +220,7 @@ bool KSocketDevice::create(int family, int type, int protocol)
 
   d->af = family;
   setSocketOptions(socketOptions());
+  setState(IO_Open);
   return true;		// successfully created
 }
 
@@ -259,7 +264,6 @@ bool KSocketDevice::listen(int backlog)
 
       resetError();
       setFlags(IO_Sequential | IO_Raw | IO_ReadWrite);
-      setState(IO_Open);
       return true;
     }
 
@@ -299,7 +303,6 @@ bool KSocketDevice::connect(const KResolverEntry& address)
     }
 
   setFlags(IO_Sequential | IO_Raw | IO_ReadWrite);
-  setState(IO_Open);
   return true;			// all is well
 }
 
@@ -529,18 +532,21 @@ KSocketAddress KSocketDevice::localAddress() const
   if (m_sockfd == -1)
     return KSocketAddress();	// not open, empty value
 
+  if (d->local.family() != AF_UNSPEC)
+    return d->local;
+
   socklen_t len;
   KSocketAddress localAddress;
   localAddress.setLength(len = 32);	// arbitrary value
   if (kde_getsockname(m_sockfd, localAddress.address(), &len) == -1)
     // error!
-    return KSocketAddress();
+    return d->local = KSocketAddress();
 
   if (len <= localAddress.length())
     {
       // it has fit already
       localAddress.setLength(len);
-      return localAddress;
+      return d->local = localAddress;
     }
 
   // no, the socket address is actually larger than we had anticipated
@@ -548,9 +554,9 @@ KSocketAddress KSocketDevice::localAddress() const
   localAddress.setLength(len);
   if (kde_getsockname(m_sockfd, localAddress.address(), &len) == -1)
     // error!
-    return KSocketAddress();
+    return d->local = KSocketAddress();
 
-  return localAddress;
+  return d->local = localAddress;
 }
 
 KSocketAddress KSocketDevice::peerAddress() const
@@ -558,18 +564,21 @@ KSocketAddress KSocketDevice::peerAddress() const
   if (m_sockfd == -1)
     return KSocketAddress();	// not open, empty value
 
+  if (d->peer.family() != AF_UNSPEC)
+    return d->peer;
+
   socklen_t len;
   KSocketAddress peerAddress;
   peerAddress.setLength(len = 32);	// arbitrary value
   if (kde_getpeername(m_sockfd, peerAddress.address(), &len) == -1)
     // error!
-    return KSocketAddress();
+    return d->peer = KSocketAddress();
 
   if (len <= peerAddress.length())
     {
       // it has fit already
       peerAddress.setLength(len);
-      return peerAddress;
+      return d->peer = peerAddress;
     }
 
   // no, the socket address is actually larger than we had anticipated
@@ -577,9 +586,9 @@ KSocketAddress KSocketDevice::peerAddress() const
   peerAddress.setLength(len);
   if (kde_getpeername(m_sockfd, peerAddress.address(), &len) == -1)
     // error!
-    return KSocketAddress();
+    return d->peer = KSocketAddress();
 
-  return peerAddress;
+  return d->peer = peerAddress;
 }
 
 KSocketAddress KSocketDevice::externalAddress() const
