@@ -14,8 +14,8 @@
 
     You should have received a copy of the GNU Library General Public License
     aint with this library; see the file COPYING.LIB.  If not, write to
-    the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
-    Boston, MA 02111-1307, USA.
+    the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
+    Boston, MA 02110-1301, USA.
 */
 
 #include <qvaluevector.h>
@@ -106,10 +106,16 @@ public:
 
 		for(QMap<QString, ArtGradientLinear *>::Iterator it = m_linearGradientMap.begin(); it != m_linearGradientMap.end(); ++it)
 		{
+			if (!it.data())
+				continue;
+			delete [] it.data()->stops;
 			delete it.data();
 		}
 		for(QMap<QString, ArtGradientRadial *>::Iterator it = m_radialGradientMap.begin(); it != m_radialGradientMap.end(); ++it)
 		{
+			if (!it.data())
+				continue;
+			delete [] it.data()->stops;
 			delete it.data();
 		}
 	}
