@@ -10,6 +10,7 @@
 #include <qtextcodec.h>
 #include <qdatastream.h>
 #include <assert.h>
+#include <kcmdlineargs.h>
 
 static bool check(QString txt, QString a, QString b)
 {
@@ -27,14 +28,62 @@ static bool check(QString txt, QString a, QString b)
   return true;
 }
 
-#if QT_VERSION < 300
-extern void qt_set_locale_codec( QTextCodec *codec );
-#endif
+void testAdjustPath()
+{
+    KURL url1("file:///home/kde/");
+    url1.adjustPath(0);
+    check( "adjustPath(0)", url1.path(), "/home/kde/" );
+    url1.adjustPath(-1);
+    check( "adjustPath(-1) removes last slash", url1.path(), "/home/kde" );
+    url1.adjustPath(-1);
+    check( "adjustPath(-1) again", url1.path(), "/home/kde" );
+    url1.adjustPath(1);
+    check( "adjustPath(1)", url1.path(), "/home/kde/" );
+
+    KURL url2("file:///home/kde//");
+    url2.adjustPath(0);
+    check( "adjustPath(0)", url2.path(), "/home/kde//" );
+    url2.adjustPath(-1);
+    check( "adjustPath(-1) removes all trailing slashes", url2.path(), "/home/kde" );
+    url2.adjustPath(1);
+    check( "adjustPath(1)", url2.path(), "/home/kde/" );
+
+    KURL ftpurl1("ftp://ftp.kde.org/");
+    ftpurl1.adjustPath(0);
+    check( "adjustPath(0)", ftpurl1.path(), "/" );
+    ftpurl1.adjustPath(-1);
+    check( "adjustPath(-1) preserves last slash", ftpurl1.path(), "/" );
+
+    KURL ftpurl2("ftp://ftp.kde.org///");
+    ftpurl2.adjustPath(0);
+    check( "adjustPath(0)", ftpurl2.path(), "///" );
+    ftpurl2.adjustPath(-1);
+    check( "adjustPath(-1) removes all but last slash", ftpurl2.path(), "/" );
+    ftpurl2.adjustPath(1);
+    check( "adjustPath(1)", ftpurl2.path(), "/" );
+
+    // Equivalent tests written by the KDirLister maintainer :)
+
+    KURL u3( QCString("ftp://brade@ftp.kde.org///") );
+    u3.adjustPath(-1);
+    check("KURL::adjustPath()", u3.url(), "ftp://brade@ftp.kde.org/");
+
+    KURL u4( QCString("ftp://brade@ftp.kde.org/kde///") );
+    u4.adjustPath(-1);
+    check("KURL::adjustPath()", u4.url(), "ftp://brade@ftp.kde.org/kde");
+
+    // applying adjustPath(-1) twice should not yield two different urls
+    // (follows from the above test)
+    KURL u5 = u4;
+    u5.adjustPath(-1);
+    check("KURL::adjustPath()", u5.url(), u4.url());
+}
 
 int main(int argc, char *argv[])
 {
   KApplication::disableAutoDcopRegistration();
-  KApplication app(argc,argv,"kurltest",false,false);
+  KCmdLineArgs::init( argc, argv, "kurltest", 0, 0, 0, 0 );
+  KApplication app( false, false );
 
   KURL::List lst;
 
@@ -52,10 +101,14 @@ int main(int argc, char *argv[])
   KURL fileURL = "file:/";
   check( "KURL::isEmpty()", fileURL.isEmpty() ? "TRUE":"FALSE", "FALSE");
 
+  fileURL = "file:///";
+  check( "KURL::isEmpty()", fileURL.isEmpty() ? "TRUE":"FALSE", "FALSE");
+
   KURL baseURL ("hTTp://www.foo.bar:80" );
   check( "KURL::isValid()", baseURL.isValid() ? "TRUE":"FALSE", "TRUE");
   check( "KURL::protocol()", baseURL.protocol(), "http"); // lowercase
   KURL url1 ( baseURL, "//www1.foo.bar" );
+  check( "KURL::host()", url1.host(), "www1.foo.bar");
   check( "KURL::url()", url1.url(), "http://www1.foo.bar");
 
   baseURL = "http://www.foo.bar";
@@ -67,7 +120,7 @@ int main(int argc, char *argv[])
   check( "KURL::url()", baseURL.url(), "http://www.foo.bar/top//test2/file2.html" );
 
   baseURL = "file:/usr/local/src/kde2/////kdelibs/kio";
-  check( "KURL::url()", baseURL.url(), "file:/usr/local/src/kde2/////kdelibs/kio" );
+  check( "KURL::url()", baseURL.url(), "file:///usr/local/src/kde2/////kdelibs/kio" );
 
   baseURL = "http://www.foo.bar";
   KURL rel_url2( baseURL, "mailto:bastian@kde.org" );
@@ -78,16 +131,42 @@ int main(int argc, char *argv[])
 
   baseURL = "file:/usr/local/src/kde2/kdelibs/kio/";
   KURL url2( baseURL, "../../////kdebase/konqueror" );
-  check( "KURL::url()", url2.url(), "file:/usr/local/src/kde2/////kdebase/konqueror" );
+  check( "KURL::url()", url2.url(), "file:///usr/local/src/kde2/////kdebase/konqueror" );
 
   QString u1 = "file:/home/dfaure/my#myref";
   url1 = u1;
-  check("KURL::url()", url1.url(), "file:/home/dfaure/my#myref");
+  check("KURL::url()", url1.url(), "file:///home/dfaure/my#myref");
   check("KURL::hasRef()", url1.hasRef() ? "yes" : "no", "yes");
   check("KURL::hasHTMLRef()", url1.hasHTMLRef() ? "yes" : "no", "yes");
   check("KURL::hasSubURL()", url1.hasSubURL() ? "yes" : "no", "no");
   check("KURL::htmlRef()", url1.htmlRef(), "myref");
-  check("KURL::upURL()", url1.upURL().url(), "file:/home/dfaure/");
+  check("KURL::upURL()", url1.upURL().url(), "file:///home/dfaure/");
+
+  u1 = "file:/home/dfaure/my#%2f";
+  url1 = u1;
+  check("KURL::url()", url1.url(), "file:///home/dfaure/my#%2f");
+  check("KURL::hasRef()", url1.hasRef() ? "yes" : "no", "yes");
+  check("KURL::hasHTMLRef()", url1.hasHTMLRef() ? "yes" : "no", "yes");
+  check("KURL::hasSubURL()", url1.hasSubURL() ? "yes" : "no", "no");
+  check("KURL::encodedHtmlRef()", url1.ref(), "%2f");
+  check("KURL::htmlRef()", url1.htmlRef(), "/");
+
+  url1 = KURL(url1, "#%6a");
+  check("KURL::url()", url1.url(), "file:///home/dfaure/my#%6a");
+  check("KURL::hasRef()", url1.hasRef() ? "yes" : "no", "yes");
+  check("KURL::hasHTMLRef()", url1.hasHTMLRef() ? "yes" : "no", "yes");
+  check("KURL::hasSubURL()", url1.hasSubURL() ? "yes" : "no", "no");
+  check("KURL::encodedHtmlRef()", url1.ref(), "%6a");
+  check("KURL::htmlRef()", url1.htmlRef(), "j");
+
+  u1 = "file:///home/dfaure/my#myref";
+  url1 = u1;
+  check("KURL::url()", url1.url(), "file:///home/dfaure/my#myref");
+  check("KURL::hasRef()", url1.hasRef() ? "yes" : "no", "yes");
+  check("KURL::hasHTMLRef()", url1.hasHTMLRef() ? "yes" : "no", "yes");
+  check("KURL::hasSubURL()", url1.hasSubURL() ? "yes" : "no", "no");
+  check("KURL::htmlRef()", url1.htmlRef(), "myref");
+  check("KURL::upURL()", url1.upURL().url(), "file:///home/dfaure/");
 
   url1 = "gg:www.kde.org";
   check("KURL::isValid()", url1.isValid()?"TRUE":"FALSE", "TRUE" );
@@ -100,31 +179,53 @@ int main(int argc, char *argv[])
 
   u1 = "file:/opt/kde2/qt2/doc/html/showimg-main-cpp.html#QObject::connect";
   url1 = u1;
-  check("KURL::url()", url1.url(), "file:/opt/kde2/qt2/doc/html/showimg-main-cpp.html#QObject::connect");
+  check("KURL::url()", url1.url(), "file:///opt/kde2/qt2/doc/html/showimg-main-cpp.html#QObject::connect");
   check("KURL::hasRef()", url1.hasRef() ? "yes" : "no", "yes");
   check("KURL::hasHTMLRef()", url1.hasHTMLRef() ? "yes" : "no", "yes");
   check("KURL::htmlRef()", url1.htmlRef(), "QObject::connect");
   check("KURL::hasSubURL()", url1.hasSubURL() ? "yes" : "no", "no");
-  check("KURL::upURL()", url1.upURL().url(), "file:/opt/kde2/qt2/doc/html/");
+  check("KURL::upURL()", url1.upURL().url(), "file:///opt/kde2/qt2/doc/html/");
+
+  url1 = KURL( QCString( "http://www.kde.org/foo.cgi?foo=bar" ) );
+  check("query", url1.query(), "?foo=bar" );
+  url1.setQuery( "toto=titi&kde=rocks" );
+  check("query", url1.query(), "?toto=titi&kde=rocks" );
+  url1.setQuery( "?kde=rocks&a=b" );
+  check("query", url1.query(), "?kde=rocks&a=b" );
+  url1.setQuery( "?" );
+  check("setQuery(\"?\") -> query", url1.query(), "?" );
+  url1.setQuery( "" );
+  check("setQuery(\"\") -> query", url1.query(), "?" );
+  url1.setQuery( QString::null );
+  check("setQuery(QString::null) -> query", url1.query(), QString::null );
+
+  u1 = "file:///opt/kde2/qt2/doc/html/showimg-main-cpp.html#QObject::connect";
+  url1 = u1;
+  check("KURL::url()", url1.url(), "file:///opt/kde2/qt2/doc/html/showimg-main-cpp.html#QObject::connect");
+  check("KURL::hasRef()", url1.hasRef() ? "yes" : "no", "yes");
+  check("KURL::hasHTMLRef()", url1.hasHTMLRef() ? "yes" : "no", "yes");
+  check("KURL::htmlRef()", url1.htmlRef(), "QObject::connect");
+  check("KURL::hasSubURL()", url1.hasSubURL() ? "yes" : "no", "no");
+  check("KURL::upURL()", url1.upURL().url(), "file:///opt/kde2/qt2/doc/html/");
 
   u1 = "file:/opt/kde2/qt2/doc/html/showimg-main-cpp.html#QObject:connect";
   url1 = u1;
-  check("KURL::url()", url1.url(), "file:/opt/kde2/qt2/doc/html/showimg-main-cpp.html#QObject:connect");
+  check("KURL::url()", url1.url(), "file:///opt/kde2/qt2/doc/html/showimg-main-cpp.html#QObject:connect");
   check("KURL::hasRef()", url1.hasRef() ? "yes" : "no", "yes");
   check("KURL::hasHTMLRef()", url1.hasHTMLRef() ? "yes" : "no", "yes");
   check("KURL::htmlRef()", url1.htmlRef(), "QObject:connect");
   check("KURL::hasSubURL()", url1.hasSubURL() ? "yes" : "no", "no");
-  check("KURL::upURL()", url1.upURL().url(), "file:/opt/kde2/qt2/doc/html/");
+  check("KURL::upURL()", url1.upURL().url(), "file:///opt/kde2/qt2/doc/html/");
 
   u1 = "file:/home/dfaure/my%20tar%20file.tgz#gzip:/#tar:/#myref";
   url1 = u1;
-  check("KURL::url()", url1.url(), "file:/home/dfaure/my%20tar%20file.tgz#gzip:/#tar:/#myref");
+  check("KURL::url()", url1.url(), "file:///home/dfaure/my%20tar%20file.tgz#gzip:/#tar:/#myref");
   check("KURL::hasRef()", url1.hasRef() ? "yes" : "no", "yes");
   check("KURL::isLocalFile()", url1.isLocalFile() ? "yes" : "no", "no"); // Not strictly local!
   //check("KURL::hasHTMLRef()", url1.hasHTMLRef() ? "yes" : "no", "yes");
   check("KURL::hasSubURL()", url1.hasSubURL() ? "yes" : "no", "yes");
   //check("KURL::htmlRef()", url1.htmlRef(), "myref");
-  check("KURL::upURL()", url1.upURL().url(), "file:/home/dfaure/");
+  check("KURL::upURL()", url1.upURL().url(), "file:///home/dfaure/");
 
   u1 = "error:/?error=14&errText=Unknown%20host%20asdfu.adgi.sdfgoi#http://asdfu.adgi.sdfgoi";
   url1 = u1;
@@ -137,38 +238,47 @@ int main(int argc, char *argv[])
 
   u1 = "file:/home/dfaure/my%20tar%20file.tgz#gzip:/#tar:/";
   url1 = u1;
-  check("KURL::url()", url1.url(), "file:/home/dfaure/my%20tar%20file.tgz#gzip:/#tar:/");
+  check("KURL::url()", url1.url(), "file:///home/dfaure/my%20tar%20file.tgz#gzip:/#tar:/");
   check("KURL::hasRef()", url1.hasRef() ? "yes" : "no", "yes");
   check("KURL::hasHTMLRef()", url1.hasHTMLRef() ? "yes" : "no", "no");
   check("KURL::htmlRef()", url1.htmlRef(), "");
   check("KURL::hasSubURL()", url1.hasSubURL() ? "yes" : "no", "yes");
-  check("KURL::upURL()", url1.upURL().url(), "file:/home/dfaure/");
+  check("KURL::upURL()", url1.upURL().url(), "file:///home/dfaure/");
+
+  u1 = "file:///home/dfaure/my%20tar%20file.tgz#gzip:/#tar:/";
+  url1 = u1;
+  check("KURL::url()", url1.url(), "file:///home/dfaure/my%20tar%20file.tgz#gzip:/#tar:/");
+  check("KURL::hasRef()", url1.hasRef() ? "yes" : "no", "yes");
+  check("KURL::hasHTMLRef()", url1.hasHTMLRef() ? "yes" : "no", "no");
+  check("KURL::htmlRef()", url1.htmlRef(), "");
+  check("KURL::hasSubURL()", url1.hasSubURL() ? "yes" : "no", "yes");
+  check("KURL::upURL()", url1.upURL().url(), "file:///home/dfaure/");
 
 #if 0
 // This URL is broken, '#' should be escaped.
   u1 = "file:/home/dfaure/cdrdao-1.1.5/dao/#CdrDriver.cc#";
   url1 = u1;
-  check("KURL::url()", url1.url(), "file:/home/dfaure/cdrdao-1.1.5/dao/#CdrDriver.cc#");
+  check("KURL::url()", url1.url(), "file:///home/dfaure/cdrdao-1.1.5/dao/#CdrDriver.cc#");
   check("KURL::hasRef()", url1.hasRef() ? "yes" : "no", "no");
   check("KURL::hasHTMLRef()", url1.hasHTMLRef() ? "yes" : "no", "no");
   check("KURL::htmlRef()", url1.htmlRef(), "");
   check("KURL::hasSubURL()", url1.hasSubURL() ? "yes" : "no", "yes");
-  check("KURL::prettyURL()", url1.upURL().url(), "file:/home/dfaure/cdrdao-1.1.5/dao/#CdrDriver.cc#");
+  check("KURL::prettyURL()", url1.upURL().url(), "file:///home/dfaure/cdrdao-1.1.5/dao/#CdrDriver.cc#");
 #endif
 
   u1 = "file:/home/dfaure/my%20tar%20file.tgz#gzip:/#tar:/README";
   url1 = u1;
-  check("KURL::url()", url1.url(), "file:/home/dfaure/my%20tar%20file.tgz#gzip:/#tar:/README");
+  check("KURL::url()", url1.url(), "file:///home/dfaure/my%20tar%20file.tgz#gzip:/#tar:/README");
   check("KURL::hasRef()", url1.hasRef() ? "yes" : "no", "yes");
   check("KURL::hasHTMLRef()", url1.hasHTMLRef() ? "yes" : "no", "no");
   check("KURL::htmlRef()", url1.htmlRef(), "");
   check("KURL::hasSubURL()", url1.hasSubURL() ? "yes" : "no", "yes");
-  check("KURL::upURL()", url1.upURL().url(), "file:/home/dfaure/my%20tar%20file.tgz#gzip:/#tar:/");
+  check("KURL::upURL()", url1.upURL().url(), "file:///home/dfaure/my%20tar%20file.tgz#gzip:/#tar:/");
 
   KURL notPretty("http://ferret.lmh.ox.ac.uk/%7Ekdecvs/");
   check("KURL::prettyURL()", notPretty.prettyURL(), "http://ferret.lmh.ox.ac.uk/~kdecvs/");
   KURL notPretty2("file:/home/test/directory%20with%20spaces");
-  check("KURL::prettyURL()", notPretty2.prettyURL(), "file:/home/test/directory with spaces");
+  check("KURL::prettyURL()", notPretty2.prettyURL(), "file:///home/test/directory with spaces");
   KURL notPretty3("fish://foo/%23README%23");
   check("KURL::prettyURL()", notPretty3.prettyURL(), "fish://foo/%23README%23");
   KURL url15581("http://alain.knaff.linux.lu/bug-reports/kde/spaces in url.html");
@@ -199,7 +309,7 @@ int main(int argc, char *argv[])
   udir.setPath("/home/dfaure/file.txt");
   printf("\n* URL is %s\n",udir.url().ascii());
   check("KURL::path()", udir.path(), "/home/dfaure/file.txt");
-  check("KURL::url()", udir.url(), "file:/home/dfaure/file.txt");
+  check("KURL::url()", udir.url(), "file:///home/dfaure/file.txt");
   check("KURL::directory(false,false)", udir.directory(false,false), "/home/dfaure/");
   check("KURL::directory(true,false)", udir.directory(true,false), "/home/dfaure");
 
@@ -211,12 +321,25 @@ int main(int argc, char *argv[])
   // ignoring trailing slash
   check("KURL::directory(false,true)", u2.directory(false,true), "/home/");
   check("KURL::directory(true,true)", u2.directory(true,true), "/home");
+
+  // cleanPath() tests (before cd() since cd uses that)
+  u2.cleanPath();
+  check("cleanPath(false)", u2.url(), "file:///home/dfaure/");
+  u2.addPath( "/..//foo" );
+  check("addPath", u2.url(), "file:///home/dfaure/..//foo");
+  u2.cleanPath(false);
+  check("cleanPath()", u2.url(), "file:///home//foo");
+  u2.cleanPath(true);
+  check("cleanPath()", u2.url(), "file:///home/foo");
+
   u2.cd("..");
-  check("KURL::cd(\"..\")", u2.url(), "file:/home");
+  check("KURL::cd(\"..\")", u2.url(), "file:///home");
   u2.cd("thomas");
-  check("KURL::cd(\"thomas\")", u2.url(), "file:/home/thomas");
+  check("KURL::cd(\"thomas\")", u2.url(), "file:///home/thomas");
+  u2.cd("../");
+  check("KURL::cd(\"../\")", u2.url(), "file:///home/");
   u2.cd("/opt/kde/bin/");
-  check("KURL::cd(\"/opt/kde/bin/\")", u2.url(), "file:/opt/kde/bin/");
+  check("KURL::cd(\"/opt/kde/bin/\")", u2.url(), "file:///opt/kde/bin/");
   u2 = "ftp://ftp.kde.org/";
   printf("\n* URL is %s\n",u2.url().ascii());
   u2.cd("pub");
@@ -227,9 +350,9 @@ int main(int argc, char *argv[])
   printf("\n* URL is %s\n",u2.url().ascii());
   // setFileName
   u2.setFileName( "myfile.txt" );
-  check("KURL::setFileName()", u2.url(), "file:/home/dfaure/myfile.txt");
+  check("KURL::setFileName()", u2.url(), "file:///home/dfaure/myfile.txt");
   u2.setFileName( "myotherfile.txt" );
-  check("KURL::setFileName()", u2.url(), "file:/home/dfaure/myotherfile.txt");
+  check("KURL::setFileName()", u2.url(), "file:///home/dfaure/myotherfile.txt");
   // more tricky, renaming a directory (kpropsdlg.cc, line ~ 238)
       QString tmpurl = "file:/home/dfaure/myolddir/";
       if ( tmpurl.at(tmpurl.length() - 1) == '/')
@@ -237,12 +360,12 @@ int main(int argc, char *argv[])
           tmpurl.truncate( tmpurl.length() - 1);
       KURL newUrl = tmpurl;
       newUrl.setFileName( "mynewdir" );
-  check("KURL::setFileName() special", newUrl.url(), "file:/home/dfaure/mynewdir");
+  check("KURL::setFileName() special", newUrl.url(), "file:///home/dfaure/mynewdir");
   // addPath tests
   newUrl.addPath( "subdir" );
-  check("KURL::addPath(\"subdir\")", newUrl.url(), "file:/home/dfaure/mynewdir/subdir");
+  check("KURL::addPath(\"subdir\")", newUrl.url(), "file:///home/dfaure/mynewdir/subdir");
   newUrl.addPath( "/foo/" );
-  check("KURL::addPath(\"/foo/\")", newUrl.url(), "file:/home/dfaure/mynewdir/subdir/foo/");
+  check("KURL::addPath(\"/foo/\")", newUrl.url(), "file:///home/dfaure/mynewdir/subdir/foo/");
   u2 = "http://www.kde.org"; // no path
   u2.addPath( "subdir" );
   check("KURL::addPath(\"subdir\")", u2.url(), "http://www.kde.org/subdir");
@@ -262,13 +385,13 @@ int main(int argc, char *argv[])
   check("KURL::path()", u2.path(), "/specials/Print");
   check("KURL::fileName()", u2.fileName(), "Print");
   u2.setFileName( "" );
-  check("KURL::setFileName()", u2.url(), "file:/specials/");
+  check("KURL::setFileName()", u2.url(), "file:///specials/");
 
-  const char * u3 = "ftp://host/dir1/dir2/myfile.txt";
-  printf("\n* URL is %s\n",u3);
-  check("KURL::hasSubURL()", KURL(u3).hasSubURL() ? "yes" : "no", "no");
+  const char * u6 = "ftp://host/dir1/dir2/myfile.txt";
+  printf("\n* URL is %s\n",u6);
+  check("KURL::hasSubURL()", KURL(u6).hasSubURL() ? "yes" : "no", "no");
   lst.clear();
-  lst = KURL::split( KURL(u3) );
+  lst = KURL::split( KURL(u6) );
   check("KURL::split()", lst.count()==1 ? "1" : "error", "1");
   check("KURL::split()", lst.first().url(), "ftp://host/dir1/dir2/myfile.txt");
   // cdUp code
@@ -303,7 +426,7 @@ int main(int argc, char *argv[])
   check("KURL::isParentOf()", ftpUrl.isParentOf( "ftp://ftp/host/subdi") ? "yes" : "no", "no");
   check("KURL::isParentOf()", ftpUrl.isParentOf( "ftp://ftp/host/subdir/blah/") ? "yes" : "no", "yes");
   check("KURL::isParentOf()", ftpUrl.isParentOf( "ftp://ftp/blah/subdir") ? "yes" : "no", "no");
-  check("KURL::isParentOf()", ftpUrl.isParentOf( "file://ftp/host/subdir/") ? "yes" : "no", "no");
+  check("KURL::isParentOf()", ftpUrl.isParentOf( "file:////ftp/host/subdir/") ? "yes" : "no", "no");
   check("KURL::isParentOf()", ftpUrl.isParentOf( "ftp://ftp/host/subdir/subsub") ? "yes" : "no", "yes");
 
   // WABA: The following tests are to test the handling of relative URLs as
@@ -592,6 +715,11 @@ int main(int argc, char *argv[])
   check("Broken stuff #5 path", broken.path(), "");
   broken = "file";
   check("Broken stuff #6 valid", broken.isValid()?"VALID":"MALFORMED", "MALFORMED");
+  broken = "/";
+  check("Broken stuff #7 valid", broken.isValid()?"VALID":"MALFORMED", "VALID");
+  check("Broken stuff #7 path", broken.path(), "/" );
+  check("Broken stuff #7 url", broken.url(), "file:///" );
+  check("Broken stuff #7 file", broken.protocol(), "file" );
 
   broken = "LABEL=USB_STICK"; // 71430, can we use KURL for this?
   check("Broken stuff #6 valid", broken.isValid()?"VALID":"MALFORMED", "MALFORMED");
@@ -602,11 +730,11 @@ int main(int argc, char *argv[])
   // UNC like names
   KURL unc1("FILE://localhost/home/root");
   check("UNC, with localhost", unc1.path(), "/home/root");
-  check("UNC, with localhost", unc1.url(), "file:/home/root");
+  check("UNC, with localhost", unc1.url(), "file:///home/root");
 #endif
   KURL unc2("file:///home/root");
   check("UNC, with empty host", unc2.path(), "/home/root");
-  check("UNC, with empty host", unc2.url(), "file:/home/root");
+  check("UNC, with empty host", unc2.url(), "file:///home/root");
 
   {
      KURL unc3("FILE://remotehost/home/root");
@@ -620,6 +748,10 @@ int main(int argc, char *argv[])
      //check("KURL::path()", url3.path(), "//atlas/dfaure"); // says Waba
      //KURL url3("file:////atlas/dfaure");
      //check("KURL::path()", url3.path(), "//atlas/dfaure"); // says Waba
+
+     KURL url4(url2, "//remotehost/home/root");
+     check("KURL::host()", url4.host(), "remotehost");
+     check("KURL::path()", url4.path(), "/home/root");
   }
 
   KURL umail1 ( "mailto:faure@kde.org" );
@@ -636,8 +768,8 @@ int main(int argc, char *argv[])
   check("man: URL, is relative", KURL::isRelativeURL("man:mmap") ? "true" : "false", "false");
   check("javascript: URL, is relative", KURL::isRelativeURL("javascript:doSomething()") ? "true" : "false", "false");
   // more isRelative
-  check("file: URL, is relative", KURL::isRelativeURL("file:/blah") ? "true" : "false", "false");
-  check("/path, is relative", KURL::isRelativeURL("/path") ? "true" : "false", "true"); // arguable
+  check("file: URL, is relative", KURL::isRelativeURL("file:///blah") ? "true" : "false", "false");
+  check("/path, is relative", KURL::isRelativeURL("/path") ? "true" : "false", "true"); // arguable, but necessary for KURL( baseURL, "//www1.foo.bar" );
   check("something, is relative", KURL::isRelativeURL("something") ? "true" : "false", "true");
   KURL about("about:konqueror");
   check("about:",about.path(),"konqueror");
@@ -651,26 +783,45 @@ int main(int argc, char *argv[])
 #else
   QTextCodec::setCodecForLocale( KGlobal::charsets()->codecForName( "iso-8859-1" ) );
 #endif
+  QString raw = "data:text/html,%00%2540%00";
+  check("data URL: encode-decode of %00", KURL(raw).url(), raw );
+
   // UTF8 tests
   KURL uloc("/home/dfaure/konqtests/Matriel");
-  check("locale8bit",uloc.url().latin1(),"file:/home/dfaure/konqtests/Mat%E9riel"); // escaping the letter would be correct too
-  check("pretty",uloc.prettyURL(),"file:/home/dfaure/konqtests/Matriel"); // escaping the letter would be correct too
+  check("url",uloc.url().latin1(),"file:///home/dfaure/konqtests/Mat%E9riel");
+  check("pretty",uloc.prettyURL(),"file:///home/dfaure/konqtests/Matriel"); // escaping the letter would be correct too
+  check("pretty + strip",uloc.prettyURL(0, KURL::StripFileProtocol),"/home/dfaure/konqtests/Matriel"); // escaping the letter would be correct too
   // 106 is MIB for UTF-8
-  check("UTF8",uloc.url(0, 106),"file:/home/dfaure/konqtests/Mat%C3%A9riel");
-  uloc = KURL("file:/home/dfaure/konqtests/Mat%C3%A9riel", 106);
+  check("UTF8",uloc.url(0, 106),"file:///home/dfaure/konqtests/Mat%C3%A9riel");
+  uloc = KURL("file:///home/dfaure/konqtests/Mat%C3%A9riel", 106);
   check("UTF8 path", uloc.path(), "/home/dfaure/konqtests/Matriel");
+  check("encodedPathAndQuery", uloc.encodedPathAndQuery(), "/home/dfaure/konqtests/Mat%E9riel");
 
   // fromPathOrURL tests
   uloc = KURL::fromPathOrURL( "/home/dfaure/konqtests/Mat%E9riel" );
   check("fromPathOrURL path", uloc.path(), "/home/dfaure/konqtests/Mat%E9riel");
   uloc = KURL::fromPathOrURL( "http://www.kde.org" );
-  check("fromPathOrURL url", uloc.url(), "http://www.kde.org");
+  check("pathOrURL url", uloc.pathOrURL(), uloc.url() );
   uloc = KURL::fromPathOrURL( "www.kde.org" );
-  check("fromPathOrURL url", uloc.isValid()?"valid":"malformed", "malformed");
+  check("fromPathOrURL malformed", uloc.isValid()?"valid":"malformed", "malformed");
   uloc = KURL::fromPathOrURL( "index.html" );
-  check("fromPathOrURL url", uloc.isValid()?"valid":"malformed", "malformed");
+  check("fromPathOrURL malformed", uloc.isValid()?"valid":"malformed", "malformed");
   uloc = KURL::fromPathOrURL( "" );
-  check("fromPathOrURL url", uloc.isValid()?"valid":"malformed", "malformed");
+  check("fromPathOrURL malformed", uloc.isValid()?"valid":"malformed", "malformed");
+
+  // pathOrURL tests
+  uloc = KURL::fromPathOrURL( "/home/dfaure/konqtests/Mat%E9riel" );
+  check("pathOrURL path", uloc.pathOrURL(), uloc.path() );
+  uloc = "http://www.kde.org";
+  check("pathOrURL url", uloc.url(), "http://www.kde.org");
+  uloc = "file:///home/dfaure/konq%20tests/Mat%E9riel#ref";
+  check("pathOrURL local file with ref", uloc.pathOrURL(), "file:///home/dfaure/konq tests/Matriel#ref" );
+  uloc = "file:///home/dfaure/konq%20tests/Mat%E9riel?query";
+  check("pathOrURL local file with query", uloc.pathOrURL(), "file:///home/dfaure/konq tests/Matriel?query" );
+  uloc = KURL::fromPathOrURL( "/home/dfaure/file#with#hash" );
+  check("pathOrURL local path with #", uloc.pathOrURL(), "/home/dfaure/file#with#hash" );
+
+  testAdjustPath();
 
 #if QT_VERSION < 300
   qt_set_locale_codec( KGlobal::charsets()->codecForName( "koi8-r" ) );
@@ -679,7 +830,7 @@ int main(int argc, char *argv[])
 #endif
   baseURL = "file:/home/coolo";
   KURL russian = baseURL.directory(false, true) + QString::fromLocal8Bit( "7" );
-  check( "russian", russian.url(), "file:/home/%C6%C7%CE7" );
+  check( "russian", russian.url(), "file:///home/%C6%C7%CE7" );
 
   KURL tobi1("http://some.host.net/path/to/file#fragmentPrecedes?theQuery");
   check("wrong order of query and hypertext reference #1", tobi1.ref(), "fragmentPrecedes");
@@ -690,14 +841,17 @@ int main(int argc, char *argv[])
 
   tobi1 = "http://host.net/path/#no-query";
   check("no query", tobi1.query(),"");
+  check("encodedPathAndQuery", tobi1.encodedPathAndQuery(), "/path/");
 
   tobi1 = "http://host.net/path?myfirstquery#andsomeReference";
   tobi1.setEncodedPathAndQuery("another/path/?another&query");
   check("setEncodedPathAndQuery test#1", tobi1.query(), "?another&query");
-  check("setEncodedPathAndQuery test#2", tobi1.path(), "another/path/");
+  check("setEncodedPathAndQuery test#2", tobi1.path(), "another/path/"); // with trailing slash
+  check("encodedPathAndQuery", tobi1.encodedPathAndQuery(), "another/path/?another&query");
   tobi1.setEncodedPathAndQuery("another/path?another&query");
   check("setEncodedPathAndQuery test#1", tobi1.query(), "?another&query");
-  check("setEncodedPathAndQuery test#2", tobi1.path(), "another/path");
+  check("setEncodedPathAndQuery test#2", tobi1.path(), "another/path"); // without trailing slash
+  check("encodedPathAndQuery", tobi1.encodedPathAndQuery(), "another/path?another&query");
 
   KURL theKow = "http://www.google.de/search?q=frerich&hlx=xx&hl=de&empty=&lr=lang+de&test=%2B%20%3A%25";
   check("queryItem (first item)", theKow.queryItem("q"), "frerich");
@@ -706,6 +860,7 @@ int main(int argc, char *argv[])
   check("queryItem (invalid item)", theKow.queryItem("InterstellarCounselor"), QString::null);
   check("queryItem (empty item)", theKow.queryItem("empty"), "");
   check("queryItem (item with encoded chars)", theKow.queryItem("test"), "+ :%");
+  check("encodedPathAndQuery", theKow.encodedPathAndQuery(), "/search?q=frerich&hlx=xx&hl=de&empty=&lr=lang+de&test=%2B%20%3A%25");
 
   // checks for queryItems(), which returns a QMap<QString,QString>:
   KURL queryUrl( "mailto:Marc%20Mutz%20%3cmutz@kde.org%3E?"
@@ -739,14 +894,23 @@ int main(int argc, char *argv[])
   check( "encode_string('%')", KURL::encode_string( "%" ), "%25" );
   check( "encode_string(':')", KURL::encode_string( ":" ), "%3A" );
 
-  KURL amantia( "http://%E1.foo" );
+  KURL amantia( "http://%E1.foo.de" );
   check("amantia.isValid()", amantia.isValid() ? "true" : "false", "true");
 #ifdef HAVE_IDNA_H
-  check("amantia.url()", amantia.url(), "http://xn--80a.foo");   // Non-ascii is allowed in IDN domain names.
+  check("amantia.url()", amantia.url(), "http://xn--80a.foo.de");   // Non-ascii is allowed in IDN domain names.
 #else
-  check("amantia.url()", amantia.url(), "http://?.foo"); // why not
+  check("amantia.url()", amantia.url(), "http://?.foo.de"); // why not
 #endif
 
+  KURL thiago( QString::fromUtf8( "http://\303\244.de" ) ); //  in utf8
+  check("thiago.isValid()", thiago.isValid() ? "true" : "false", "true");
+#ifdef HAVE_IDNA_H
+  check("thiago.url()", thiago.url(), "http://xn--4ca.de");   // Non-ascii is allowed in IDN domain names.
+#else
+  check("thiago.url()", thiago.url(), QString::fromUtf8( "http://\303\244.de" ) );
+#endif
+
+
   KURL smb("smb://domain;username:password@server/share");
   check("smb.isValid()", smb.isValid() ? "true" : "false", "true");
   check("smb.user()", smb.user(), "domain;username");
@@ -829,7 +993,7 @@ int main(int argc, char *argv[])
 
   KURL local_file_5;
   local_file_5.setPath("/foo?bar");
-  check("local_file_5.url()", local_file_5.url(), "file:/foo%3Fbar");
+  check("local_file_5.url()", local_file_5.url(), "file:///foo%3Fbar");
 
   QString basePath = "/home/bastian";
 
@@ -910,6 +1074,13 @@ int main(int argc, char *argv[])
   url1 = "mailto:User@Host.COM?subject=Hello";
   check("KURL(\"mailto:User@Host.COM?subject=Hello\").path()", url1.path(), "User@host.com");
 
+  KURL emptyUserTest1("http://www.foobar.com/");
+  KURL emptyUserTest2("http://www.foobar.com/");
+  emptyUserTest2.setUser("");
+  check("Empty vs. null fields: user", emptyUserTest1==emptyUserTest2?"TRUE":"FALSE","TRUE");
+  emptyUserTest2.setPass("");
+  check("Empty vs. null fields: password", emptyUserTest1==emptyUserTest2?"TRUE":"FALSE","TRUE");
+
   printf("\nTest OK !\n");
 }
 
