@@ -12,8 +12,8 @@
  *
  *  You should have received a copy of the GNU Library General Public License
  *  along with this library; see the file COPYING.LIB.  If not, write to
- *  the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
- *  Boston, MA 02111-1307, USA.
+ *  the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
+ *  Boston, MA 02110-1301, USA.
  **/
 
 #include <sys/types.h>
@@ -26,6 +26,7 @@
 #include <kglobal.h>
 #include <kstandarddirs.h>
 #include <kservice.h>
+#include <kde_file.h>
 
 #include <qmap.h>
 #include <qfile.h>
@@ -42,7 +43,7 @@ static void foldNode(QDomElement &docElem, QDomElement &e, QMap<QString,QDomElem
    if (it != dupeList.end())
    {
       kdDebug(7021) << e.tagName() << " and " << s << " requires combining!" << endl;
-          
+
       docElem.removeChild(*it);
       dupeList.remove(it);
    }
@@ -59,7 +60,7 @@ static void replaceNode(QDomElement &docElem, QDomNode &n, const QStringList &li
       e.appendChild(txt);
       docElem.insertAfter(e, n);
    }
-  
+
    QDomNode next = n.nextSibling();
    docElem.removeChild(n);
    n = next;
@@ -71,7 +72,7 @@ void VFolderMenu::registerFile(const QString &file)
    int i = file.findRev('/');
    if (i < 0)
       return;
-   
+
    QString dir = file.left(i+1); // Include trailing '/'
    registerDirectory(dir);
 }
@@ -104,6 +105,13 @@ QStringList VFolderMenu::allDirectories()
    return m_allDirectories;
 }
 
+static void
+track(const QString &menuId, const QString &menuName, QDict<KService> *includeList, QDict<KService> *excludeList, QDict<KService> *itemList, const QString &comment)
+{
+   if (itemList->find(menuId))
+      printf("%s: %s INCL %d EXCL %d\n", menuName.latin1(), comment.latin1(), includeList->find(menuId) ? 1 : 0, excludeList->find(menuId) ? 1 : 0);
+}
+
 void
 VFolderMenu::includeItems(QDict<KService> *items1, QDict<KService> *items2)
 {
@@ -134,13 +142,13 @@ VFolderMenu::excludeItems(QDict<KService> *items1, QDict<KService> *items2)
    }
 }
 
-VFolderMenu::SubMenu* 
+VFolderMenu::SubMenu*
 VFolderMenu::takeSubMenu(SubMenu *parentMenu, const QString &menuName)
 {
    int i = menuName.find('/');
    QString s1 = i > 0 ? menuName.left(i) : menuName;
    QString s2 = menuName.mid(i+1);
-      
+
    // Look up menu
    for(SubMenu *menu = parentMenu->subMenus.first(); menu; menu = parentMenu->subMenus.next())
    {
@@ -163,6 +171,11 @@ VFolderMenu::takeSubMenu(SubMenu *parentMenu, const QString &menuName)
 void
 VFolderMenu::mergeMenu(SubMenu *menu1, SubMenu *menu2, bool reversePriority)
 {
+   if (m_track)
+   {
+      track(m_trackId, menu1->name, &(menu1->items), &(menu1->excludeItems), &(menu2->items), QString("Before MenuMerge w. %1 (incl)").arg(menu2->name));
+      track(m_trackId, menu1->name, &(menu1->items), &(menu1->excludeItems), &(menu2->excludeItems), QString("Before MenuMerge w. %1 (excl)").arg(menu2->name));
+   }
    if (reversePriority)
    {
       // Merge menu1 with menu2, menu1 takes precedent
@@ -206,17 +219,23 @@ VFolderMenu::mergeMenu(SubMenu *menu1, SubMenu *menu2, bool reversePriority)
          menu1->layoutNode = menu2->layoutNode;
    }
 
+   if (m_track)
+   {
+      track(m_trackId, menu1->name, &(menu1->items), &(menu1->excludeItems), &(menu2->items), QString("After MenuMerge w. %1 (incl)").arg(menu2->name));
+      track(m_trackId, menu1->name, &(menu1->items), &(menu1->excludeItems), &(menu2->excludeItems), QString("After MenuMerge w. %1 (excl)").arg(menu2->name));
+   }
+
    delete menu2;
 }
 
-void 
+void
 VFolderMenu::insertSubMenu(SubMenu *parentMenu, const QString &menuName, SubMenu *newMenu, bool reversePriority)
 {
    int i = menuName.find('/');
-   
+
    QString s1 = menuName.left(i);
    QString s2 = menuName.mid(i+1);
-      
+
    // Look up menu
    for(SubMenu *menu = parentMenu->subMenus.first(); menu; menu = parentMenu->subMenus.next())
    {
@@ -249,11 +268,11 @@ VFolderMenu::insertSubMenu(SubMenu *parentMenu, const QString &menuName, SubMenu
    }
 }
 
-void 
+void
 VFolderMenu::insertService(SubMenu *parentMenu, const QString &name, KService *newService)
 {
    int i = name.find('/');
-   
+
    if (i == -1)
    {
      // Add it here
@@ -263,7 +282,7 @@ VFolderMenu::insertService(SubMenu *parentMenu, const QString &name, KService *n
 
    QString s1 = name.left(i);
    QString s2 = name.mid(i+1);
-      
+
    // Look up menu
    for(SubMenu *menu = parentMenu->subMenus.first(); menu; menu = parentMenu->subMenus.next())
    {
@@ -273,7 +292,7 @@ VFolderMenu::insertService(SubMenu *parentMenu, const QString &name, KService *n
          return;
       }
    }
-   
+
    SubMenu *menu = new SubMenu;
    menu->name = s1;
    parentMenu->subMenus.append(menu);
@@ -281,7 +300,7 @@ VFolderMenu::insertService(SubMenu *parentMenu, const QString &name, KService *n
 }
 
 
-VFolderMenu::VFolderMenu() : m_usedAppsDict(797)
+VFolderMenu::VFolderMenu() : m_usedAppsDict(797), m_track(false)
 {
    m_rootMenu = 0;
    initDirs();
@@ -308,7 +327,7 @@ VFolderMenu::~VFolderMenu()
       KService::List *list = info->dictCategories.find(category); \
       if (list) for(KService::List::ConstIterator it = list->begin(); \
              it != list->end(); ++it) \
-      { 
+      {
 #define FOR_CATEGORY_END } }
 
 KService *
@@ -351,7 +370,7 @@ VFolderMenu::buildApplicationIndex(bool unusedOnly)
             info->applications.remove(tmpIt.currentKey());
             continue;
          }
-         
+
          QStringList cats = s->categories();
          for(QStringList::ConstIterator it2 = cats.begin();
              it2 != cats.end(); ++it2)
@@ -386,10 +405,10 @@ VFolderMenu::loadAppsInfo()
    m_appsInfo = m_currentMenu->apps_info;
    if (!m_appsInfo)
       return; // No appsInfo for this menu
-   
+
    if (m_appsInfoStack.first() == m_appsInfo)
       return; // Already added (By createAppsInfo?)
-      
+
    m_appsInfoStack.prepend(m_appsInfo); // Add
 }
 
@@ -399,32 +418,32 @@ VFolderMenu::unloadAppsInfo()
    m_appsInfo = m_currentMenu->apps_info;
    if (!m_appsInfo)
       return; // No appsInfo for this menu
-   
+
    if (m_appsInfoStack.first() != m_appsInfo)
    {
       return; // Already removed (huh?)
    }
-      
+
    m_appsInfoStack.remove(m_appsInfo); // Remove
    m_appsInfo = 0;
 }
 
-QString 
+QString
 VFolderMenu::absoluteDir(const QString &_dir, const QString &baseDir, bool keepRelativeToCfg)
 {
    QString dir = _dir;
-   if (!dir.startsWith("/"))
+   if (QDir::isRelativePath(dir))
    {
       dir = baseDir + dir;
    }
    if (!dir.endsWith("/"))
       dir += '/';
 
-   if (!dir.startsWith("/") && !keepRelativeToCfg)
+   if (QDir::isRelativePath(dir) && !keepRelativeToCfg)
    {
       dir = KGlobal::dirs()->findResource("xdgconf-menu", dir);
-   }      
-   
+   }
+
    dir = KGlobal::dirs()->realPath(dir);
 
    return dir;
@@ -441,7 +460,18 @@ static void tagBaseDir(QDomDocument &doc, const QString &tag, const QString &dir
    }
 }
 
-QDomDocument 
+static void tagBasePath(QDomDocument &doc, const QString &tag, const QString &path)
+{
+   QDomNodeList mergeFileList = doc.elementsByTagName(tag);
+   for(int i = 0; i < (int)mergeFileList.count(); i++)
+   {
+      QDomAttr attr = doc.createAttribute("__BasePath");
+      attr.setValue(path);
+      mergeFileList.item(i).toElement().setAttributeNode(attr);
+   }
+}
+
+QDomDocument
 VFolderMenu::loadDoc()
 {
    QDomDocument doc;
@@ -466,25 +496,26 @@ VFolderMenu::loadDoc()
    file.close();
 
    tagBaseDir(doc, "MergeFile", m_docInfo.baseDir);
+   tagBasePath(doc, "MergeFile", m_docInfo.path);
    tagBaseDir(doc, "MergeDir", m_docInfo.baseDir);
    tagBaseDir(doc, "DirectoryDir", m_docInfo.baseDir);
    tagBaseDir(doc, "AppDir", m_docInfo.baseDir);
    tagBaseDir(doc, "LegacyDir", m_docInfo.baseDir);
 
    return doc;
-}                                            
+}
 
 
-void 
+void
 VFolderMenu::mergeFile(QDomElement &parent, const QDomNode &mergeHere)
 {
 kdDebug(7021) << "VFolderMenu::mergeFile: " << m_docInfo.path << endl;
    QDomDocument doc = loadDoc();
-   
+
    QDomElement docElem = doc.documentElement();
    QDomNode n = docElem.firstChild();
    QDomNode last = mergeHere;
-   while( !n.isNull() ) 
+   while( !n.isNull() )
    {
       QDomElement e = n.toElement(); // try to convert the node to an element.
       QDomNode next = n.nextSibling();
@@ -503,10 +534,10 @@ kdDebug(7021) << "VFolderMenu::mergeFile: " << m_docInfo.path << endl;
       docElem.removeChild(n);
       n = next;
    }
-}         
+}
 
 
-void 
+void
 VFolderMenu::mergeMenus(QDomElement &docElem, QString &name)
 {
    QMap<QString,QDomElement> menuNodes;
@@ -585,17 +616,22 @@ VFolderMenu::mergeMenus(QDomElement &docElem, QString &name)
              docElem2.removeChild(n2);
              n2 = n3;
            }
-           // We still have duplicated Name entries 
+           // We still have duplicated Name entries
            // but we don't care about that
-           
+
            docElem.removeChild(docElem2);
            menuNodes.remove(it);
          }
          menuNodes.insert(name, e);
       }
       else if( e.tagName() == "MergeFile") {
-         pushDocInfo(e.text(), e.attribute("__BaseDir"));
-         mergeFile(docElem, n);
+         if ((e.attribute("type") == "parent"))
+            pushDocInfoParent(e.attribute("__BasePath"), e.attribute("__BaseDir"));
+         else
+            pushDocInfo(e.text(), e.attribute("__BaseDir"));
+
+         if (!m_docInfo.path.isEmpty())
+            mergeFile(docElem, n);
          popDocInfo();
 
          QDomNode last = n;
@@ -614,7 +650,7 @@ VFolderMenu::mergeMenus(QDomElement &docElem, QString &name)
          }
 
          QStringList fileList;
-         if (dir.startsWith("/"))
+         if (!QDir::isRelativePath(dir))
          {
             // Absolute
             fileList = KGlobal::dirs()->findAllResources("xdgconf-menu", dir+"*.menu", false, false);
@@ -632,7 +668,7 @@ VFolderMenu::mergeMenus(QDomElement &docElem, QString &name)
             mergeFile(docElem, n);
             popDocInfo();
          }
-         
+
          QDomNode last = n;
          n = n.nextSibling();
          docElem.removeChild(last); // Remove the MergeDir node
@@ -656,15 +692,20 @@ VFolderMenu::mergeMenus(QDomElement &docElem, QString &name)
    }
 }
 
-void 
+void
 VFolderMenu::pushDocInfo(const QString &fileName, const QString &baseDir)
 {
    m_docInfoStack.push(m_docInfo);
    if (!baseDir.isEmpty())
-      m_docInfo.baseDir = baseDir;
+   {
+      if (!QDir::isRelativePath(baseDir))
+         m_docInfo.baseDir = KGlobal::dirs()->relativeLocation("xdgconf-menu", baseDir);
+      else
+         m_docInfo.baseDir = baseDir;
+   }
 
    QString baseName = fileName;
-   if (baseName.startsWith("/"))
+   if (!QDir::isRelativePath(baseName))
       registerFile(baseName);
    else
       baseName = m_docInfo.baseDir + baseName;
@@ -691,6 +732,30 @@ VFolderMenu::pushDocInfo(const QString &fileName, const QString &baseDir)
    }
 }
 
+void
+VFolderMenu::pushDocInfoParent(const QString &basePath, const QString &baseDir)
+{
+    m_docInfoStack.push(m_docInfo);
+
+   m_docInfo.baseDir = baseDir;
+
+   QString fileName = basePath.mid(basePath.findRev('/')+1);
+   m_docInfo.baseName = fileName.left( fileName.length() - 5 );
+   QString baseName = QDir::cleanDirPath(m_docInfo.baseDir + fileName);
+
+   QStringList result = KGlobal::dirs()->findAllResources("xdgconf-menu", baseName);
+
+   while( !result.isEmpty() && (result[0] != basePath))
+      result.remove(result.begin());
+
+   if (result.count() <= 1)
+   {
+      m_docInfo.path = QString::null; // No parent found
+      return;
+   }
+   m_docInfo.path = result[1];
+}
+
 void
 VFolderMenu::popDocInfo()
 {
@@ -700,7 +765,7 @@ VFolderMenu::popDocInfo()
 QString
 VFolderMenu::locateMenuFile(const QString &fileName)
 {
-   if (fileName.startsWith("/"))
+   if (!QDir::isRelativePath(fileName))
    {
       if (KStandardDirs::exists(fileName))
          return fileName;
@@ -719,7 +784,7 @@ VFolderMenu::locateDirectoryFile(const QString &fileName)
    if (fileName.isEmpty())
       return QString::null;
 
-   if (fileName.startsWith("/"))
+   if (!QDir::isRelativePath(fileName))
    {
       if (KStandardDirs::exists(fileName))
          return fileName;
@@ -736,7 +801,7 @@ VFolderMenu::locateDirectoryFile(const QString &fileName)
       if (KStandardDirs::exists(tmp))
          return tmp;
    }
-   
+
    return QString::null;
 }
 
@@ -746,7 +811,7 @@ VFolderMenu::initDirs()
    m_defaultDataDirs = QStringList::split(':', KGlobal::dirs()->kfsstnd_prefixes());
    QString localDir = m_defaultDataDirs.first();
    m_defaultDataDirs.remove(localDir); // Remove local dir
-   
+
    m_defaultAppDirs = KGlobal::dirs()->findDirs("xdgdata-apps", QString::null);
    m_defaultDirectoryDirs = KGlobal::dirs()->findDirs("xdgdata-dirs", QString::null);
    m_defaultLegacyDirs = KGlobal::dirs()->resourceDirs("apps");
@@ -756,7 +821,7 @@ void
 VFolderMenu::loadMenu(const QString &fileName)
 {
    m_defaultMergeDirs.clear();
-    
+
    if (!fileName.endsWith(".menu"))
       return;
 
@@ -770,34 +835,38 @@ VFolderMenu::loadMenu(const QString &fileName)
       kdWarning(7021) << "Load error (" << m_docInfo.path << ")" << endl;
       return;
    }
-    
+
    QDomElement e = m_doc.documentElement();
    QString name;
    mergeMenus(e, name);
 }
 
-void 
+void
 VFolderMenu::processCondition(QDomElement &domElem, QDict<KService> *items)
 {
    if (domElem.tagName() == "And")
    {
       QDomNode n = domElem.firstChild();
-      if (!n.isNull())
+      // Look for the first child element
+      while (!n.isNull()) // loop in case of comments
       {
-         QDomElement e = n.toElement(); 
-         processCondition(e, items);
+         QDomElement e = n.toElement();
          n = n.nextSibling();
+         if ( !e.isNull() ) {
+             processCondition(e, items);
+             break; // we only want the first one
+         }
       }
 
       QDict<KService> andItems;
       while( !n.isNull() ) {
-         QDomElement e = n.toElement(); 
+         QDomElement e = n.toElement();
          if (e.tagName() == "Not")
          {
             // Special handling for "and not"
             QDomNode n2 = e.firstChild();
             while( !n2.isNull() ) {
-               QDomElement e2 = n2.toElement(); 
+               QDomElement e2 = n2.toElement();
                andItems.clear();
                processCondition(e2, &andItems);
                excludeItems(items, &andItems);
@@ -816,19 +885,25 @@ VFolderMenu::processCondition(QDomElement &domElem, QDict<KService> *items)
    else if (domElem.tagName() == "Or")
    {
       QDomNode n = domElem.firstChild();
-      if (!n.isNull())
+      // Look for the first child element
+      while (!n.isNull()) // loop in case of comments
       {
-         QDomElement e = n.toElement(); 
-         processCondition(e, items);
+         QDomElement e = n.toElement();
          n = n.nextSibling();
+         if ( !e.isNull() ) {
+             processCondition(e, items);
+             break; // we only want the first one
+         }
       }
 
       QDict<KService> orItems;
       while( !n.isNull() ) {
-         QDomElement e = n.toElement(); 
-         orItems.clear();
-         processCondition(e, &orItems);
-         includeItems(items, &orItems);
+         QDomElement e = n.toElement();
+         if ( !e.isNull() ) {
+             orItems.clear();
+             processCondition(e, &orItems);
+             includeItems(items, &orItems);
+         }
          n = n.nextSibling();
       }
    }
@@ -840,14 +915,16 @@ VFolderMenu::processCondition(QDomElement &domElem, QDict<KService> *items)
          items->replace(s->menuId(), s);
       }
       FOR_ALL_APPLICATIONS_END
-      
+
       QDict<KService> notItems;
       QDomNode n = domElem.firstChild();
       while( !n.isNull() ) {
-         QDomElement e = n.toElement(); 
-         notItems.clear();
-         processCondition(e, &notItems);
-         excludeItems(items, &notItems);
+         QDomElement e = n.toElement();
+         if ( !e.isNull() ) {
+             notItems.clear();
+             processCondition(e, &notItems);
+             excludeItems(items, &notItems);
+         }
          n = n.nextSibling();
       }
    }
@@ -890,7 +967,7 @@ VFolderMenu::loadApplications(const QString &dir, const QString &prefix)
       return;
 
    struct dirent *ep;
-   struct stat buff;
+   KDE_struct_stat buff;
 
    QString _dot(".");
    QString _dotdot("..");
@@ -902,21 +979,21 @@ VFolderMenu::loadApplications(const QString &dir, const QString &prefix)
          continue;
 
       QString pathfn = dir + fn;
-      if ( stat( QFile::encodeName(pathfn), &buff ) != 0 ) {
+      if ( KDE_stat( QFile::encodeName(pathfn), &buff ) != 0 ) {
          continue; // Couldn't stat (e.g. no read permissions)
       }
       if ( S_ISDIR( buff.st_mode )) {
          loadApplications(pathfn + '/', prefix + fn + '-');
          continue;
       }
-      
+
       if ( S_ISREG( buff.st_mode))
       {
          if (!fn.endsWith(".desktop"))
             continue;
-            
+
          KService *service = 0;
-         emit newService(pathfn, &service); 
+         emit newService(pathfn, &service);
          if (service)
             addApplication(prefix+fn, service);
       }
@@ -929,12 +1006,13 @@ VFolderMenu::processKDELegacyDirs()
 {
 kdDebug(7021) << "processKDELegacyDirs()" << endl;
 
+   QDict<KService> items;
    QString prefix = "kde-";
 
    QStringList relFiles;
    QRegExp files("\\.(desktop|kdelnk)$");
    QRegExp dirs("\\.directory$");
-   
+
    (void) KGlobal::dirs()->findAllResources( "apps",
                                              QString::null,
                                              true, // Recursive!
@@ -948,16 +1026,16 @@ kdDebug(7021) << "processKDELegacyDirs()" << endl;
          QString name = *it;
          if (!name.endsWith("/.directory"))
             continue; // Probably ".directory", skip it.
-            
+
          name = name.left(name.length()-11);
-      
+
          SubMenu *newMenu = new SubMenu;
          newMenu->directoryFile = locate("apps", *it);
 
          insertSubMenu(m_currentMenu, name, newMenu);
          continue;
       }
-      
+
       if (files.search(*it) != -1)
       {
          QString name = *it;
@@ -976,14 +1054,14 @@ kdDebug(7021) << "processKDELegacyDirs()" << endl;
 
             // TODO: add Legacy category
             addApplication(id, service);
-
+            items.replace(service->menuId(), service);
             if (service->categories().isEmpty())
-            {
                insertService(m_currentMenu, name, service);
-            }
+
          }
       }
    }
+   markUsedApplications(&items);
    m_legacyLoaded = true;
 }
 
@@ -992,13 +1070,14 @@ VFolderMenu::processLegacyDir(const QString &dir, const QString &relDir, const Q
 {
 kdDebug(7021) << "processLegacyDir(" << dir << ", " << relDir << ", " << prefix << ")" << endl;
 
+   QDict<KService> items;
    // We look for a set of files.
    DIR *dp = opendir( QFile::encodeName(dir));
    if (!dp)
       return;
 
    struct dirent *ep;
-   struct stat buff;
+   KDE_struct_stat buff;
 
    QString _dot(".");
    QString _dotdot("..");
@@ -1010,7 +1089,7 @@ kdDebug(7021) << "processLegacyDir(" << dir << ", " << relDir << ", " << prefix
          continue;
 
       QString pathfn = dir + fn;
-      if ( stat( QFile::encodeName(pathfn), &buff ) != 0 ) {
+      if ( KDE_stat( QFile::encodeName(pathfn), &buff ) != 0 ) {
          continue; // Couldn't stat (e.g. no read permissions)
       }
       if ( S_ISDIR( buff.st_mode )) {
@@ -1019,19 +1098,19 @@ kdDebug(7021) << "processLegacyDir(" << dir << ", " << relDir << ", " << prefix
          m_currentMenu = new SubMenu;
          m_currentMenu->name = fn;
          m_currentMenu->directoryFile = dir + fn + "/.directory";
-      
+
          parentMenu->subMenus.append(m_currentMenu);
-         
+
          processLegacyDir(pathfn + '/', relDir+fn+'/', prefix);
          m_currentMenu = parentMenu;
          continue;
       }
-      
+
       if ( S_ISREG( buff.st_mode))
       {
          if (!fn.endsWith(".desktop"))
             continue;
-            
+
          KService *service = 0;
          emit newService(pathfn, &service);
          if (service)
@@ -1040,18 +1119,20 @@ kdDebug(7021) << "processLegacyDir(" << dir << ", " << relDir << ", " << prefix
 
             // TODO: Add legacy category
             addApplication(id, service);
-            
+            items.replace(service->menuId(), service);
+
             if (service->categories().isEmpty())
                m_currentMenu->items.replace(id, service);
          }
       }
     }
     closedir( dp );
+    markUsedApplications(&items);
 }
 
 
 
-void 
+void
 VFolderMenu::processMenu(QDomElement &docElem, int pass)
 {
    SubMenu *parentMenu = m_currentMenu;
@@ -1171,7 +1252,7 @@ VFolderMenu::processMenu(QDomElement &docElem, int pass)
          m_currentMenu = m_rootMenu;
       }
    }
-   
+
    // Process AppDir and LegacyDir
    if (pass == 0)
    {
@@ -1183,7 +1264,7 @@ VFolderMenu::processMenu(QDomElement &docElem, int pass)
          {
             createAppsInfo();
             QString dir = absoluteDir(e.text(), e.attribute("__BaseDir"));
-            
+
             registerDirectory(dir);
 
             loadApplications(dir, QString::null);
@@ -1198,10 +1279,10 @@ kdDebug(7021) << "Processing KDE Legacy dirs for <KDE>" << endl;
                m_currentMenu = new SubMenu;
 
                processKDELegacyDirs();
-               
+
                m_legacyNodes.replace("<KDE>", m_currentMenu);
                m_currentMenu = oldMenu;
-               
+
                kdeLegacyDirsDone = true;
             }
          }
@@ -1209,7 +1290,7 @@ kdDebug(7021) << "Processing KDE Legacy dirs for <KDE>" << endl;
          {
             createAppsInfo();
             QString dir = absoluteDir(e.text(), e.attribute("__BaseDir"));
-            
+
             QString prefix = e.attributes().namedItem("prefix").toAttr().value();
 
             if (m_defaultLegacyDirs.contains(dir))
@@ -1246,7 +1327,7 @@ kdDebug(7021) << "Processing KDE Legacy dirs for " << dir << endl;
    }
 
    loadAppsInfo(); // Update the scope wrt the list of applications
-   
+
    if (((pass == 1) && !onlyUnallocated) || ((pass == 2) && onlyUnallocated))
    {
       n = docElem.firstChild();
@@ -1256,14 +1337,21 @@ kdDebug(7021) << "Processing KDE Legacy dirs for " << dir << endl;
          if (e.tagName() == "Include")
          {
             QDict<KService> items;
-            
+
             QDomNode n2 = e.firstChild();
             while( !n2.isNull() ) {
-               QDomElement e2 = n2.toElement(); 
+               QDomElement e2 = n2.toElement();
                items.clear();
                processCondition(e2, &items);
+               if (m_track)
+                  track(m_trackId, m_currentMenu->name, &(m_currentMenu->items), &(m_currentMenu->excludeItems), &items, "Before <Include>");
                includeItems(&(m_currentMenu->items), &items);
                excludeItems(&(m_currentMenu->excludeItems), &items);
+               markUsedApplications(&items);
+
+               if (m_track)
+                  track(m_trackId, m_currentMenu->name, &(m_currentMenu->items), &(m_currentMenu->excludeItems), &items, "After <Include>");
+
                n2 = n2.nextSibling();
             }
          }
@@ -1274,11 +1362,15 @@ kdDebug(7021) << "Processing KDE Legacy dirs for " << dir << endl;
 
             QDomNode n2 = e.firstChild();
             while( !n2.isNull() ) {
-               QDomElement e2 = n2.toElement(); 
+               QDomElement e2 = n2.toElement();
                items.clear();
                processCondition(e2, &items);
+               if (m_track)
+                  track(m_trackId, m_currentMenu->name, &(m_currentMenu->items), &(m_currentMenu->excludeItems), &items, "Before <Exclude>");
                excludeItems(&(m_currentMenu->items), &items);
                includeItems(&(m_currentMenu->excludeItems), &items);
+               if (m_track)
+                  track(m_trackId, m_currentMenu->name, &(m_currentMenu->items), &(m_currentMenu->excludeItems), &items, "After <Exclude>");
                n2 = n2.nextSibling();
             }
          }
@@ -1295,7 +1387,7 @@ kdDebug(7021) << "Processing KDE Legacy dirs for " << dir << endl;
          processMenu(e, pass);
       }
 // We insert legacy dir in pass 0, this way the order in the .menu-file determines
-// which .directory file gets used, but the menu-entries of legacy-menus will always 
+// which .directory file gets used, but the menu-entries of legacy-menus will always
 // have the lowest priority.
 //      else if (((pass == 1) && !onlyUnallocated) || ((pass == 2) && onlyUnallocated))
       else if (pass == 0)
@@ -1350,12 +1442,12 @@ kdDebug(7021) << "Processing KDE Legacy dirs for " << dir << endl;
               if (menu)
               {
                 insertSubMenu(m_currentMenu, dest, menu, true); // Destination has priority
-              } 
+              }
             }
          }
          n = n.nextSibling();
       }
-   
+
    }
 
    unloadAppsInfo(); // Update the scope wrt the list of applications
@@ -1366,16 +1458,83 @@ kdDebug(7021) << "Processing KDE Legacy dirs for " << dir << endl;
    m_currentMenu = parentMenu;
 }
 
+
+
+static QString parseAttribute( const QDomElement &e)
+{
+    QString option;
+    if ( e.hasAttribute( "show_empty" ) )
+    {
+        QString str = e.attribute( "show_empty" );
+        if ( str=="true" )
+            option= "ME ";
+        else if ( str=="false" )
+            option= "NME ";
+        else
+            kdDebug()<<" Error in parsing show_empty attribute :"<<str<<endl;
+    }
+    if ( e.hasAttribute( "inline" ) )
+    {
+        QString str = e.attribute( "inline" );
+        if (  str=="true" )
+            option+="I ";
+        else if ( str=="false" )
+            option+="NI ";
+        else
+            kdDebug()<<" Error in parsing inlibe attribute :"<<str<<endl;
+    }
+    if ( e.hasAttribute( "inline_limit" ) )
+    {
+        bool ok;
+        int value = e.attribute( "inline_limit" ).toInt(&ok);
+        if ( ok )
+            option+=QString( "IL[%1] " ).arg( value );
+    }
+    if ( e.hasAttribute( "inline_header" ) )
+    {
+        QString str = e.attribute( "inline_header" );
+        if ( str=="true")
+            option+="IH ";
+        else if ( str == "false" )
+            option+="NIH ";
+        else
+            kdDebug()<<" Error in parsing of inline_header attribute :"<<str<<endl;
+
+    }
+    if ( e.hasAttribute( "inline_alias" ) && e.attribute( "inline_alias" )=="true")
+    {
+        QString str = e.attribute( "inline_alias" );
+        if ( str=="true" )
+            option+="IA";
+        else if ( str=="false" )
+            option+="NIA";
+        else
+            kdDebug()<<" Error in parsing inline_alias attribute :"<<str<<endl;
+    }
+    if( !option.isEmpty())
+    {
+        option = option.prepend(":O");
+    }
+    return option;
+
+}
+
 static QStringList parseLayoutNode(const QDomElement &docElem)
 {
    QStringList layout;
-   
+
+   QString optionDefaultLayout;
+   if( docElem.tagName()=="DefaultLayout")
+       optionDefaultLayout =  parseAttribute( docElem);
+   if ( !optionDefaultLayout.isEmpty() )
+       layout.append( optionDefaultLayout );
+
    QDomNode n = docElem.firstChild();
    while( !n.isNull() ) {
       QDomElement e = n.toElement(); // try to convert the node to an element.
       if (e.tagName() == "Separator")
       {
-         layout.append(":S"); 
+         layout.append(":S");
       }
       else if (e.tagName() == "Filename")
       {
@@ -1384,6 +1543,9 @@ static QStringList parseLayoutNode(const QDomElement &docElem)
       else if (e.tagName() == "Menuname")
       {
          layout.append("/"+e.text());
+         QString option = parseAttribute( e );
+         if( !option.isEmpty())
+             layout.append( option );
       }
       else if (e.tagName() == "Merge")
       {
@@ -1395,20 +1557,20 @@ static QStringList parseLayoutNode(const QDomElement &docElem)
          else if (type == "all")
             layout.append(":A");
       }
-      
+
       n = n.nextSibling();
    }
    return layout;
 }
 
-void 
+void
 VFolderMenu::layoutMenu(VFolderMenu::SubMenu *menu, QStringList defaultLayout)
 {
    if (!menu->defaultLayoutNode.isNull())
    {
       defaultLayout = parseLayoutNode(menu->defaultLayoutNode);
    }
-   
+
    if (menu->layoutNode.isNull())
    {
      menu->layoutList = defaultLayout;
@@ -1419,7 +1581,7 @@ VFolderMenu::layoutMenu(VFolderMenu::SubMenu *menu, QStringList defaultLayout)
      if (menu->layoutList.isEmpty())
         menu->layoutList = defaultLayout;
    }
-   
+
    for(VFolderMenu::SubMenu *subMenu = menu->subMenus.first(); subMenu; subMenu = menu->subMenus.next())
    {
       layoutMenu(subMenu, defaultLayout);
@@ -1427,13 +1589,9 @@ VFolderMenu::layoutMenu(VFolderMenu::SubMenu *menu, QStringList defaultLayout)
 }
 
 void
-VFolderMenu::markUsedApplications(VFolderMenu::SubMenu *menu)
+VFolderMenu::markUsedApplications(QDict<KService> *items)
 {
-   for(VFolderMenu::SubMenu *subMenu = menu->subMenus.first(); subMenu; subMenu = menu->subMenus.next())
-   {
-      markUsedApplications(subMenu);
-   }
-   for(QDictIterator<KService> it(menu->items); it.current(); ++it)
+   for(QDictIterator<KService> it(*items); it.current(); ++it)
    {
       m_usedAppsDict.replace(it.current()->menuId(), it.current());
    }
@@ -1461,7 +1619,7 @@ VFolderMenu::parseMenu(const QString &file, bool forceLegacyLoad)
    QDomElement docElem = m_doc.documentElement();
 
    for (int pass = 0; pass <= 2; pass++)
-   {  
+   {
       processMenu(docElem, pass);
 
       if (pass == 0)
@@ -1470,8 +1628,6 @@ VFolderMenu::parseMenu(const QString &file, bool forceLegacyLoad)
       }
       if (pass == 1)
       {
-         markUsedApplications(m_rootMenu);
-
          buildApplicationIndex(true);
       }
       if (pass == 2)
@@ -1482,14 +1638,21 @@ VFolderMenu::parseMenu(const QString &file, bool forceLegacyLoad)
          layoutMenu(m_rootMenu, defaultLayout);
       }
    }
-   
+
    if (!m_legacyLoaded && forceLegacyLoad)
    {
       m_forcedLegacyLoad = true;
       processKDELegacyDirs();
    }
 
-   return m_rootMenu;   
+   return m_rootMenu;
+}
+
+void
+VFolderMenu::setTrackId(const QString &id)
+{
+   m_track = !id.isEmpty();
+   m_trackId = id;
 }
 
 #include "vfolder_menu.moc"
