@@ -1,6 +1,6 @@
 /* vi: ts=8 sts=4 sw=4
  *
- * $Id: process.cpp 268078 2003-11-18 23:37:01Z adridg $
+ * $Id: process.cpp 592751 2006-10-05 16:22:44Z coolo $
  *
  * This file is part of the KDE project, module kdesu.
  * Copyright (C) 1999,2000 Geert Jansen <jansen@kde.org>
@@ -48,6 +48,7 @@
 #include <qcstring.h>
 #include <qfile.h>
 
+#include <kconfig.h>
 #include <kdebug.h>
 #include <kstandarddirs.h>
 
@@ -73,7 +74,15 @@ int PtyProcess::waitMS(int fd,int ms)
 */
 bool PtyProcess::checkPid(pid_t pid)
 {
+	KConfig* config = KGlobal::config();
+	config->setGroup("super-user-command");
+	QString superUserCommand = config->readEntry("super-user-command", DEFAULT_SUPER_USER_COMMAND);
+	//sudo does not accept signals from user so we except it
+	if (superUserCommand == "sudo") {
+		return true;
+	} else {
 	return kill(pid,0) == 0;
+	}
 }
 
 /*
@@ -296,6 +305,14 @@ int PtyProcess::exec(const QCString &command, const QCStringList &args)
         putenv((*it).data());
     }
     unsetenv("KDE_FULL_SESSION");
+    
+    // set temporarily LC_ALL to C, for su (to be able to parse "Password:")
+    const char* old_lc_all = getenv( "LC_ALL" );
+    if( old_lc_all != NULL )
+        setenv( "KDESU_LC_ALL", old_lc_all, 1 );
+    else
+        unsetenv( "KDESU_LC_ALL" );
+    setenv("LC_ALL", "C", 1);
 
     // From now on, terminal output goes through the tty.
 
@@ -316,7 +333,7 @@ int PtyProcess::exec(const QCString &command, const QCStringList &args)
     const char **argp = (const char **)malloc((args.count()+2)*sizeof(char *));
     int i = 0;
     argp[i++] = path;
-    for (QCStringList::ConstIterator it=args.begin(); it!=args.end(); it++)
+    for (QCStringList::ConstIterator it=args.begin(); it!=args.end(); ++it)
         argp[i++] = *it;
 
     argp[i] = 0L;
