@@ -13,16 +13,15 @@
 
    You should have received a copy of the GNU Library General Public License
    along with this library; see the file COPYING.LIB.  If not, write to
-   the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
-   Boston, MA 02111-1307, USA.
+   the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
+   Boston, MA 02110-1301, USA.
 */
 #include <kedittoolbar.h>
 
 #include <qdom.h>
-
 #include <qlayout.h>
-#include <kaction.h>
-
+#include <qdir.h>
+#include <qfile.h>
 #include <qheader.h>
 #include <qcombobox.h>
 #include <qdragobject.h>
@@ -30,25 +29,28 @@
 #include <qlabel.h>
 #include <qvaluelist.h>
 #include <qapplication.h>
+#include <qtextstream.h>
 
+#include <kaction.h>
 #include <kstandarddirs.h>
 #include <klocale.h>
 #include <kicontheme.h>
 #include <kiconloader.h>
 #include <kinstance.h>
+#include <kmessagebox.h>
 #include <kxmlguifactory.h>
 #include <kseparator.h>
 #include <kconfig.h>
 #include <klistview.h>
-
-#include <qtextstream.h>
-#include <qfile.h>
 #include <kdebug.h>
-#include "kpushbutton.h"
+#include <kpushbutton.h>
 #include <kprocio.h>
 
-#define LINESEPARATORSTRING I18N_NOOP("--- line separator ---")
-#define SEPARATORSTRING I18N_NOOP("--- separator ---")
+static const char * const lineseparatorstring = I18N_NOOP("--- line separator ---");
+static const char * const separatorstring = I18N_NOOP("--- separator ---");
+
+#define LINESEPARATORSTRING i18n(lineseparatorstring)
+#define SEPARATORSTRING i18n(separatorstring)
 
 static void dump_xml(const QDomDocument& doc)
 {
@@ -105,9 +107,9 @@ public:
   virtual QString key(int column, bool) const
   {
     QString s = text( column );
-    if ( s == i18n(LINESEPARATORSTRING) )
+    if ( s == LINESEPARATORSTRING )
       return "0";
-    if ( s == i18n(SEPARATORSTRING) )
+    if ( s == SEPARATORSTRING )
       return "1";
     return "2" + s;
   }
@@ -184,7 +186,7 @@ protected:
   virtual QDragObject *dragObject()
   {
     ToolbarItem *item = dynamic_cast<ToolbarItem*>(selectedItem());
-    if ( item != 0 ) {
+    if ( item ) {
       ToolbarItemDrag *obj = new ToolbarItemDrag(item,
                                  this, "ToolbarAction drag item");
       const QPixmap *pm = item->pixmap(0);
@@ -239,7 +241,7 @@ public:
     QString xml_file = xmlFile(_xml_file);
     //kdDebug() << "loadXMLFile xml_file=" << xml_file << endl;
 
-    if ( xml_file[0] == '/' )
+    if ( !QDir::isRelativePath(xml_file) )
       raw_xml = KXMLGUIFactory::readConfigFile(xml_file);
     else
       raw_xml = KXMLGUIFactory::readConfigFile(xml_file, m_instance);
@@ -339,7 +341,7 @@ public:
   KActionCollection* m_collection;
   KInstance         *m_instance;
 
-  XmlData     m_currentXmlData;
+  XmlData*     m_currentXmlData;
   QDomElement m_currentToolbarElem;
 
   QString            m_xmlFile;
@@ -363,46 +365,61 @@ public:
 class KEditToolbarPrivate {
 public:
     bool m_accept;
+
+    // Save parameters for recreating widget after resetting toolbar
+    bool m_global;
+    KActionCollection* m_collection;
+    QString m_file;
+    KXMLGUIFactory* m_factory;
 };
 
 const char *KEditToolbar::s_defaultToolbar = 0L;
 
 KEditToolbar::KEditToolbar(KActionCollection *collection, const QString& file,
                            bool global, QWidget* parent, const char* name)
-  : KDialogBase(Swallow, i18n("Configure Toolbars"), Ok|Apply|Cancel, Ok, parent, name),
+  : KDialogBase(Swallow, i18n("Configure Toolbars"), Default|Ok|Apply|Cancel, Ok, parent, name),
     m_widget(new KEditToolbarWidget(QString::fromLatin1(s_defaultToolbar), collection, file, global, this))
 {
     init();
+    d->m_global = global;
+    d->m_collection = collection;
+    d->m_file = file;
 }
 
 KEditToolbar::KEditToolbar(const QString& defaultToolbar, KActionCollection *collection,
                            const QString& file, bool global,
                            QWidget* parent, const char* name)
-  : KDialogBase(Swallow, i18n("Configure Toolbars"), Ok|Apply|Cancel, Ok, parent, name),
+  : KDialogBase(Swallow, i18n("Configure Toolbars"), Default|Ok|Apply|Cancel, Ok, parent, name),
     m_widget(new KEditToolbarWidget(defaultToolbar, collection, file, global, this))
 {
     init();
+    d->m_global = global;
+    d->m_collection = collection;
+    d->m_file = file;
 }
 
 KEditToolbar::KEditToolbar(KXMLGUIFactory* factory, QWidget* parent, const char* name)
-    : KDialogBase(Swallow, i18n("Configure Toolbars"), Ok|Apply|Cancel, Ok, parent, name),
+    : KDialogBase(Swallow, i18n("Configure Toolbars"), Default|Ok|Apply|Cancel, Ok, parent, name),
       m_widget(new KEditToolbarWidget(QString::fromLatin1(s_defaultToolbar), factory, this))
 {
     init();
+    d->m_factory = factory;
 }
 
 KEditToolbar::KEditToolbar(const QString& defaultToolbar,KXMLGUIFactory* factory,
                            QWidget* parent, const char* name)
-    : KDialogBase(Swallow, i18n("Configure Toolbars"), Ok|Apply|Cancel, Ok, parent, name),
+    : KDialogBase(Swallow, i18n("Configure Toolbars"), Default|Ok|Apply|Cancel, Ok, parent, name),
       m_widget(new KEditToolbarWidget(defaultToolbar, factory, this))
 {
     init();
+    d->m_factory = factory;
 }
 
 void KEditToolbar::init()
 {
     d = new KEditToolbarPrivate();
     d->m_accept = false;
+    d->m_factory = 0;
 
     setMainWidget(m_widget);
 
@@ -425,6 +442,71 @@ void KEditToolbar::acceptOK(bool b)
     d->m_accept = b;
 }
 
+void KEditToolbar::slotDefault()
+{
+    if ( KMessageBox::warningContinueCancel(this, i18n("Do you really want to reset all toolbars of this application to their default? The changes will be applied immediately."), i18n("Reset Toolbars"),i18n("Reset"))!=KMessageBox::Continue )
+        return;
+
+    delete m_widget;
+    d->m_accept = false;
+
+    if ( d->m_factory )
+    {
+        const QString localPrefix = locateLocal("data", "");
+        QPtrList<KXMLGUIClient> clients(d->m_factory->clients());
+        QPtrListIterator<KXMLGUIClient> it( clients );
+
+        for( ; it.current(); ++it)
+        {
+            KXMLGUIClient *client = it.current();
+            QString file = client->xmlFile();
+
+            if (file.isNull())
+                continue;
+
+            if (QDir::isRelativePath(file))
+            {
+                const KInstance *instance = client->instance() ? client->instance() : KGlobal::instance();
+                file = locateLocal("data", QString::fromLatin1( instance->instanceName() + '/' ) + file);
+            }
+            else
+            {
+                if (!file.startsWith(localPrefix))
+                    continue;
+            }
+
+            if ( QFile::exists( file ) )
+                if ( !QFile::remove( file ) )
+                    kdWarning() << "Could not delete " << file << endl;
+        }
+
+        m_widget = new KEditToolbarWidget(QString::null, d->m_factory, this);
+        m_widget->rebuildKXMLGUIClients();
+    }
+    else
+    {
+        int slash = d->m_file.findRev('/')+1;
+        if (slash)
+            d->m_file = d->m_file.mid(slash);
+        QString xml_file = locateLocal("data", QString::fromLatin1( KGlobal::instance()->instanceName() + '/' ) + d->m_file);
+
+        if ( QFile::exists( xml_file ) )
+            if ( !QFile::remove( xml_file ) )
+                kdWarning() << "Could not delete " << xml_file << endl;
+
+        m_widget = new KEditToolbarWidget(QString::null, d->m_collection, d->m_file, d->m_global, this);
+    }
+
+    setMainWidget(m_widget);
+    m_widget->show();
+
+    connect(m_widget, SIGNAL(enableOk(bool)), SLOT(acceptOK(bool)));
+    connect(m_widget, SIGNAL(enableOk(bool)), SLOT(enableButtonApply(bool)));
+
+    enableButtonApply(false);
+    emit newToolbarConfig();
+}
+
 void KEditToolbar::slotOk()
 {
   if (!d->m_accept) {
@@ -602,7 +684,7 @@ bool KEditToolbarWidget::save()
   for ( ; it != d->m_xmlFiles.end(); ++it)
   {
     // let's not save non-modified files
-    if ( (*it).m_isModified == false )
+    if ( !((*it).m_isModified) )
       continue;
 
     // let's also skip (non-existent) merged files
@@ -611,6 +693,7 @@ bool KEditToolbarWidget::save()
 
     dump_xml((*it).m_document);
 
+    kdDebug(240) << "Saving " << (*it).m_xmlFile << endl;
     // if we got this far, we might as well just save it
     KXMLGUIFactory::saveConfigFile((*it).m_document, (*it).m_xmlFile);
   }
@@ -618,6 +701,16 @@ bool KEditToolbarWidget::save()
   if ( !factory() )
     return true;
 
+  rebuildKXMLGUIClients();
+
+  return true;
+}
+
+void KEditToolbarWidget::rebuildKXMLGUIClients()
+{
+  if ( !factory() )
+    return;
+
   QPtrList<KXMLGUIClient> clients(factory()->clients());
   //kdDebug(240) << "factory: " << clients.count() << " clients" << endl;
 
@@ -660,8 +753,6 @@ bool KEditToolbarWidget::save()
   cit.toFirst();
   for( ; cit.current(); ++cit)
     factory()->addClient( cit.current() );
-
-  return true;
 }
 
 void KEditToolbarWidget::setupLayout()
@@ -869,12 +960,13 @@ void KEditToolbarWidget::loadActionList(QDomElement& elem)
   m_downAction->setEnabled(false);
 
   // We'll use this action collection
-  KActionCollection* actionCollection = d->m_currentXmlData.m_actionCollection;
+  KActionCollection* actionCollection = d->m_currentXmlData->m_actionCollection;
 
   // store the names of our active actions
   QMap<QString, bool> active_list;
 
   // see if our current action is in this toolbar
+  KIconLoader *loader = KGlobal::instance()->iconLoader();
   QDomNode n = elem.lastChild();
   for( ; !n.isNull(); n = n.previousSibling() )
   {
@@ -885,9 +977,9 @@ void KEditToolbarWidget::loadActionList(QDomElement& elem)
       ToolbarItem *act = new ToolbarItem(m_activeList, tagSeparator, sep_name.arg(sep_num++), QString::null);
       bool isLineSep = ( it.attribute(attrLineSeparator, "true").lower() == QString::fromLatin1("true") );
       if(isLineSep)
-        act->setText(1, i18n(LINESEPARATORSTRING));
+        act->setText(1, LINESEPARATORSTRING);
       else
-        act->setText(1, i18n(SEPARATORSTRING));
+        act->setText(1, SEPARATORSTRING);
       it.setAttribute( attrName, act->internalName() );
       continue;
     }
@@ -926,7 +1018,7 @@ void KEditToolbarWidget::loadActionList(QDomElement& elem)
         act->setText(1, action->plainText());
         if (action->hasIcon())
           if (!action->icon().isEmpty())
-            act->setPixmap(0, BarIcon(action->icon(), 16));
+            act->setPixmap(0, loader->loadIcon(action->icon(), KIcon::Toolbar, 16, KIcon::DefaultState, 0, true) );
           else // Has iconset
             act->setPixmap(0, action->iconSet(KIcon::Toolbar).pixmap());
 
@@ -949,16 +1041,16 @@ void KEditToolbarWidget::loadActionList(QDomElement& elem)
     act->setText(1, action->plainText());
     if (action->hasIcon())
       if (!action->icon().isEmpty())
-        act->setPixmap(0, BarIcon(action->icon(), 16));
+        act->setPixmap(0, loader->loadIcon(action->icon(), KIcon::Toolbar, 16, KIcon::DefaultState, 0, true) );
       else // Has iconset
         act->setPixmap(0, action->iconSet(KIcon::Toolbar).pixmap());
   }
 
   // finally, add default separators to the inactive list
   ToolbarItem *act = new ToolbarItem(m_inactiveList, tagSeparator, sep_name.arg(sep_num++), QString::null);
-  act->setText(1, i18n(LINESEPARATORSTRING));
+  act->setText(1, LINESEPARATORSTRING);
   act = new ToolbarItem(m_inactiveList, tagSeparator, sep_name.arg(sep_num++), QString::null);
-  act->setText(1, i18n(SEPARATORSTRING));
+  act->setText(1, SEPARATORSTRING);
 }
 
 KActionCollection *KEditToolbarWidget::actionCollection() const
@@ -981,7 +1073,7 @@ void KEditToolbarWidget::slotToolbarSelected(const QString& _text)
       if ( name == _text )
       {
         // save our current settings
-        d->m_currentXmlData     = (*xit);
+        d->m_currentXmlData     = & (*xit);
         d->m_currentToolbarElem = (*it);
 
         // load in our values
@@ -1014,10 +1106,10 @@ void KEditToolbarWidget::slotInactiveSelected(QListViewItem *item)
 void KEditToolbarWidget::slotActiveSelected(QListViewItem *item)
 {
   ToolbarItem* toolitem = static_cast<ToolbarItem *>(item);
-  m_removeAction->setEnabled( item != 0 );
+  m_removeAction->setEnabled( item );
 
   static const QString &tagAction = KGlobal::staticQString( "Action" );
-  d->m_changeIcon->setEnabled( item != 0 &&
+  d->m_changeIcon->setEnabled( item &&
                                d->m_hasKDialog &&
                                toolitem->internalTag() == tagAction );
 
@@ -1053,11 +1145,11 @@ void KEditToolbarWidget::slotDropped(KListView *list, QDropEvent *e, QListViewIt
 
   if (list == m_activeList) {
     if (e->source() == m_activeList) {
-      // has been dragged within the active list (moved) -> remove the old item.
-      removeActive(item);
+      // has been dragged within the active list (moved).
+      moveActive(item, after);
     }
-
-    insertActive(item, after, true);
+    else
+      insertActive(item, after, true);
   } else if (list == m_inactiveList) {
     // has been dragged to the inactive list -> remove from the active list.
     removeActive(item);
@@ -1079,6 +1171,8 @@ void KEditToolbarWidget::slotInsertButton()
   // we're modified, so let this change
   emit enableOk(true);
 
+  // TODO: #### this causes #97572.
+  // It would be better to just "delete item; loadActions( ... , ActiveListOnly );" or something.
   slotToolbarSelected( m_toolbarCombo->currentText() );
 }
 
@@ -1105,9 +1199,9 @@ void KEditToolbarWidget::insertActive(ToolbarItem *item, QListViewItem *before,
 
   QDomElement new_item;
   // let's handle the separator specially
-  if (item->text(1) == i18n(LINESEPARATORSTRING)) {
+  if (item->text(1) == LINESEPARATORSTRING) {
     new_item = domDocument().createElement(tagSeparator);
-  } else if (item->text(1) == i18n(SEPARATORSTRING)) {
+  } else if (item->text(1) == SEPARATORSTRING) {
     new_item = domDocument().createElement(tagSeparator);
     new_item.setAttribute(attrLineSeparator, "false");
   } else
@@ -1172,49 +1266,51 @@ void KEditToolbarWidget::slotUpButton()
   if (!item->itemAbove())
     return;
 
-  static const QString &attrNoMerge = KGlobal::staticQString( "noMerge" );
-
   // we're modified, so let this change
   emit enableOk(true);
 
-  // now iterate through to find where we are
-  QDomElement elem = d->findElementForToolbarItem( item );
-  if ( !elem.isNull() )
-  {
-    // cool, i found me.  now clone myself
-    ToolbarItem *clone = new ToolbarItem(m_activeList,
-                                         item->itemAbove()->itemAbove(),
-                                         item->internalTag(),
-                                         item->internalName(),
-                                         item->statusText());
-    clone->setText(1, item->text(1));
+  moveActive( item, item->itemAbove()->itemAbove() );
+  delete item;
+}
 
-    // only set new pixmap if exists
-    if( item->pixmap(0) )
-      clone->setPixmap(0, *item->pixmap(0));
+void KEditToolbarWidget::moveActive( ToolbarItem* item, QListViewItem* before )
+{
+  QDomElement e = d->findElementForToolbarItem( item );
 
-    // remove the old me
-    m_activeList->takeItem(item);
-    delete item;
+  if ( e.isNull() )
+    return;
 
-    // select my clone
-    m_activeList->setSelected(clone, true);
+  // cool, i found me.  now clone myself
+  ToolbarItem *clone = new ToolbarItem(m_activeList,
+                                       before,
+                                       item->internalTag(),
+                                       item->internalName(),
+                                       item->statusText());
 
-    // make clone visible
-    m_activeList->ensureItemVisible(clone);
+  clone->setText(1, item->text(1));
 
-    // and do the real move in the DOM
-    QDomNode prev = elem.previousSibling();
-    while ( prev.toElement().tagName() == QString( "WeakSeparator" ) )
-        prev = prev.previousSibling();
-    d->m_currentToolbarElem.insertBefore(elem, prev);
+  // only set new pixmap if exists
+  if( item->pixmap(0) )
+    clone->setPixmap(0, *item->pixmap(0));
 
-    // and set this container as a noMerge
-    d->m_currentToolbarElem.setAttribute( attrNoMerge, "1");
+  // select my clone
+  m_activeList->setSelected(clone, true);
 
-    // update the local doc
-    updateLocal(d->m_currentToolbarElem);
-  }
+  // make clone visible
+  m_activeList->ensureItemVisible(clone);
+
+  // and do the real move in the DOM
+  if ( !before )
+    d->m_currentToolbarElem.insertBefore(e, d->m_currentToolbarElem.firstChild() );
+  else
+    d->m_currentToolbarElem.insertAfter(e, d->findElementForToolbarItem( (ToolbarItem*)before ));
+
+  // and set this container as a noMerge
+  static const QString &attrNoMerge = KGlobal::staticQString( "noMerge" );
+  d->m_currentToolbarElem.setAttribute( attrNoMerge, "1");
+
+  // update the local doc
+  updateLocal(d->m_currentToolbarElem);
 }
 
 void KEditToolbarWidget::slotDownButton()
@@ -1225,49 +1321,11 @@ void KEditToolbarWidget::slotDownButton()
   if (!item->itemBelow())
     return;
 
-  static const QString &attrNoMerge = KGlobal::staticQString( "noMerge" );
-
   // we're modified, so let this change
   emit enableOk(true);
 
-  // now iterate through to find where we are
-  QDomElement elem = d->findElementForToolbarItem( item );
-  if ( !elem.isNull() )
-  {
-    // cool, i found me.  now clone myself
-    ToolbarItem *clone = new ToolbarItem(m_activeList,
-                                         item->itemBelow(),
-                                         item->internalTag(),
-                                         item->internalName(),
-                                         item->statusText());
-    clone->setText(1, item->text(1));
-
-    // only set new pixmap if exists
-    if( item->pixmap(0) )
-      clone->setPixmap(0, *item->pixmap(0));
-
-    // remove the old me
-    m_activeList->takeItem(item);
-    delete item;
-
-    // select my clone
-    m_activeList->setSelected(clone, true);
-
-    // make clone visible
-    m_activeList->ensureItemVisible(clone);
-
-    // and do the real move in the DOM
-    QDomNode next = elem.nextSibling();
-    while ( next.toElement().tagName() == QString( "WeakSeparator" ) )
-      next = next.nextSibling();
-    d->m_currentToolbarElem.insertAfter(elem, next);
-
-    // and set this container as a noMerge
-    d->m_currentToolbarElem.setAttribute( attrNoMerge, "1");
-
-    // update the local doc
-    updateLocal(d->m_currentToolbarElem);
-  }
+  moveActive( item, item->itemBelow() );
+  delete item;
 }
 
 void KEditToolbarWidget::updateLocal(QDomElement& elem)
@@ -1283,7 +1341,7 @@ void KEditToolbarWidget::updateLocal(QDomElement& elem)
     if ( (*xit).m_type == XmlData::Shell ||
          (*xit).m_type == XmlData::Part )
     {
-      if ( d->m_currentXmlData.m_xmlFile == (*xit).m_xmlFile )
+      if ( d->m_currentXmlData->m_xmlFile == (*xit).m_xmlFile )
       {
         (*xit).m_isModified = true;
         return;
@@ -1317,11 +1375,17 @@ void KEditToolbarWidget::slotChangeIcon()
 {
   // We can't use KIconChooser here, since it's in libkio
   // ##### KDE4: reconsider this, e.g. move KEditToolbar to libkio
+  
+  //if the process is already running (e.g. when somebody clicked the change button twice (see #127149)) - do nothing... 
+  //otherwise m_kdialogProcess will be overwritten and set to zero in slotProcessExited()...crash!
+  if ( d->m_kdialogProcess && d->m_kdialogProcess->isRunning() )
+        return;
+  
   d->m_kdialogProcess = new KProcIO;
   QString kdialogExe = KStandardDirs::findExe(QString::fromLatin1("kdialog"));
   (*d->m_kdialogProcess) << kdialogExe;
   (*d->m_kdialogProcess) << "--embed";
-  (*d->m_kdialogProcess) << QString::number( topLevelWidget()->winId() );
+  (*d->m_kdialogProcess) << QString::number( (ulong)topLevelWidget()->winId() );
   (*d->m_kdialogProcess) << "--geticon";
   (*d->m_kdialogProcess) << "Toolbar";
   (*d->m_kdialogProcess) << "Actions";
@@ -1345,8 +1409,14 @@ void KEditToolbarWidget::slotProcessExited( KProcess* )
   m_toolbarCombo->setEnabled( true );
 
   QString icon;
+
+  if (!d->m_kdialogProcess) {
+         kdError(240) << "Something is wrong here! m_kdialogProcess is zero!" << endl;
+         return;
+  }
+
   if ( !d->m_kdialogProcess->normalExit() ||
-       d->m_kdialogProcess->exitStatus() != 0 ||
+       d->m_kdialogProcess->exitStatus() ||
        d->m_kdialogProcess->readln(icon, true) <= 0 ) {
     delete d->m_kdialogProcess;
     d->m_kdialogProcess = 0;
@@ -1357,10 +1427,12 @@ void KEditToolbarWidget::slotProcessExited( KProcess* )
   if(item){
     item->setPixmap(0, BarIcon(icon, 16));
 
-    d->m_currentXmlData.m_isModified = true;
+    Q_ASSERT( d->m_currentXmlData->m_type != XmlData::Merged );
+
+    d->m_currentXmlData->m_isModified = true;
 
     // Get hold of ActionProperties tag
-    QDomElement elem = KXMLGUIFactory::actionPropertiesElement( d->m_currentXmlData.m_document );
+    QDomElement elem = KXMLGUIFactory::actionPropertiesElement( d->m_currentXmlData->m_document );
     // Find or create an element for this action
     QDomElement act_elem = KXMLGUIFactory::findActionByName( elem, item->internalName(), true /*create*/ );
     Q_ASSERT( !act_elem.isNull() );
@@ -1369,7 +1441,7 @@ void KEditToolbarWidget::slotProcessExited( KProcess* )
     // we're modified, so let this change
     emit enableOk(true);
   }
-  
+
   delete d->m_kdialogProcess;
   d->m_kdialogProcess = 0;
 }
