@@ -22,7 +22,7 @@ class HelpProtocol : public KIO::SlaveBase
 {
 public:
 
-    HelpProtocol( const QCString &pool, const QCString &app);
+    HelpProtocol( bool ghelp, const QCString &pool, const QCString &app);
     virtual ~HelpProtocol() { }
 
     virtual void get( const KURL& url );
@@ -37,8 +37,10 @@ private:
     QString lookupFile(const QString &fname, const QString &query,
                        bool &redirect);
 
-    void notFound();
-    QString parsed;
+    void unicodeError( const QString &t );
+
+    QString mParsed;
+    bool mGhelp;
 };
 
 
