@@ -15,8 +15,8 @@
  *
  * You should have received a copy of the GNU Library General Public License
  * along with this library; see the file COPYING.LIB.  If not, write to
- * the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
- * Boston, MA 02111-1307, USA.
+ * the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
+ * Boston, MA 02110-1301, USA.
  *
  */
 // --------------------------------------------------------------------------
@@ -844,7 +844,7 @@ void HTMLTableSectionElement::setVAlign( const DOMString &value )
 HTMLCollection HTMLTableSectionElement::rows() const
 {
     if(!impl) return HTMLCollection();
-    return HTMLCollection(impl, HTMLCollectionImpl::TABLE_ROWS);
+    return HTMLCollection(impl, HTMLCollectionImpl::TSECTION_ROWS);
 }
 
 HTMLElement HTMLTableSectionElement::insertRow( long index )
