@@ -16,7 +16,7 @@
  *
  *  You should have received a copy of the GNU Library General Public
  *  License along with this library; if not, write to the Free Software
- *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+ *  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
  */
 
 #include "kjs_window.h"
@@ -67,7 +67,7 @@ void JSEventListener::handleEvent(DOM::Event &evt)
   if (part)
     proxy = part->jScript();
 
-  if (proxy && listener.implementsCall()) {
+  if (proxy && listener.isValid() && listener.implementsCall()) {
     ref();
 
     KJS::ScriptInterpreter *interpreter = static_cast<KJS::ScriptInterpreter *>(proxy->interpreter());
@@ -78,24 +78,10 @@ void JSEventListener::handleEvent(DOM::Event &evt)
 
     // Set "this" to the event's current target
     Object thisObj = Object::dynamicCast(getDOMNode(exec,evt.currentTarget()));
-    ScopeChain oldScope = listener.scope();
-    if ( thisObj.isValid() ) {
-      ScopeChain scope = oldScope;
-      // Add the event's target element to the scope
-      // (and the document, and the form - see KJS::HTMLElement::eventHandlerScope)
-      static_cast<DOMNode*>(thisObj.imp())->pushEventHandlerScope(exec, scope);
-      listener.setScope( scope );
-    }
-    else {
-      if ( m_hackThisObj.isValid() ) { // special hack for Image
-        thisObj = m_hackThisObj;
-      }
-      else
-      {
-        // Window events (window.onload/window.onresize etc.) must have 'this' set to the window.
-        // DocumentImpl::defaultEventHandler sets currentTarget to 0 to mean 'window'.
-        thisObj = win;
-      }
+    if ( !thisObj.isValid() ) {
+      // Window events (window.onload/window.onresize etc.) must have 'this' set to the window.
+      // DocumentImpl::defaultEventHandler sets currentTarget to 0 to mean 'window'.
+      thisObj = win;
     }
 
     Window *window = static_cast<Window*>(win.imp());
@@ -109,8 +95,6 @@ void JSEventListener::handleEvent(DOM::Event &evt)
     Value retval = listener.call(exec, thisObj, args);
     guard.stop();
 
-    listener.setScope( oldScope );
-
     window->setCurrentEvent( 0 );
     interpreter->setCurrentEvent( 0 );
     if ( exec->hadException() )
@@ -139,16 +123,23 @@ Object JSEventListener::listenerObj() const
   return listener;
 }
 
-JSLazyEventListener::JSLazyEventListener(const QString &_code, const QString &_name, const Object &_win, bool _html)
-  : JSEventListener(Object(), 0, _win, _html),
+JSLazyEventListener::JSLazyEventListener(const QString &_code, const QString &_name, const Object &_win, DOM::NodeImpl* _originalNode)
+  : JSEventListener(Object(), 0, _win, true),
     code(_code), name(_name),
     parsed(false)
 {
+  // We don't retain the original node, because we assume it
+  // will stay alive as long as this handler object is around
+  // and we need to avoid a reference cycle. If JS transfers
+  // this handler to another node, parseCode will be called and
+  // then originalNode is no longer needed.
+
+  originalNode = _originalNode;
 }
 
 JSLazyEventListener::~JSLazyEventListener()
 {
-  if (!listener.isNull() && listener.imp()) {
+  if (listener.isValid()) {
     static_cast<Window*>(win.imp())->jsEventListeners.remove(listener.imp());
   }
 }
@@ -156,7 +147,7 @@ JSLazyEventListener::~JSLazyEventListener()
 void JSLazyEventListener::handleEvent(DOM::Event &evt)
 {
   parseCode();
-  if (!listener.isNull()) {
+  if (listener.isValid()) {
     JSEventListener::handleEvent(evt);
   }
 }
@@ -190,7 +181,7 @@ void JSLazyEventListener::parseCode() const
       args.append(KJS::String(code));
       listener = constr.construct(exec, args); // ### is globalExec ok ?
 
-      if ( exec->hadException() ) {
+      if (exec->hadException()) {
         exec->clearException();
 
         // failed to parse, so let's just make this listener a no-op
@@ -200,13 +191,28 @@ void JSLazyEventListener::parseCode() const
       } else {
         DeclaredFunctionImp *declFunc = static_cast<DeclaredFunctionImp*>(listener.imp());
         declFunc->setName(Identifier(name));
+
+        if (originalNode)
+        {
+          // Add the event's home element to the scope
+          // (and the document, and the form - see KJS::HTMLElement::eventHandlerScope)
+          ScopeChain scope = listener.scope();
+
+          Object thisObj = Object::dynamicCast(getDOMNode(exec, originalNode));
+
+          if (thisObj.isValid()) {
+            static_cast<DOMNode*>(thisObj.imp())->pushEventHandlerScope(exec, scope);
+
+            listener.setScope(scope);
+          }
+        }
       }
     }
 
     // no more need to keep the unparsed code around
     code = QString();
 
-    if (!listener.isNull() && listener.imp()) {
+    if (listener.isValid()) {
       static_cast<Window*>(win.imp())->jsEventListeners.insert(listener.imp(),
                                                                (KJS::JSEventListener *)(this));
     }
@@ -217,55 +223,6 @@ void JSLazyEventListener::parseCode() const
 
 // -------------------------------------------------------------------------
 
-const ClassInfo EventConstructor::info = { "EventConstructor", 0, &EventConstructorTable, 0 };
-/*
-@begin EventConstructorTable 3
-  CAPTURING_PHASE	DOM::Event::CAPTURING_PHASE	DontDelete|ReadOnly
-  AT_TARGET		DOM::Event::AT_TARGET		DontDelete|ReadOnly
-  BUBBLING_PHASE	DOM::Event::BUBBLING_PHASE	DontDelete|ReadOnly
-# Reverse-engineered from Netscape
-  MOUSEDOWN		1				DontDelete|ReadOnly
-  MOUSEUP		2				DontDelete|ReadOnly
-  MOUSEOVER		4				DontDelete|ReadOnly
-  MOUSEOUT		8				DontDelete|ReadOnly
-  MOUSEMOVE		16				DontDelete|ReadOnly
-  MOUSEDRAG		32				DontDelete|ReadOnly
-  CLICK			64				DontDelete|ReadOnly
-  DBLCLICK		128				DontDelete|ReadOnly
-  KEYDOWN		256				DontDelete|ReadOnly
-  KEYUP			512				DontDelete|ReadOnly
-  KEYPRESS		1024				DontDelete|ReadOnly
-  DRAGDROP		2048				DontDelete|ReadOnly
-  FOCUS			4096				DontDelete|ReadOnly
-  BLUR			8192				DontDelete|ReadOnly
-  SELECT		16384				DontDelete|ReadOnly
-  CHANGE		32768				DontDelete|ReadOnly
-@end
-*/
-
-EventConstructor::EventConstructor(ExecState *exec)
-  : DOMObject(exec->interpreter()->builtinObjectPrototype())
-{
-}
-
-Value EventConstructor::tryGet(ExecState *exec, const Identifier &p) const
-{
-  return DOMObjectLookupGetValue<EventConstructor, DOMObject>(exec,p,&EventConstructorTable,this);
-}
-
-Value EventConstructor::getValueProperty(ExecState *, int token) const
-{
-  // We use the token as the value to return directly
-  return Number(token);
-}
-
-Value KJS::getEventConstructor(ExecState *exec)
-{
-  return cacheGlobalObject<EventConstructor>(exec, "[[event.constructor]]");
-}
-
-// -------------------------------------------------------------------------
-
 const ClassInfo DOMEvent::info = { "Event", 0, &DOMEventTable, 0 };
 /*
 @begin DOMEventTable 7
@@ -395,8 +352,10 @@ Value KJS::getDOMEvent(ExecState *exec, DOM::Event e)
   ScriptInterpreter* interp = static_cast<ScriptInterpreter *>(exec->interpreter());
   DOMObject *ret = interp->getDOMObject(ei);
   if (!ret) {
-    if (ei->isTextEvent())
+    if (ei->isTextInputEvent())
       ret = new DOMTextEvent(exec, e);
+    else if (ei->isKeyboardEvent())
+      ret = new DOMKeyboardEvent(exec, e);
     else if (ei->isMouseEvent())
       ret = new DOMMouseEvent(exec, e);
     else if (ei->isUIEvent())
@@ -415,7 +374,7 @@ Value KJS::getDOMEvent(ExecState *exec, DOM::Event e)
 DOM::Event KJS::toEvent(const Value& val)
 {
   Object obj = Object::dynamicCast(val);
-  if (obj.isNull() || !obj.inherits(&DOMEvent::info))
+  if (!obj.isValid() || !obj.inherits(&DOMEvent::info))
     return DOM::Event();
 
   const DOMEvent *dobj = static_cast<const DOMEvent*>(obj.imp());
@@ -423,6 +382,34 @@ DOM::Event KJS::toEvent(const Value& val)
 }
 
 // -------------------------------------------------------------------------
+/*
+@begin EventConstantsTable 23
+  CAPTURING_PHASE   DOM::Event::CAPTURING_PHASE DontDelete|ReadOnly
+  AT_TARGET     DOM::Event::AT_TARGET       DontDelete|ReadOnly
+  BUBBLING_PHASE    DOM::Event::BUBBLING_PHASE  DontDelete|ReadOnly
+# Reverse-engineered from Netscape
+  MOUSEDOWN     1               DontDelete|ReadOnly
+  MOUSEUP       2               DontDelete|ReadOnly
+  MOUSEOVER     4               DontDelete|ReadOnly
+  MOUSEOUT      8               DontDelete|ReadOnly
+  MOUSEMOVE     16              DontDelete|ReadOnly
+  MOUSEDRAG     32              DontDelete|ReadOnly
+  CLICK         64              DontDelete|ReadOnly
+  DBLCLICK      128             DontDelete|ReadOnly
+  KEYDOWN       256             DontDelete|ReadOnly
+  KEYUP         512             DontDelete|ReadOnly
+  KEYPRESS      1024                DontDelete|ReadOnly
+  DRAGDROP      2048                DontDelete|ReadOnly
+  FOCUS         4096                DontDelete|ReadOnly
+  BLUR          8192                DontDelete|ReadOnly
+  SELECT        16384               DontDelete|ReadOnly
+  CHANGE        32768               DontDelete|ReadOnly
+@end
+*/
+CREATE_CONSTANT_TABLE(EventConstants, "EventConstants")
+
+IMPLEMENT_PSEUDO_CONSTRUCTOR_WITH_PARENT(EventConstructor, "EventConstructor", DOMEventProto, EventConstants)
+// -------------------------------------------------------------------------
 
 
 const ClassInfo EventExceptionConstructor::info = { "EventExceptionConstructor", 0, &EventExceptionConstructorTable, 0 };
@@ -605,8 +592,11 @@ Value DOMMouseEvent::getValueProperty(ExecState *exec, int token) const
   case OffsetY: // MSIE extension
   {
     DOM::Node node = event.target();
-    node.handle()->getDocument()->updateRendering();
-    khtml::RenderObject *rend = node.handle() ? node.handle()->renderer() : 0L;
+    khtml::RenderObject *rend = 0;
+    if (node.handle()) {
+        node.handle()->getDocument()->updateRendering();
+        rend = node.handle()->renderer();
+    }
     int x = static_cast<DOM::MouseEvent>(event).clientX();
     int y = static_cast<DOM::MouseEvent>(event).clientY();
     if ( rend ) {
@@ -691,27 +681,75 @@ Value DOMMouseEventProtoFunc::tryCall(ExecState *exec, Object &thisObj, const Li
 
 // -------------------------------------------------------------------------
 
-const ClassInfo DOMTextEvent::info = { "TextEvent", &DOMUIEvent::info, &DOMTextEventTable, 0 };
+const ClassInfo DOMKeyEventBase::info = { "KeyEventBase", &DOMUIEvent::info, &DOMKeyEventBaseTable, 0 };
 
 /*
-@begin DOMTextEventTable 2
-  keyVal   	 DOMTextEvent::Key	     DontDelete|ReadOnly
-  virtKeyVal	 DOMTextEvent::VirtKey        DontDelete|ReadOnly
-  outputString	 DOMTextEvent::OutputString   DontDelete|ReadOnly
-  inputGenerated DOMTextEvent::InputGenerated DontDelete|ReadOnly
-  numPad         DOMTextEvent::NumPad         DontDelete|ReadOnly
+@begin DOMKeyEventBaseTable 5
+  keyVal   	 DOMKeyEventBase::Key	      DontDelete|ReadOnly
+  virtKeyVal	 DOMKeyEventBase::VirtKey     DontDelete|ReadOnly
+  ctrlKey        DOMKeyEventBase::CtrlKey     DontDelete|ReadOnly
+  altKey         DOMKeyEventBase::AltKey      DontDelete|ReadOnly
+  shiftKey       DOMKeyEventBase::ShiftKey    DontDelete|ReadOnly
+  altKey         DOMKeyEventBase::AltKey      DontDelete|ReadOnly
+@end
+*/
+
+DOMKeyEventBase::DOMKeyEventBase(const Object &proto, DOM::TextEvent ke) :
+  DOMUIEvent(proto, ke) {}
+
+DOMKeyEventBase::~DOMKeyEventBase()
+{}
+
+Value DOMKeyEventBase::tryGet(ExecState *exec,const Identifier &p) const
+{
+#ifdef KJS_VERBOSE
+  kdDebug(6070) << "DOMKeyEventBase::tryGet " << p.qstring() << endl;
+#endif
+  return DOMObjectLookupGetValue<DOMKeyEventBase,DOMUIEvent>(exec,p,&DOMKeyEventBaseTable,this);
+}
+
+Value DOMKeyEventBase::getValueProperty(ExecState *, int token) const
+{
+  DOM::KeyEventBaseImpl* tevent = impl();
+  switch (token) {
+  case Key:
+    return Number(tevent->keyVal());
+  case VirtKey:
+    return Number(tevent->virtKeyVal());
+  // these modifier attributes actually belong into a KeyboardEvent interface,
+  // but we want them on "keypress" as well.
+  case CtrlKey:
+    return Boolean(tevent->ctrlKey());
+  case ShiftKey:
+    return Boolean(tevent->shiftKey());
+  case AltKey:
+    return Boolean(tevent->altKey());
+  case MetaKey:
+    return Boolean(tevent->metaKey());
+  default:
+    kdDebug(6070) << "WARNING: Unhandled token in DOMTextEvent::getValueProperty : " << token << endl;
+    return KJS::Undefined();
+  }
+}
+
+// -------------------------------------------------------------------------
+const ClassInfo DOMTextEvent::info = { "TextEvent", &DOMKeyEventBase::info, &DOMTextEventTable, 0 };
+
+/*
+@begin DOMTextEventTable 1
+  data           DOMTextEvent::Data          DontDelete|ReadOnly
 @end
 @begin DOMTextEventProtoTable 1
-  initTextEvent	DOMTextEvent::InitTextEvent	DontDelete|Function 10
-  # Missing: initTextEventNS, initModifier
+  initTextEvent	DOMTextEvent::InitTextEvent	DontDelete|Function 5
+  # Missing: initTextEventNS
 @end
 */
 DEFINE_PROTOTYPE("DOMTextEvent",DOMTextEventProto)
 IMPLEMENT_PROTOFUNC_DOM(DOMTextEventProtoFunc)
-IMPLEMENT_PROTOTYPE_WITH_PARENT(DOMTextEventProto,DOMTextEventProtoFunc,DOMUIEventProto)
+IMPLEMENT_PROTOTYPE_WITH_PARENT(DOMTextEventProto,DOMTextEventProtoFunc,DOMUIEventProto) //Note: no proto in KeyBase
 
 DOMTextEvent::DOMTextEvent(ExecState *exec, DOM::TextEvent ke) :
-  DOMUIEvent(DOMTextEventProto::self(exec), ke) {}
+  DOMKeyEventBase(DOMTextEventProto::self(exec), ke) {}
 
 DOMTextEvent::~DOMTextEvent()
 {
@@ -722,52 +760,135 @@ Value DOMTextEvent::tryGet(ExecState *exec, const Identifier &p) const
 #ifdef KJS_VERBOSE
   kdDebug(6070) << "DOMTextEvent::tryGet " << p.qstring() << endl;
 #endif
-  return DOMObjectLookupGetValue<DOMTextEvent,DOMUIEvent>(exec,p,&DOMTextEventTable,this);
+  return DOMObjectLookupGetValue<DOMTextEvent,DOMKeyEventBase>(exec,p,&DOMTextEventTable,this);
 }
 
 Value DOMTextEvent::getValueProperty(ExecState *, int token) const
 {
+  DOM::TextEventImpl* tevent = impl();
   switch (token) {
-  case Key:
-    return Number(static_cast<DOM::TextEvent>(event).keyVal());
-  case VirtKey:
-    return Number(static_cast<DOM::TextEvent>(event).virtKeyVal());
-  case OutputString:
-    return String(static_cast<DOM::TextEvent>(event).outputString());
-  case InputGenerated:
-    return Boolean(static_cast<DOM::TextEvent>(event).inputGenerated());
-  case NumPad:
-    return Boolean(static_cast<DOM::TextEvent>(event).numPad());
+  case Data:
+    return String(tevent->data());
   default:
     kdDebug(6070) << "WARNING: Unhandled token in DOMTextEvent::getValueProperty : " << token << endl;
-    return Value();
+    return KJS::Undefined();
   }
 }
 
 Value DOMTextEventProtoFunc::tryCall(ExecState *exec, Object &thisObj, const List &args)
 {
   KJS_CHECK_THIS( KJS::DOMTextEvent, thisObj );
-  DOM::TextEvent keyEvent = static_cast<DOMTextEvent *>(thisObj.imp())->toTextEvent();
+  DOM::TextEventImpl* keyEvent = static_cast<DOMTextEvent *>(thisObj.imp())->impl();
   switch (id) {
     case DOMTextEvent::InitTextEvent:
-      keyEvent.initTextEvent(args[0].toString(exec).string(), // typeArg
+      keyEvent->initTextEvent(args[0].toString(exec).string(), // typeArg
                             args[1].toBoolean(exec), // canBubbleArg
                             args[2].toBoolean(exec), // cancelableArg
                             toAbstractView(args[3]), // viewArg
-                            args[4].toInteger(exec), // detailArg
-                            args[5].toString(exec).string(),  // outputStringArg
-                            args[6].toInteger(exec), // keyValArg
-                            args[7].toInteger(exec), // virtKeyValArg
-                            args[8].toBoolean(exec), // inputGeneratedArg
-                            args[9].toBoolean(exec));// numPadArg
+                            args[4].toString(exec).string()); // dataArg
 
       return Undefined();
   }
   return Undefined();
 }
+// -------------------------------------------------------------------------
+const ClassInfo DOMKeyboardEvent::info = { "KeyboardEvent", &DOMKeyEventBase::info, &DOMKeyboardEventTable, 0 };
+
+/*
+@begin DOMKeyboardEventTable 2
+  keyIdentifier  DOMKeyboardEvent::KeyIdentifier  DontDelete|ReadOnly
+  keyLocation    DOMKeyboardEvent::KeyLocation    DontDelete|ReadOnly
+@end
+@begin DOMKeyboardEventProtoTable 2
+  initKeyboardEvent	DOMKeyboardEvent::InitKeyboardEvent	DontDelete|Function 7
+  getModifierState      DOMKeyboardEvent::GetModifierState      DontDelete|Function 1
+@end
+*/
+DEFINE_PROTOTYPE("DOMKeyboardEvent",DOMKeyboardEventProto)
+IMPLEMENT_PROTOFUNC_DOM(DOMKeyboardEventProtoFunc)
+IMPLEMENT_PROTOTYPE_WITH_PARENT(DOMKeyboardEventProto,DOMKeyboardEventProtoFunc,DOMUIEventProto) //Note: no proto in KeyBase
+
+DOMKeyboardEvent::DOMKeyboardEvent(ExecState *exec, DOM::TextEvent ke) :
+  DOMKeyEventBase(DOMKeyboardEventProto::self(exec), ke) {}
+
+DOMKeyboardEvent::~DOMKeyboardEvent()
+{
+}
+
+Value DOMKeyboardEvent::tryGet(ExecState *exec, const Identifier &p) const
+{
+#ifdef KJS_VERBOSE
+  kdDebug(6070) << "DOMKeyboardEvent::tryGet " << p.qstring() << endl;
+#endif
+  return DOMObjectLookupGetValue<DOMKeyboardEvent,DOMKeyEventBase>(exec,p,&DOMKeyboardEventTable,this);
+}
+
+Value DOMKeyboardEvent::getValueProperty(ExecState *, int token) const
+{
+  DOM::KeyboardEventImpl* tevent = impl();
+  switch (token) {
+  case KeyIdentifier:
+    return String(tevent->keyIdentifier());
+  case KeyLocation:
+    return Number(tevent->keyLocation());
+  default:
+    kdDebug(6070) << "WARNING: Unhandled token in DOMKeyboardEvent::getValueProperty : " << token << endl;
+    return KJS::Undefined();
+  }
+}
+
+Value DOMKeyboardEventProtoFunc::tryCall(ExecState *exec, Object &thisObj, const List &args)
+{
+  KJS_CHECK_THIS( KJS::DOMKeyboardEvent, thisObj );
+  DOM::KeyboardEventImpl* keyEvent = static_cast<DOMKeyboardEvent *>(thisObj.imp())->impl();
+  switch (id) {
+    case DOMKeyboardEvent::InitKeyboardEvent:
+      keyEvent->initKeyboardEvent(args[0].toString(exec).string(), // typeArg
+                            args[1].toBoolean(exec), // canBubbleArg
+                            args[2].toBoolean(exec), // cancelableArg
+                            toAbstractView(args[3]), // viewArg
+                            args[4].toString(exec).string(), // keyIdentifierArg
+                            args[5].toInteger(exec),         // keyLocationArg
+                            args[6].toString(exec).string()); //modifiersList
+      break;
+    case DOMKeyboardEvent::GetModifierState:
+      return Boolean(keyEvent->getModifierState(args[0].toString(exec).string()));
+  }
+  return Undefined();
+}
 
 // -------------------------------------------------------------------------
+const ClassInfo KeyboardEventConstructor::info = { "KeyboardEventConstructor", 0, &KeyboardEventConstructorTable, 0 };
+/*
+@begin KeyboardEventConstructorTable 4
+  DOM_KEY_LOCATION_STANDARD  DOM::KeyboardEventImpl::DOM_KEY_LOCATION_STANDARD DontDelete|ReadOnly
+  DOM_KEY_LOCATION_LEFT      DOM::KeyboardEventImpl::DOM_KEY_LOCATION_LEFT     DontDelete|ReadOnly
+  DOM_KEY_LOCATION_RIGHT     DOM::KeyboardEventImpl::DOM_KEY_LOCATION_RIGHT    DontDelete|ReadOnly
+  DOM_KEY_LOCATION_NUMPAD    DOM::KeyboardEventImpl::DOM_KEY_LOCATION_NUMPAD   DontDelete|ReadOnly
+@end
+*/
+KeyboardEventConstructor::KeyboardEventConstructor(ExecState* exec)
+  : DOMObject(exec->interpreter()->builtinObjectPrototype())
+{}
 
+Value KeyboardEventConstructor::tryGet(ExecState *exec, const Identifier &p) const
+{
+  return DOMObjectLookupGetValue<KeyboardEventConstructor,DOMObject>(exec,p,&KeyboardEventConstructorTable,this);
+}
+
+Value KeyboardEventConstructor::getValueProperty(ExecState *, int token) const
+{
+  // We use the token as the value to return directly
+  return Number(token);
+}
+
+Value KJS::getKeyboardEventConstructor(ExecState *exec)
+{
+  return cacheGlobalObject<KeyboardEventConstructor>(exec, "[[keyboardEvent.constructor]]");
+}
+
+
+// -------------------------------------------------------------------------
 const ClassInfo MutationEventConstructor::info = { "MutationEventConstructor", 0, &MutationEventConstructorTable, 0 };
 /*
 @begin MutationEventConstructorTable 3
