@@ -53,6 +53,8 @@ namespace KJS {
     DOM::Document toDocument() const { return static_cast<DOM::Document>( node ); }
   };
 
+  DEFINE_PSEUDO_CONSTRUCTOR(HTMLDocumentPseudoCtor)
+
   class HTMLElement : public DOMElement {
   public:
     HTMLElement(ExecState *exec, const DOM::HTMLElement& e) : DOMElement(exec, e) { }
@@ -82,8 +84,7 @@ namespace KJS {
            LinkSheet, TitleText, MetaName, MetaHttpEquiv, MetaContent, MetaScheme,
            BaseHref, BaseTarget, IsIndexForm, IsIndexPrompt, StyleDisabled,
            StyleSheet, StyleType, StyleMedia, BodyBackground, BodyVLink, BodyText,
-           BodyLink, BodyALink, BodyBgColor,  BodyScrollLeft, BodyScrollTop,
-           BodyScrollHeight, BodyScrollWidth, BodyOnLoad,
+           BodyLink, BodyALink, BodyBgColor,  BodyOnLoad,
            FormAction, FormEncType, FormElements, FormLength, FormAcceptCharset,
            FormReset, FormTarget, FormName, FormMethod, FormSubmit, SelectAdd,
            SelectTabIndex, SelectValue, SelectSelectedIndex, SelectLength,
@@ -94,12 +95,14 @@ namespace KJS {
            OptionLabel, OptionValue, InputBlur, InputReadOnly, InputAccept,
            InputSize, InputDefaultValue, InputTabIndex, InputValue, InputType,
            InputFocus, InputMaxLength, InputDefaultChecked, InputDisabled,
-           InputChecked, InputForm, InputAccessKey, InputAlign, InputAlt,
+           InputChecked, InputIndeterminate, InputForm, InputAccessKey, InputAlign, InputAlt,
            InputName, InputSrc, InputUseMap, InputSelect, InputClick,
+           InputSelectionStart, InputSelectionEnd, InputSetSelectionRange,
            TextAreaAccessKey, TextAreaName, TextAreaDefaultValue, TextAreaSelect,
            TextAreaCols, TextAreaDisabled, TextAreaForm, TextAreaType,
            TextAreaTabIndex, TextAreaReadOnly, TextAreaRows, TextAreaValue,
-           TextAreaBlur, TextAreaFocus, ButtonBlur, ButtonFocus, ButtonForm, ButtonTabIndex, ButtonName,
+           TextAreaBlur, TextAreaFocus, TextAreaSelectionStart, TextAreaSelectionEnd, TextAreaSetSelectionRange,
+           TextAreaTextLength, ButtonBlur, ButtonFocus, ButtonForm, ButtonTabIndex, ButtonName,
            ButtonDisabled, ButtonAccessKey, ButtonType, ButtonValue, LabelHtmlFor,
            LabelForm, LabelAccessKey, FieldSetForm, LegendForm, LegendAccessKey,
            LegendAlign, UListType, UListCompact, OListStart, OListCompact,
@@ -111,7 +114,7 @@ namespace KJS {
            AnchorAccessKey, AnchorCoords, AnchorHref, AnchorProtocol, AnchorHost,
            AnchorCharset, AnchorHrefLang, AnchorHostname, AnchorType, AnchorFocus,
            AnchorPort, AnchorPathName, AnchorHash, AnchorSearch, AnchorName,
-           AnchorRev, AnchorTabIndex, AnchorTarget, AnchorText, AnchorBlur,
+           AnchorRev, AnchorTabIndex, AnchorTarget, AnchorText, AnchorBlur, AnchorClick,
            ImageName, ImageAlign, ImageHspace, ImageVspace, ImageUseMap, ImageAlt,
            ImageLowSrc, ImageWidth, ImageIsMap, ImageBorder, ImageHeight,
            ImageLongDesc, ImageSrc, ImageX, ImageY, ImageComplete, ObjectHspace, ObjectHeight, ObjectAlign,
@@ -144,14 +147,15 @@ namespace KJS {
            FrameSetRows, FrameSrc, FrameLocation, FrameFrameBorder, FrameScrolling,
            FrameMarginWidth, FrameLongDesc, FrameMarginHeight, FrameName,
            FrameContentDocument, FrameContentWindow,
-           FrameNoResize, IFrameLongDesc, IFrameAlign,
+           FrameNoResize, FrameWidth, FrameHeight, IFrameLongDesc, IFrameAlign,
            IFrameFrameBorder, IFrameSrc, IFrameName, IFrameHeight,
            IFrameMarginHeight, IFrameMarginWidth, IFrameScrolling, IFrameWidth,
            IFrameContentDocument, IFrameContentWindow,
            MarqueeStart, MarqueeStop,
            LayerTop, LayerLeft, LayerVisibility, LayerBgColor, LayerClip, LayerDocument, LayerLayers,
            ElementInnerHTML, ElementTitle, ElementId, ElementDir, ElementLang,
-           ElementClassName, ElementInnerText, ElementDocument, ElementChildren, ElementAll };
+           ElementClassName, ElementInnerText, ElementDocument,
+	   ElementChildren, ElementAll, ElementScrollIntoView };
 
     DOM::HTMLElement toElement() const { return static_cast<DOM::HTMLElement>(node); }
   };
@@ -167,7 +171,8 @@ namespace KJS {
 
   class HTMLCollection : public DOMObject {
   public:
-    HTMLCollection(ExecState *exec, const DOM::HTMLCollection& c);
+    HTMLCollection(ExecState *exec,  const DOM::HTMLCollection& c);
+    HTMLCollection(const KJS::Object& proto, const DOM::HTMLCollection& c);
     ~HTMLCollection();
     virtual Value tryGet(ExecState *exec, const Identifier &propertyName) const;
     virtual Value call(ExecState *exec, Object &thisObj, const List&args);
@@ -175,6 +180,7 @@ namespace KJS {
     virtual bool implementsCall() const { return true; }
     virtual bool toBoolean(ExecState *) const;
     virtual bool hasProperty(ExecState *exec, const Identifier &p) const;
+    virtual ReferenceList propList(ExecState *exec, bool recursive);
     enum { Item, NamedItem, Tags };
     Value getNamedItems(ExecState *exec, const Identifier &propertyName) const;
     virtual const ClassInfo* classInfo() const { return &info; }
@@ -188,10 +194,15 @@ namespace KJS {
 
   class HTMLSelectCollection : public HTMLCollection {
   public:
-    HTMLSelectCollection(ExecState *exec, const DOM::HTMLCollection& c, const DOM::HTMLSelectElement& e)
-      : HTMLCollection(exec, c), element(e) { }
+    enum { Add };
+    HTMLSelectCollection(ExecState *exec, const DOM::HTMLCollection& c, const DOM::HTMLSelectElement& e);
     virtual Value tryGet(ExecState *exec, const Identifier &propertyName) const;
     virtual void tryPut(ExecState *exec, const Identifier &propertyName, const Value& value, int attr = None);
+    
+    virtual const ClassInfo* classInfo() const { return &info; }
+    static const ClassInfo info;
+    
+    DOM::HTMLSelectElement toElement() const { return element; }
   private:
     DOM::HTMLSelectElement element;
   };
@@ -220,26 +231,6 @@ namespace KJS {
 
   Value getHTMLCollection(ExecState *exec, const DOM::HTMLCollection& c, bool hide=false);
   Value getSelectHTMLCollection(ExecState *exec, const DOM::HTMLCollection& c, const DOM::HTMLSelectElement& e);
-
-  /* Helper function object for determining the number
-   * of occurrences of xxxx as in document.xxxx or window.xxxx.
-   * The order of the TagLength array is the order of preference.
-   */
-  class NamedTagLengthDeterminer {
-  public:
-    struct TagLength {
-      DOM::NodeImpl::Id id; unsigned long length; DOM::NodeImpl *last;
-    };
-    NamedTagLengthDeterminer(const DOM::DOMString& n, TagLength *t, int l)
-      : name(n), tags(t), nrTags(l) {}
-    void operator () (DOM::NodeImpl *start);
-  private:
-    const DOM::DOMString& name;
-    TagLength *tags;
-    int nrTags;
-  };
-
-
 } // namespace
 
 #endif
