@@ -15,7 +15,7 @@
  *
  *  You should have received a copy of the GNU Lesser General Public
  *  License along with this library; if not, write to the Free Software
- *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+ *  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
  */
 
 #include "xmlhttprequest.h"
@@ -45,6 +45,11 @@
 using KIO::NetAccess;
 #endif
 
+#define BANNED_HTTP_HEADERS "authorization,proxy-authorization,"\
+                            "content-length,host,connect,copy,move,"\
+                            "delete,head,trace,put,propfind,proppatch,"\
+                            "mkcol,lock,unlock,options,via"
+
 using namespace KJS;
 using khtml::Decoder;
 
@@ -110,6 +115,7 @@ Object XMLHttpRequestConstructorImp::construct(ExecState *exec, const List &)
 
 const ClassInfo XMLHttpRequest::info = { "XMLHttpRequest", 0, &XMLHttpRequestTable, 0 };
 
+
 /* Source for XMLHttpRequestTable.
 @begin XMLHttpRequestTable 7
   readyState		XMLHttpRequest::ReadyState		DontDelete|ReadOnly
@@ -216,6 +222,7 @@ XMLHttpRequest::XMLHttpRequest(ExecState *exec, const DOM::Document &d)
     qObject(new XMLHttpRequestQObject(this)),
     doc(static_cast<DOM::DocumentImpl*>(d.handle())),
     async(true),
+    contentType(QString::null),
     job(0),
     state(Uninitialized),
     onReadyStateChangeListener(0),
@@ -239,6 +246,8 @@ void XMLHttpRequest::changeState(XMLHttpRequestState newState)
   if (state != newState) {
     state = newState;
 
+    ref();
+
     if (onReadyStateChangeListener != 0 && doc->view() && doc->view()->part()) {
       DOM::Event ev = doc->view()->part()->document().createEvent("HTMLEvents");
       ev.initEvent("readystatechange", true, true);
@@ -250,11 +259,17 @@ void XMLHttpRequest::changeState(XMLHttpRequestState newState)
       ev.initEvent("load", true, true);
       onLoadListener->handleEvent(ev);
     }
+
+    deref();
   }
 }
 
 bool XMLHttpRequest::urlMatchesDocumentDomain(const KURL& _url) const
 {
+  // No need to do work if _url is not valid...
+  if (!_url.isValid())
+    return false;
+
   KURL documentURL(doc->URL());
 
   // a local file can load anything
@@ -278,7 +293,7 @@ void XMLHttpRequest::open(const QString& _method, const KURL& _url, bool _async)
   aborted = false;
 
   // clear stuff from possible previous load
-  requestHeaders = QString();
+  requestHeaders.clear();
   responseHeaders = QString();
   response = QString();
   createdDocument = false;
@@ -295,7 +310,7 @@ void XMLHttpRequest::open(const QString& _method, const KURL& _url, bool _async)
   }
 
 
-  method = _method;
+  method = _method.lower();
   url = _url;
   async = _async;
 
@@ -305,18 +320,59 @@ void XMLHttpRequest::open(const QString& _method, const KURL& _url, bool _async)
 void XMLHttpRequest::send(const QString& _body)
 {
   aborted = false;
-  if (method.lower() == "post" && (url.protocol().lower() == "http" || url.protocol().lower() == "https") ) {
-      // FIXME: determine post encoding correctly by looking in headers for charset
-      job = KIO::http_post( url, QCString(_body.utf8()), false );
+
+  if (method == "post") {
+    QString protocol = url.protocol().lower();
+
+    // Abondon the request when the protocol is other than "http",
+    // instead of blindly changing it to a "get" request.
+    if (!protocol.startsWith("http") && !protocol.startsWith("webdav"))
+    {
+      abort();
+      return;
+    }
+
+    // FIXME: determine post encoding correctly by looking in headers
+    // for charset.
+    QByteArray buf;
+    QCString str = _body.utf8();
+    buf.duplicate(str.data(), str.size() - 1);
+
+    job = KIO::http_post( url, buf, false );
+    if(contentType.isNull())
+      job->addMetaData( "content-type", "Content-type: text/plain" );
+    else
+      job->addMetaData( "content-type", contentType );
   }
-  else
-  {
-     job = KIO::get( url, false, false );
+  else {
+    job = KIO::get( url, false, false );
   }
-  if (requestHeaders.length() > 0) {
-    job->addMetaData("customHTTPHeader", requestHeaders);
+
+  if (!requestHeaders.isEmpty()) {
+    QString rh;
+    QMap<QString, QString>::ConstIterator begin = requestHeaders.begin();
+    QMap<QString, QString>::ConstIterator end = requestHeaders.end();
+    for (QMap<QString, QString>::ConstIterator i = begin; i != end; ++i) {
+      if (i != begin)
+        rh += "\r\n";
+      rh += i.key() + ": " + i.data();
+    }
+
+    job->addMetaData("customHTTPHeader", rh);
+  }
+
+  job->addMetaData("PropagateHttpHeader", "true");
+
+  // Set the default referrer if one is not already supplied
+  // through setRequestHeader. NOTE: the user can still disable
+  // this feature at the protocol level (kio_http).
+  if (requestHeaders.find("Referer") == requestHeaders.end()) {
+    KURL documentURL(doc->URL());
+    documentURL.setPass(QString::null);
+    documentURL.setUser(QString::null);
+    job->addMetaData("referrer", documentURL.url());
+    // kdDebug() << "Adding referrer: " << documentURL << endl;
   }
-  job->addMetaData( "PropagateHttpHeader", "true" );
 
   if (!async) {
     QByteArray data;
@@ -366,14 +422,43 @@ void XMLHttpRequest::abort()
   aborted = true;
 }
 
-void XMLHttpRequest::setRequestHeader(const QString& name, const QString &value)
+void XMLHttpRequest::setRequestHeader(const QString& _name, const QString &value)
 {
-  if (requestHeaders.length() > 0) {
-    requestHeaders += "\r\n";
+  QString name = _name.lower().stripWhiteSpace();
+
+  // Content-type needs to be set seperately from the other headers
+  if(name == "content-type") {
+    contentType = "Content-type: " + value;
+    return;
   }
-  requestHeaders += name;
-  requestHeaders += ": ";
-  requestHeaders += value;
+
+  // Sanitize the referrer header to protect against spoofing...
+  if(name == "referer") {
+    KURL referrerURL(value);
+    if (urlMatchesDocumentDomain(referrerURL))
+      requestHeaders[name] = referrerURL.url();
+    return;
+  }
+
+  // Sanitize the request headers below and handle them as if they are
+  // calls to open. Otherwise, we will end up ignoring them all together!
+  // TODO: Do something about "put" which kio_http sort of supports and
+  // the webDAV headers such as PROPFIND etc...
+  if (name == "get"  || name == "post") {
+    KURL reqURL (doc->URL(), value.stripWhiteSpace());
+    open(name, reqURL, async);
+    return;
+  }
+
+  // Reject all banned headers. See BANNED_HTTP_HEADERS above.
+  // kdDebug() << "Banned HTTP Headers: " << BANNED_HTTP_HEADERS << endl;
+  QStringList bannedHeaders = QStringList::split(',',
+                                  QString::fromLatin1(BANNED_HTTP_HEADERS));
+
+  if (bannedHeaders.contains(name))
+    return;   // Denied
+
+  requestHeaders[name] = value.stripWhiteSpace();
 }
 
 Value XMLHttpRequest::getAllResponseHeaders() const
@@ -421,14 +506,14 @@ Value XMLHttpRequest::getResponseHeader(const QString& name) const
   return String(responseHeaders.mid(headerLinePos + matchLength, endOfLine - (headerLinePos + matchLength)).stripWhiteSpace());
 }
 
-Value XMLHttpRequest::getStatus() const
+static Value httpStatus(const QString& response, bool textStatus = false)
 {
-  if (responseHeaders.isEmpty()) {
+  if (response.isEmpty()) {
     return Undefined();
   }
 
-  int endOfLine = responseHeaders.find("\n");
-  QString firstLine = endOfLine == -1 ? responseHeaders : responseHeaders.left(endOfLine);
+  int endOfLine = response.find("\n");
+  QString firstLine = (endOfLine == -1) ? response : response.left(endOfLine);
   int codeStart = firstLine.find(" ");
   int codeEnd = firstLine.find(" ", codeStart + 1);
 
@@ -436,6 +521,11 @@ Value XMLHttpRequest::getStatus() const
     return Undefined();
   }
 
+  if (textStatus) {
+    QString statusText = firstLine.mid(codeEnd + 1, endOfLine - (codeEnd + 1)).stripWhiteSpace();
+    return String(statusText);
+  }
+
   QString number = firstLine.mid(codeStart + 1, codeEnd - (codeStart + 1));
 
   bool ok = false;
@@ -447,24 +537,14 @@ Value XMLHttpRequest::getStatus() const
   return Number(code);
 }
 
+Value XMLHttpRequest::getStatus() const
+{
+  return httpStatus(responseHeaders);
+}
+
 Value XMLHttpRequest::getStatusText() const
 {
-  if (responseHeaders.isEmpty()) {
-    return Undefined();
-  }
-
-  int endOfLine = responseHeaders.find("\n");
-  QString firstLine = endOfLine == -1 ? responseHeaders : responseHeaders.left(endOfLine);
-  int codeStart = firstLine.find(" ");
-  int codeEnd = firstLine.find(" ", codeStart + 1);
-
-  if (codeStart == -1 || codeEnd == -1) {
-    return Undefined();
-  }
-
-  QString statusText = firstLine.mid(codeEnd + 1, endOfLine - (codeEnd + 1)).stripWhiteSpace();
-
-  return String(statusText);
+  return httpStatus(responseHeaders, true);
 }
 
 void XMLHttpRequest::processSyncLoadResults(const QByteArray &data, const KURL &finalURL, const QString &headers)
@@ -526,6 +606,16 @@ void XMLHttpRequest::slotData(KIO::Job*, const QByteArray &_data)
 {
   if (state < Loaded ) {
     responseHeaders = job->queryMetaData("HTTP-Headers");
+
+    // NOTE: Replace a 304 response with a 200! Both IE and Mozilla do this.
+    // Problem first reported through bug# 110272.
+    int codeStart = responseHeaders.find("304");
+    if ( codeStart != -1) {
+      int codeEnd = responseHeaders.find("\n", codeStart+3);
+      if (codeEnd != -1)
+        responseHeaders.replace(codeStart, (codeEnd-codeStart), "200 OK");
+    }
+
     changeState(Loaded);
   }
 
@@ -535,9 +625,20 @@ void XMLHttpRequest::slotData(KIO::Job*, const QByteArray &_data)
 #endif
 
   if ( decoder == NULL ) {
+    int pos = responseHeaders.find("content-type:", 0, false);
+
+    if ( pos > -1 ) {
+      pos += 13;
+      int index = responseHeaders.find('\n', pos);
+      QString type = responseHeaders.mid(pos, (index-pos));
+      index = type.find (';');
+      if (index > -1)
+        encoding = type.mid( index+1 ).remove(QRegExp("charset[ ]*=[ ]*", false)).stripWhiteSpace();
+    }
+
     decoder = new Decoder;
     if (!encoding.isNull())
-      decoder->setEncoding(encoding.latin1());
+      decoder->setEncoding(encoding.latin1(), Decoder::EncodingFromHTTPHeader);
     else {
       // FIXME: Inherit the default encoding from the parent document?
     }
@@ -622,24 +723,23 @@ Value XMLHttpRequestProtoFunc::tryCall(ExecState *exec, Object &thisObj, const L
       }
 
       QString body;
-
       if (args.size() >= 1) {
-	Object obj = Object::dynamicCast(args[0]);
-	if (!obj.isNull() && obj.inherits(&DOMDocument::info)) {
-	  DOM::Node docNode = static_cast<KJS::DOMDocument *>(obj.imp())->toNode();
-	  DOM::DocumentImpl *doc = static_cast<DOM::DocumentImpl *>(docNode.handle());
-
-	  try {
-	    body = doc->toString().string();
-	    // FIXME: also need to set content type, including encoding!
-
-	  } catch(DOM::DOMException& e) {
-	     Object err = Error::create(exec, GeneralError, "Exception serializing document");
-	     exec->setException(err);
-	  }
-	} else {
-	  body = args[0].toString(exec).qstring();
-	}
+        Object obj = Object::dynamicCast(args[0]);
+        if (obj.isValid() && obj.inherits(&DOMDocument::info)) {
+          DOM::Node docNode = static_cast<KJS::DOMDocument *>(obj.imp())->toNode();
+          DOM::DocumentImpl *doc = static_cast<DOM::DocumentImpl *>(docNode.handle());
+          
+          try {
+            body = doc->toString().string();
+            // FIXME: also need to set content type, including encoding!
+  
+          } catch(DOM::DOMException& e) {
+            Object err = Error::create(exec, GeneralError, "Exception serializing document");
+            exec->setException(err);
+          }
+        } else {
+          body = args[0].toString(exec).qstring();
+        }
       }
 
       request->send(body);
