@@ -268,7 +268,7 @@ void KHTMLParser::parseToken(Token *t)
 
     // if this tag is forbidden inside the current context, pop
     // blocks until we are allowed to add it...
-    while(forbiddenTag[t->tid]) {
+    while(blockStack && forbiddenTag[t->tid]) {
 #ifdef PARSER_DEBUG
         kdDebug( 6035 ) << "t->id: " << t->tid << " is forbidden :-( " << endl;
 #endif
@@ -329,6 +329,11 @@ bool KHTMLParser::insertNode(NodeImpl *n, bool flat)
 #ifdef PARSER_DEBUG
         kdDebug( 6035 ) << "added " << n->nodeName().string() << " to " << tmp->nodeName().string() << ", new current=" << newNode->nodeName().string() << endl;
 #endif
+        // We allow TABLE > FORM in dtd.cpp, but do not allow the form have children in this case
+        if (current->id() == ID_TABLE && id == ID_FORM) {
+            flat = true;
+            static_cast<HTMLFormElementImpl*>(n)->setMalformed(true);
+        }
 
 	// don't push elements without end tag on the stack
         if(tagPriority[id] != 0 && !flat) {
@@ -353,6 +358,7 @@ bool KHTMLParser::insertNode(NodeImpl *n, bool flat)
 	    if(n->isInline()) m_inline = true;
         }
 
+
 #if SPEED_DEBUG < 1
         if(tagPriority[id] == 0 && n->renderer())
             n->renderer()->calcMinMaxWidth();
@@ -366,6 +372,21 @@ bool KHTMLParser::insertNode(NodeImpl *n, bool flat)
         HTMLElementImpl *e;
         bool handled = false;
 
+        // first switch on current element for elements with optional end-tag and inline-only content
+        switch(current->id())
+        {
+        case ID_P:
+        case ID_DT:
+            if(!n->isInline())
+            {
+                popBlock(current->id());
+                return insertNode(n);
+            }
+            break;
+        default:
+            break;
+        }
+
         // switch according to the element to insert
         switch(id)
         {
@@ -514,6 +535,13 @@ bool KHTMLParser::insertNode(NodeImpl *n, bool flat)
                 handled = true;
             }
             break;
+        case ID_DT:
+            e = new HTMLDListElementImpl(document);
+            if ( insertNode(e) ) {
+                insertNode(n);
+                return true;
+            }
+            break;
         case ID_AREA:
         {
             if(map)
@@ -652,7 +680,11 @@ bool KHTMLParser::insertNode(NodeImpl *n, bool flat)
             {
                 NodeImpl *node = current;
                 NodeImpl *parent = node->parentNode();
-
+                // A script may have removed the current node's parent from the DOM
+                // http://bugzilla.opendarwin.org/show_bug.cgi?id=7137
+                // FIXME: we should do real recovery here and re-parent with the correct node.
+                if (!parent)
+                    return false;
                 NodeImpl *parentparent = parent->parentNode();
 
                 if (n->isTextNode() ||
@@ -666,6 +698,8 @@ bool KHTMLParser::insertNode(NodeImpl *n, bool flat)
                     node = (node->id() == ID_TABLE) ? node :
                            ((node->id() == ID_TR ) ? parentparent : parent);
                     NodeImpl *parent = node->parentNode();
+                    if (!parent)
+                        return false;
                     int exceptioncode = 0;
 #ifdef PARSER_DEBUG
                     kdDebug( 6035 ) << "calling insertBefore(" << n->nodeName().string() << "," << node->nodeName().string() << ")" << endl;
@@ -1179,8 +1213,6 @@ void KHTMLParser::processCloseTag(Token *t)
     {
     case ID_HTML+ID_CLOSE_TAG:
     case ID_BODY+ID_CLOSE_TAG:
-    case ID_DT+ID_CLOSE_TAG:
-    case ID_DD+ID_CLOSE_TAG:
         // we never trust those close tags, since stupid webpages close
         // them prematurely
         return;
@@ -1240,6 +1272,8 @@ bool KHTMLParser::isResidualStyleTag(int _id)
         case ID_VAR:
         case ID_DEL:
         case ID_INS:
+        case ID_WBR:
+        case ID_NOBR:
             return true;
         default:
             return false;
@@ -1374,6 +1408,7 @@ void KHTMLParser::handleResidualStyleCloseTagAcrossBlocks(HTMLStackElem* elem)
     // The end result will be: <b>...</b><p><b>Foo</b>Goo</p>
     //
     // Step 1: Remove |blockElem| from its parent, doing a batch detach of all the kids.
+    SharedPtr<NodeImpl> guard(blockElem);
     blockElem->parentNode()->removeChild(blockElem, exceptionCode);
 
     // Step 2: Clone |residualElem|.
@@ -1385,9 +1420,18 @@ void KHTMLParser::handleResidualStyleCloseTagAcrossBlocks(HTMLStackElem* elem)
     NodeImpl* currNode = blockElem->firstChild();
     while (currNode) {
         NodeImpl* nextNode = currNode->nextSibling();
+        SharedPtr<NodeImpl> guard(currNode); //Protect from deletion while moving
         blockElem->removeChild(currNode, exceptionCode);
         newNode->appendChild(currNode, exceptionCode);
         currNode = nextNode;
+
+ // TODO - To be replaced.
+        // Re-register form elements with currently active form, step 1 will have removed them
+        if (form && currNode && currNode->isGenericFormElement())
+        {
+            HTMLGenericFormElementImpl *e = static_cast<HTMLGenericFormElementImpl *>(currNode);
+            form->registerFormElement(e);
+        }
     }
 
     // Step 4: Place |newNode| under |blockElem|.  |blockElem| is still out of the document, so no
