@@ -15,8 +15,8 @@
  *
  * You should have received a copy of the GNU Library General Public License
  * along with this library; see the file COPYING.LIB.  If not, write to
- * the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
- * Boston, MA 02111-1307, USA.
+ * the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
+ * Boston, MA 02110-1301, USA.
  *
  */
 #include "rendering/render_body.h"
@@ -56,14 +56,14 @@ void RenderBody::setStyle(RenderStyle* style)
 void RenderBody::paintBoxDecorations(PaintInfo& paintInfo, int _tx, int _ty)
 {
     //kdDebug( 6040 ) << renderName() << "::paintDecorations()" << endl;
-    QColor c;
-    CachedImage *bg = 0;
+    QColor bgColor;
+    const BackgroundLayer *bgLayer = 0;
 
-    if( parent()->style()->backgroundColor().isValid() || parent()->style()->backgroundImage() ) {
+    if( parent()->style()->hasBackground() ) {
         // the root element already has a non-transparent background of its own
         // so we must fork our own. (CSS2.1 - 14.2 4)
-	c =  style()->backgroundColor();
-	bg = style()->backgroundImage();
+	bgColor =  style()->backgroundColor();
+	bgLayer = style()->backgroundLayers();
     }
 
     int w = width();
@@ -74,18 +74,18 @@ void RenderBody::paintBoxDecorations(PaintInfo& paintInfo, int _tx, int _ty)
     int end = kMin( paintInfo.r.y()+paintInfo.r.height(), _ty + h );
     int mh = end - my;
 
-    paintBackground(paintInfo.p, c, bg, my, mh, _tx, _ty, w, h);
+    paintBackgrounds(paintInfo.p, bgColor, bgLayer, my, mh, _tx, _ty, w, h);
 
     if(style()->hasBorder())
 	paintBorder( paintInfo.p, _tx, _ty, w, h, style() );
 
 }
 
-void RenderBody::repaint(bool immediate)
+void RenderBody::repaint(Priority p)
 {
     RenderObject *cb = containingBlock();
     if(cb)
-	cb->repaint(immediate);
+	cb->repaint(p);
 }
 
 void RenderBody::layout()
