@@ -19,8 +19,8 @@
  *
  * You should have received a copy of the GNU Library General Public License
  * along with this library; see the file COPYING.LIB.  If not, write to
- * the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
- * Boston, MA 02111-1307, USA.
+ * the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
+ * Boston, MA 02110-1301, USA.
  *
  */
 
@@ -43,6 +43,8 @@
 using namespace khtml;
 using namespace Enumerate;
 
+const int cMarkerPadding = 7;
+
 // -------------------------------------------------------------------------
 
 RenderListItem::RenderListItem(DOM::NodeImpl* node)
@@ -157,11 +159,6 @@ void RenderListItem::updateMarkerLocation()
 
 void RenderListItem::calcMinMaxWidth()
 {
-    if (!m_counter) {
-        m_counter = getCounter("list-item", true);
-        if (m_marker) m_counter->setRenderer(m_marker);
-    }
-
     // Make sure our marker is in the correct location.
     updateMarkerLocation();
     if (!minMaxKnown())
@@ -206,6 +203,8 @@ RenderListMarker::~RenderListMarker()
 {
     if(m_listImage)
         m_listImage->deref(this);
+    if (m_listItem)
+        m_listItem->resetListMarker();
 }
 
 void RenderListMarker::setStyle(RenderStyle *s)
@@ -245,7 +244,6 @@ void RenderListMarker::paint(PaintInfo& paintInfo, int _tx, int _ty)
 #endif
     p->setFont(style()->font());
     const QFontMetrics fm = p->fontMetrics();
-    int offset = fm.ascent()*2/3;
 
 
     // The marker needs to adjust its tx, for the case where it's an outside marker.
@@ -275,43 +273,27 @@ void RenderListMarker::paint(PaintInfo& paintInfo, int _tx, int _ty)
         }
     }
 
-
-    bool isPrinting = (paintInfo.p->device()->devType() == QInternal::Printer);
-    if (isPrinting)
-    {
-        if (_ty < paintInfo.r.y())
-        {
-            // This has been painted already we suppose.
-            return;
-        }
-        if (_ty + m_height + paddingBottom() + borderBottom() > paintInfo.r.bottom())
-        {
-            RenderCanvas *rootObj = canvas();
-            if (_ty < rootObj->truncatedAt())
-                rootObj->setBestTruncatedAt(_ty, this);
-            // Let's paint this on the next page.
-            return;
-        }
-    }
-
+    int offset = fm.ascent()*2/3;
+    bool haveImage = m_listImage && !m_listImage->isErrorImage();
+    if (haveImage)
+        offset = m_listImage->pixmap().width();
 
     int xoff = 0;
     int yoff = fm.ascent() - offset;
 
-    if (!listPositionInside())
-        if (listItem->style()->direction() == LTR)
-            xoff = -7 - offset;
+    int bulletWidth = offset/2;
+    if (offset%2)
+        bulletWidth++;
+    if (!listPositionInside()) {
+        if (listItem && listItem->style()->direction() == LTR)
+            xoff = -cMarkerPadding - offset;
         else
-            xoff = offset;
+            xoff = cMarkerPadding + (haveImage ? 0 : (offset - bulletWidth));
+    }
+    else if (style()->direction() == RTL)
+        xoff += haveImage ? cMarkerPadding : (m_width - bulletWidth);
 
     if ( m_listImage && !m_listImage->isErrorImage()) {
-	if ( !listPositionInside() ) {
-            if (style()->direction() == LTR)
-                xoff -= m_listImage->pixmap().width() - fm.ascent()*1/3;
-            else
-                xoff -= fm.ascent()*1/3;
-	}
-
 	p->drawPixmap( QPoint( _tx + xoff, _ty ), m_listImage->pixmap());
         return;
     }
@@ -357,17 +339,29 @@ void RenderListMarker::paint(PaintInfo& paintInfo, int _tx, int _ty)
     case LNONE:
         return;
     default:
-        if (!m_item.isNull()) {
+        if (!m_item.isEmpty()) {
             if(listPositionInside()) {
-            	if(style()->direction() == LTR)
-        	    p->drawText(_tx, _ty, 0, 0, Qt::AlignLeft|Qt::DontClip, m_item);
-            	else
-            	    p->drawText(_tx, _ty, 0, 0, Qt::AlignRight|Qt::DontClip, m_item);
+            	if( style()->direction() == LTR) {
+                    p->drawText(_tx, _ty, 0, 0, Qt::AlignLeft|Qt::DontClip, m_item);
+                    p->drawText(_tx + fm.width(m_item), _ty, 0, 0, Qt::AlignLeft|Qt::DontClip,
+                                QString::fromLatin1(". "));
+                }
+            	else {
+                    const QString& punct(QString::fromLatin1(" ."));
+                    p->drawText(_tx, _ty, 0, 0, Qt::AlignLeft|Qt::DontClip, punct);
+            	    p->drawText(_tx + fm.width(punct), _ty, 0, 0, Qt::AlignLeft|Qt::DontClip, m_item);
+                }
             } else {
-                if(style()->direction() == LTR)
-            	    p->drawText(_tx-offset/2, _ty, 0, 0, Qt::AlignRight|Qt::DontClip, m_item);
-            	else
-            	    p->drawText(_tx+offset/2 + parent()->width(), _ty, 0, 0, Qt::AlignLeft|Qt::DontClip, m_item);
+                if (style()->direction() == LTR) {
+                    const QString& punct(QString::fromLatin1(". "));
+                    p->drawText(_tx-offset/2, _ty, 0, 0, Qt::AlignRight|Qt::DontClip, punct);
+                    p->drawText(_tx-offset/2-fm.width(punct), _ty, 0, 0, Qt::AlignRight|Qt::DontClip, m_item);
+                }
+            	else {
+                    const QString& punct(QString::fromLatin1(" ."));
+            	    p->drawText(_tx+offset/2, _ty, 0, 0, Qt::AlignLeft|Qt::DontClip, punct);
+                    p->drawText(_tx+offset/2+fm.width(punct), _ty, 0, 0, Qt::AlignLeft|Qt::DontClip, m_item);
+                }
 	    }
         }
     }
@@ -403,7 +397,7 @@ void RenderListMarker::calcMinMaxWidth()
     m_markerWidth = m_width = 0;
 
     if(m_listImage && !m_listImage->isErrorImage()) {
-        m_markerWidth = m_listImage->pixmap().width() + 5;
+        m_markerWidth = m_listImage->pixmap().width() + cMarkerPadding;
         if (listPositionInside())
             m_width = m_markerWidth;
         m_height = m_listImage->pixmap().height();
@@ -412,7 +406,31 @@ void RenderListMarker::calcMinMaxWidth()
         return;
     }
 
+    const QFontMetrics &fm = style()->fontMetrics();
+    m_height = fm.ascent();
+
+    // Skip uncounted elements
+    switch(style()->listStyleType()) {
+    // Glyphs:
+        case LDISC:
+        case LCIRCLE:
+        case LSQUARE:
+        case LBOX:
+        case LDIAMOND:
+            m_markerWidth = fm.ascent();
+        	goto end;
+        default:
+            break;
+    }
+
+    { // variable scope
     CounterNode *counter = m_listItem->m_counter;
+    if (!counter) {
+        counter = m_listItem->getCounter("list-item", true);
+        counter->setRenderer(this);
+        m_listItem->m_counter = counter;
+    }
+
 
     assert(counter);
     int value = counter->count();
@@ -420,22 +438,8 @@ void RenderListMarker::calcMinMaxWidth()
     int total = value;
     if (counter->parent()) total = counter->parent()->total();
 
-    const QFontMetrics &fm = style()->fontMetrics();
-    m_height = fm.ascent();
-
     switch(style()->listStyleType())
     {
-// Glyphs:
-    case LDISC:
-    case LCIRCLE:
-    case LSQUARE:
-    case LBOX:
-    case LDIAMOND:
-        m_markerWidth = fm.ascent();
-    	goto end;
-// Unsupported:
-    case CJK_IDEOGRAPHIC:
-        // ### unsupported, we use decimal instead
 // Numeric:
     case LDECIMAL:
         m_item.setNum ( value );
@@ -456,10 +460,19 @@ void RenderListMarker::calcMinMaxWidth()
     case ARABIC_INDIC:
         m_item = toArabicIndic( value );
         break;
+    case LAO:
+        m_item = toLao( value );
+        break;
     case PERSIAN:
     case URDU:
         m_item = toPersianUrdu( value );
         break;
+    case THAI:
+        m_item = toThai( value );
+        break;
+    case TIBETAN:
+        m_item = toTibetan( value );
+        break;
 // Algoritmic:
     case LOWER_ROMAN:
         m_item = toRoman( value, false );
@@ -479,11 +492,11 @@ void RenderListMarker::calcMinMaxWidth()
 // Alphabetic:
     case LOWER_ALPHA:
     case LOWER_LATIN:
-        m_item = toLatin( value, 'a' );
+        m_item = toLowerLatin( value );
         break;
     case UPPER_ALPHA:
     case UPPER_LATIN:
-        m_item = toLatin( value, 'A' );
+        m_item = toUpperLatin( value );
         break;
     case LOWER_GREEK:
         m_item = toLowerGreek( value );
@@ -503,19 +516,35 @@ void RenderListMarker::calcMinMaxWidth()
     case KATAKANA_IROHA:
         m_item = toKatakanaIroha( value );
         break;
-// Quotes:
-    case OPEN_QUOTE:
-        m_item = style()->openQuote( value );
+// Ideographic:
+    case JAPANESE_FORMAL:
+        m_item = toJapaneseFormal( value );
         break;
-    case CLOSE_QUOTE:
-        m_item = style()->closeQuote( value );
+    case JAPANESE_INFORMAL:
+        m_item = toJapaneseInformal( value );
         break;
+    case SIMP_CHINESE_FORMAL:
+        m_item = toSimpChineseFormal( value );
+        break;
+    case SIMP_CHINESE_INFORMAL:
+        m_item = toSimpChineseInformal( value );
+        break;
+    case TRAD_CHINESE_FORMAL:
+        m_item = toTradChineseFormal( value );
+        break;
+    case CJK_IDEOGRAPHIC:
+        // CSS 3 List says treat as trad-chinese-informal
+    case TRAD_CHINESE_INFORMAL:
+        m_item = toTradChineseInformal( value );
+        break;
+// special:
     case LNONE:
         break;
+    default:
+        KHTMLAssert(false);
+    }
+    m_markerWidth = fm.width(m_item) + fm.width(QString::fromLatin1(". "));
     }
-    m_item += QString::fromLatin1(". ");
-    m_markerWidth = fm.width(m_item);
-
 
 end:
     if(listPositionInside())
