@@ -4,7 +4,8 @@
  * Copyright (C) 1999-2003 Lars Knoll (knoll@kde.org)
  *           (C) 1999 Antti Koivisto (koivisto@kde.org)
  *           (C) 2000-2003 Dirk Mueller (mueller@kde.org)
- *           (C) 2002-2003 Apple Computer, Inc.
+ *           (C) 2002-2006 Apple Computer, Inc.
+ *           (C) 2006 Germain Garand <germain@ebooksfrance.org>
  *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Library General Public
@@ -18,8 +19,8 @@
  *
  * You should have received a copy of the GNU Library General Public License
  * along with this library; see the file COPYING.LIB.  If not, write to
- * the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
- * Boston, MA 02111-1307, USA.
+ * the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
+ * Boston, MA 02110-1301, USA.
  *
  */
 
@@ -34,6 +35,7 @@
 #include "rendering/render_inline.h"
 #include "rendering/render_text.h"
 #include "rendering/render_replaced.h"
+#include "rendering/render_generated.h"
 #include "rendering/counter_tree.h"
 
 #include "xml/dom_elementimpl.h"
@@ -76,6 +78,8 @@ using namespace khtml;
 static void *baseOfRenderObjectBeingDeleted;
 #endif
 
+//#define MASK_DEBUG
+
 void* RenderObject::operator new(size_t sz, RenderArena* renderArena) throw()
 {
     return renderArena->allocate(sz);
@@ -98,24 +102,12 @@ RenderObject *RenderObject::createObject(DOM::NodeImpl* node,  RenderStyle* styl
     case NONE:
         break;
     case INLINE:
-    case INLINE_BLOCK:
-    case BLOCK:
-        // In compat mode, if <td> has a display of block, build a table cell instead.
-        // This corrects erroneous HTML.  A better fix would be to implement full-blown
-        // CSS2 anonymous table render object construction, but until then, this will have
-        // to suffice. -dwh
-        if (style->display() == BLOCK && node->id() == ID_TD && style->htmlHacks())
-            o = new (arena) RenderTableCell(node);
-        // In quirks mode if <table> has a display of block, make a table. If it has
-        // a display of inline, make an inline-table.
-        else if (node->id() == ID_TABLE && style->htmlHacks())
-            o = new (arena) RenderTable(node);
-        else if (style->display() == INLINE)
-            o = new (arena) RenderInline(node);
-        else
-            o = new (arena) RenderBlock(node);
+        o = new (arena) RenderInline(node);
         break;
-    case TABLE_CAPTION:
+    case BLOCK:
+        o = new (arena) RenderBlock(node);
+        break;
+    case INLINE_BLOCK:
         o = new (arena) RenderBlock(node);
         break;
     case LIST_ITEM:
@@ -145,6 +137,9 @@ RenderObject *RenderObject::createObject(DOM::NodeImpl* node,  RenderStyle* styl
     case TABLE_CELL:
         o = new (arena) RenderTableCell(node);
         break;
+    case TABLE_CAPTION:
+        o = new (arena) RenderBlock(node);
+        break;
     }
     return o;
 }
@@ -157,7 +152,6 @@ RenderObject::RenderObject(DOM::NodeImpl* node)
       m_parent( 0 ),
       m_previous( 0 ),
       m_next( 0 ),
-      m_counterList( 0 ),
       m_verticalPosition( PositionUndefined ),
       m_needsLayout( false ),
       m_normalChildNeedsLayout( false ),
@@ -175,19 +169,29 @@ RenderObject::RenderObject(DOM::NodeImpl* node)
       m_recalcMinMax( false ),
       m_isText( false ),
       m_inline( true ),
+      m_attached( false ),
 
       m_replaced( false ),
       m_mouseInside( false ),
       m_hasFirstLine( false ),
-      m_isSelectionBorder( false )
+      m_isSelectionBorder( false ),
+      m_isRoot( false ),
+      m_afterPageBreak( false ),
+      m_needsPageClear( false ),
+      m_containsPageBreak( false )
 {
   assert( node );
+  if (node->getDocument()->documentElement() == node) setIsRoot(true);
 }
 
 RenderObject::~RenderObject()
 {
-    if(m_style->backgroundImage())
-        m_style->backgroundImage()->deref(this);
+    const BackgroundLayer* bgLayer = m_style->backgroundLayers();
+    while (bgLayer) {
+        if(bgLayer->backgroundImage())
+            bgLayer->backgroundImage()->deref(this);
+        bgLayer = bgLayer->next();
+    }
 
     if (m_style)
         m_style->deref();
@@ -226,12 +230,12 @@ RenderObject* RenderObject::objectAbove() const
     }
     return obj;
 }
-
+/*
 bool RenderObject::isRoot() const
 {
     return !isAnonymous() &&
         element()->getDocument()->documentElement() == element();
-}
+}*/
 
 bool RenderObject::isHR() const
 {
@@ -270,6 +274,38 @@ void RenderObject::insertChildNode(RenderObject*, RenderObject*)
     KHTMLAssert(0);
 }
 
+RenderObject *RenderObject::nextRenderer() const
+{
+    if (firstChild())
+        return firstChild();
+    else if (nextSibling())
+        return nextSibling();
+    else {
+        const RenderObject *r = this;
+        while (r && !r->nextSibling())
+            r = r->parent();
+        if (r)
+            return r->nextSibling();
+    }
+    return 0;
+}
+
+RenderObject *RenderObject::previousRenderer() const
+{
+    if (previousSibling()) {
+        RenderObject *r = previousSibling();
+        while (r->lastChild())
+            r = r->lastChild();
+        return r;
+    }
+    else if (parent()) {
+        return parent();
+    }
+    else {
+        return 0;
+    }
+}
+
 static void addLayers(RenderObject* obj, RenderLayer* parentLayer, RenderObject*& newObject,
                       RenderLayer*& beforeChild)
 {
@@ -293,7 +329,7 @@ void RenderObject::addLayers(RenderLayer* parentLayer, RenderObject* newObject)
 {
     if (!parentLayer)
         return;
-    
+
     RenderObject* object = newObject;
     RenderLayer* beforeChild = 0;
     ::addLayers(this, parentLayer, object, beforeChild);
@@ -335,36 +371,33 @@ RenderLayer* RenderObject::findNextLayer(RenderLayer* parentLayer, RenderObject*
     // Error check the parent layer passed in.  If it's null, we can't find anything.
     if (!parentLayer)
         return 0;
-
-    // Step 1: Descend into our siblings trying to find the next layer.  If we do find
-    // a layer, and if its parent layer matches our desired parent layer, then we have
-    // a match.
-    for (RenderObject* curr = startPoint ? startPoint->nextSibling() : firstChild();
-         curr; curr = curr->nextSibling()) {
-        RenderLayer* nextLayer = curr->findNextLayer(parentLayer, 0, false);
-        if (nextLayer) {
-            if (nextLayer->parent() == parentLayer)
+    
+    // Step 1: If our layer is a child of the desired parent, then return our layer.
+    RenderLayer* ourLayer = layer();
+    if (ourLayer && ourLayer->parent() == parentLayer)
+        return ourLayer;
+    
+    // Step 2: If we don't have a layer, or our layer is the desired parent, then descend
+    // into our siblings trying to find the next layer whose parent is the desired parent.
+    if (!ourLayer || ourLayer == parentLayer) {
+        for (RenderObject* curr = startPoint ? startPoint->nextSibling() : firstChild();
+             curr; curr = curr->nextSibling()) {
+            RenderLayer* nextLayer = curr->findNextLayer(parentLayer, 0, false);
+            if (nextLayer)
                 return nextLayer;
-            return 0;
         }
     }
-
-    // Step 2: If our layer is the desired parent layer, then we're finished.  We didn't
+    
+    // Step 3: If our layer is the desired parent layer, then we're finished.  We didn't
     // find anything.
-    RenderLayer* ourLayer = layer();
     if (parentLayer == ourLayer)
         return 0;
-
-    // Step 3: If we have a layer, then return that layer.  It will be checked against
-    // the desired parent layer in the for loop above.
-    if (ourLayer)
-        return ourLayer;
-
+    
     // Step 4: If |checkParent| is set, climb up to our parent and check its siblings that
     // follow us to see if we can locate a layer.
     if (checkParent && parent())
         return parent()->findNextLayer(parentLayer, this, true);
-
+    
     return 0;
 }
 
@@ -380,12 +413,20 @@ RenderLayer* RenderObject::enclosingLayer() const
     return 0;
 }
 
+RenderLayer* RenderObject::enclosingStackingContext() const
+{
+    RenderLayer* l = enclosingLayer();
+    while (l && !l->isStackingContext())
+        l = l->parent();
+    return l;
+}
+
 int RenderObject::offsetLeft() const
 {
     if ( isPositioned() )
         return xPos();
 
-    if ( isBody() )
+    if ( isBody() && style()->htmlHacks() )
         return 0;
 
     int x = xPos();
@@ -400,7 +441,7 @@ int RenderObject::offsetLeft() const
          curr = curr->parent() )
         x += curr->xPos();
 
-    if ( offsetPar && offsetPar->isBody() )
+    if ( offsetPar && offsetPar->isBody() && style()->htmlHacks() )
         x += offsetPar->xPos();
 
     return x;
@@ -411,7 +452,7 @@ int RenderObject::offsetTop() const
     if ( isPositioned() )
         return yPos();
 
-    if ( isBody() )
+    if ( isBody() && style()->htmlHacks() )
         return 0;
 
     int y = yPos();
@@ -425,7 +466,7 @@ int RenderObject::offsetTop() const
          curr = curr->parent() )
         y += curr->yPos();
 
-    if ( offsetPar && offsetPar->isBody() )
+    if ( offsetPar && offsetPar->isBody() && style()->htmlHacks() )
         y += offsetPar->yPos();
 
     return y;
@@ -433,10 +474,17 @@ int RenderObject::offsetTop() const
 
 RenderObject* RenderObject::offsetParent() const
 {
+    if (isBody()) 
+        return 0;
+
+    // can't really use containing blocks here (#113280)
     bool skipTables = isPositioned() || isRelPositioned();
+    bool strict = !style()->htmlHacks();
     RenderObject* curr = parent();
-    while (curr && !curr->isPositioned() && !curr->isRelPositioned() && !curr->isBody()) {
-        if (!skipTables && (curr->isTableCell() || curr->isTable()))
+    while (curr && (!curr->element() ||
+                    (!curr->isPositioned() && !curr->isRelPositioned() &&
+                        !(strict && skipTables ? curr->isRoot() : curr->isBody())))) {
+        if (!skipTables && curr->element() && (curr->isTableCell() || curr->isTable()))
             break;
         curr = curr->parent();
     }
@@ -460,31 +508,28 @@ int RenderObject::clientHeight() const
 // scrollWidth/scrollHeight is the size including the overflow area
 short RenderObject::scrollWidth() const
 {
-    return (style()->hidesOverflow() && layer()) ? layer()->scrollWidth() : overflowWidth();
+    return (style()->hidesOverflow() && layer()) ? layer()->scrollWidth() : overflowWidth() - overflowLeft();
 }
 
 int RenderObject::scrollHeight() const
 {
-    return (style()->hidesOverflow() && layer()) ? layer()->scrollHeight() : overflowHeight();
+    return (style()->hidesOverflow() && layer()) ? layer()->scrollHeight() : overflowHeight() - overflowTop();
 }
 
 bool RenderObject::hasStaticX() const
 {
-    return (style()->left().isVariable() && style()->right().isVariable()) ||
-            style()->left().isStatic() ||
-            style()->right().isStatic();
+    return (style()->left().isVariable() && style()->right().isVariable());
 }
 
 bool RenderObject::hasStaticY() const
 {
-    return (style()->top().isVariable() && style()->bottom().isVariable()) || style()->top().isStatic();
+    return (style()->top().isVariable() && style()->bottom().isVariable());
 }
 
 void RenderObject::setPixmap(const QPixmap&, const QRect& /*r*/, CachedImage* image)
 {
     //repaint bg when it finished loading
-    if(image && parent() && style() && style()->backgroundImage() == image
-       && image->valid_rect().size() == image->pixmap_size() ) {
+    if(image && parent() && style() && style()->backgroundLayers()->containsImage(image)) {
         isBody() ? canvas()->repaint() : repaint();
     }
 }
@@ -494,7 +539,7 @@ void RenderObject::setNeedsLayout(bool b, bool markParents)
     bool alreadyNeededLayout = m_needsLayout;
     m_needsLayout = b;
     if (b) {
-        if (!alreadyNeededLayout && markParents) {
+        if (!alreadyNeededLayout && markParents && m_parent) {
             dirtyFormattingContext( false );
             markContainingBlocksForLayout();
         }
@@ -557,9 +602,12 @@ RenderBlock *RenderObject::containingBlock() const
     }
     else if(m_style->position() == ABSOLUTE) {
         while (o &&
-               ( o->style()->position() == STATIC || ( o->isInline() && !o->isReplaced() ) ) &&
-               !o->isRoot() && !o->isCanvas())
+               ( o->style()->position() == STATIC || ( o->isInline() && !o->isReplaced() ) ) && !o->isCanvas()) {
+               // for relpos inlines, return the nearest block - it will host the positioned objects list
+               if (o->isInline() && !o->isReplaced() && o->style()->position() == RELATIVE)
+                   return o->containingBlock();
             o = o->parent();
+        }
     } else {
         while(o && ( ( o->isInline() && !o->isReplaced() ) || o->isTableRow() || o->isTableSection() ||
                        o->isTableCol() || o->isFrameSet() ) )
@@ -1009,8 +1057,7 @@ void RenderObject::paintOutline(QPainter *p, int _tx, int _ty, int w, int h, con
 
     const QColor& oc = style->outlineColor();
     EBorderStyle os = style->outlineStyle();
-    // ### outline-offset should be implemented in renderInline before reactivated here
-    int offset = 0; // style->outlineOffset();
+    int offset = style->outlineOffset();
 
 #ifdef APPLE_CHANGES
     if (style->outlineStyleIsAuto()) {
@@ -1049,9 +1096,9 @@ void RenderObject::paint( PaintInfo&, int /*tx*/, int /*ty*/)
 {
 }
 
-void RenderObject::repaintRectangle(int x, int y, int w, int h, bool immediate, bool f)
+void RenderObject::repaintRectangle(int x, int y, int w, int h, Priority p, bool f)
 {
-    if(parent()) parent()->repaintRectangle(x, y, w, h, immediate, f);
+    if(parent()) parent()->repaintRectangle(x, y, w, h, p, f);
 }
 
 #ifdef ENABLE_DUMP
@@ -1059,10 +1106,15 @@ void RenderObject::repaintRectangle(int x, int y, int w, int h, bool immediate,
 QString RenderObject::information() const
 {
     QString str;
+    int x; int y;
+    absolutePosition(x,y);
+    x += inlineXPos();
+    y += inlineYPos();
     QTextStream ts( &str, IO_WriteOnly );
     ts << renderName()
         << "(" << (style() ? style()->refCount() : 0) << ")"
        << ": " << (void*)this << "  ";
+    ts << "{" << x << " " << y << "} ";
     if (isInline()) ts << "il ";
     if (childrenInline()) ts << "ci ";
     if (isFloating()) ts << "fl ";
@@ -1082,7 +1134,7 @@ QString RenderObject::information() const
         if (element()->hasAnchor()) ts << "anchor ";
         if (element()->focused()) ts << "focus ";
         ts << " <" <<  getTagName(element()->id()) << ">";
-        
+
     } else if (isPseudoAnonymous() && style() && style()->styleType() != RenderStyle::NOPSEUDO) {
         ts << " <" <<  getTagName(node()->id());
         QString pseudo;
@@ -1143,6 +1195,11 @@ static QTextStream &operator<<(QTextStream &ts, const QRect &r)
     return ts << "at (" << r.x() << "," << r.y() << ") size " << r.width() << "x" << r.height();
 }
 
+//A bit like getTagName, but handles XML, too.
+static QString lookupTagName(NodeImpl* node) {
+    return node->getDocument()->getName(NodeImpl::ElementId, node->id()).string();
+}
+
 void RenderObject::dump(QTextStream &ts, const QString &ind) const
 {
     if ( !layer() )
@@ -1155,13 +1212,13 @@ void RenderObject::dump(QTextStream &ts, const QString &ind) const
     }
 
     if (element()) {
-        QString tagName(getTagName(element()->id()));
+        QString tagName(lookupTagName(element()));
         if (!tagName.isEmpty()) {
             ts << " {" << tagName << "}";
         }
     } else if (isPseudoAnonymous() && style() && style()->styleType() != RenderStyle::NOPSEUDO) {
         QString pseudo;
-        QString tagName(getTagName(node()->id()));
+        QString tagName(lookupTagName(node()));
         switch (style()->styleType()) {
           case RenderStyle::FIRST_LETTER:
             pseudo = ":first-letter"; break;
@@ -1193,6 +1250,7 @@ void RenderObject::dump(QTextStream &ts, const QString &ind) const
     if (minMaxKnown()) { ts << " minMaxKnown"; }
     if (overhangingContents()) { ts << " overhangingContents"; }
     if (hasFirstLine()) { ts << " hasFirstLine"; }
+    if (afterPageBreak()) { ts << " afterPageBreak"; }
 }
 #endif
 
@@ -1208,25 +1266,45 @@ void RenderObject::setStyle(RenderStyle *style)
         return;
 
     RenderStyle::Diff d = m_style ? m_style->diff( style ) : RenderStyle::Layout;
-
     //qDebug("m_style: %p new style, diff=%d", m_style,  d);
 
-    if ( d == RenderStyle::Visible && m_parent && m_style &&
-         m_style->outlineWidth() > style->outlineWidth() )
-        repaint();
+    Priority pri = NormalPriority;
+    if (m_style) {
+        pri = HighPriority;
+        if ( d >= RenderStyle::Visible && !isText() && m_parent &&
+             ( d == RenderStyle::Position ||
+               m_style->outlineWidth() > style->outlineWidth() ||
+               ( m_style->hasClip() && !(m_style->clip() == style->clip()) ) ) ) {
+            // schedule a repaint with the old style
+            if (layer() && !isInlineFlow())
+                layer()->repaint(pri);
+            else
+                repaint(pri);
+        }
 
-    if ( m_style &&
-         ( ( isFloating() && m_style->floating() != style->floating() ) ||
-           ( isPositioned() && m_style->position() != style->position() &&
-             style->position() != ABSOLUTE && style->position() != FIXED ) ) )
-        removeFromObjectLists();
+        if ( ( isFloating() && m_style->floating() != style->floating() ) ||
+               ( isPositioned() && m_style->position() != style->position() &&
+                 style->position() != ABSOLUTE && style->position() != FIXED ) )
+            removeFromObjectLists();
 
-    // reset style flags
-    m_floating = false;
-    m_positioned = false;
-    m_relPositioned = false;
-    m_paintBackground = false;
+        if ( layer() ) {
+            if ( ( m_style->hasAutoZIndex() != style->hasAutoZIndex() ||
+                   m_style->zIndex() != style->zIndex() ||
+                   m_style->visibility() != style->visibility() ) ) {
+                layer()->stackingContext()->dirtyZOrderLists();
+                layer()->dirtyZOrderLists();
+            }
+        }
 
+        // reset style flags
+        m_floating = false;
+        m_positioned = false;
+        m_relPositioned = false;
+        m_paintBackground = false;
+    }
+
+    // only honour z-index for non-static objects
+    // ### and objects with opacity
     if ( style->position() == STATIC ) {
         if ( isRoot() )
             style->setZIndex( 0 );
@@ -1234,64 +1312,105 @@ void RenderObject::setStyle(RenderStyle *style)
             style->setHasAutoZIndex();
     }
 
-    if ( layer() && style && m_style ) {
-        if ( ( m_style->hasAutoZIndex() != style->hasAutoZIndex() ||
-               m_style->zIndex() != style->zIndex() ||
-               m_style->visibility() != style->visibility() ) ) {
-            layer()->stackingContext()->dirtyZOrderLists();
-            layer()->dirtyZOrderLists();
-        }
-    }
-
     RenderStyle *oldStyle = m_style;
     m_style = style;
 
-    CachedImage* ob = 0;
-    CachedImage* nb = 0;
+    updateBackgroundImages(oldStyle);
+
+    m_style->ref();
 
-    if (m_style)
-    {
-        m_style->ref();
-        nb = m_style->backgroundImage();
-    }
     if (oldStyle)
-    {
-        ob = oldStyle->backgroundImage();
         oldStyle->deref();
-    }
 
-    if( ob != nb ) {
-        if(ob) ob->deref(this);
-        if(nb) nb->ref(this);
-    }
+    setShouldPaintBackgroundOrBorder(m_style->hasBorder() || m_style->hasBackground());
 
-    setShouldPaintBackgroundOrBorder(m_style->backgroundColor().isValid() ||
-                                        m_style->hasBorder() || nb );
     m_hasFirstLine = (style->getPseudoStyle(RenderStyle::FIRST_LINE) != 0);
     if (m_parent) {
-        if ( d >= RenderStyle::Position ) {
+        if (d == RenderStyle::Position && !attemptDirectLayerTranslation())
+            d = RenderStyle::Layout;
+
+        if ( d > RenderStyle::Position) {
+            // we must perform a full layout
             if (!isText() && d == RenderStyle::CbLayout) {
                 dirtyFormattingContext( true );
             }
             setNeedsLayoutAndMinMaxRecalc();
-        } else if (!isText() && d == RenderStyle::Visible) {
-            if (layer())
-                layer()->repaint();
+        } else if (!isText() && d >= RenderStyle::Visible) {
+            // a repaint is enough
+            if (layer()) {
+                if (canvas() && canvas()->needsWidgetMasks()) {
+                    // update our widget masks
+                    RenderLayer *p, *d = 0;
+                    for (p=layer()->parent();p;p=p->parent())
+                        if (p->hasOverlaidWidgets()) d=p;
+                    if (d) // deepest
+                        d->updateWidgetMasks( canvas()->layer() );
+                }
+            }
+            if (layer() && !isInlineFlow())
+                layer()->repaint(pri);
             else
-                repaint();
+                repaint(pri);
         }
     }
 }
 
+bool RenderObject::attemptDirectLayerTranslation()
+{
+    // When the difference between two successive styles is only 'Position'
+    // we may attempt to save a layout by directly updating the object position.
+
+    KHTMLAssert( m_style->position() != STATIC );
+    if (!layer())
+        return false;
+    setInline(m_style->isDisplayInlineType());
+    setPositioned(m_style->position() != RELATIVE);
+    setRelPositioned(m_style->position() == RELATIVE);
+    int oldXPos = xPos();
+    int oldYPos = yPos();
+    int oldWidth = width();
+    int oldHeight = height();
+    calcWidth();
+    calcHeight();
+    if (oldWidth != width() || oldHeight != height()) {
+        // implicit size change or overconstrained dimensions:
+        // we'll need a layout.
+        setWidth(oldWidth);
+        setHeight(oldHeight);
+        // kdDebug() << "Layer translation failed for " << information() << endl;
+        return false;
+    }
+    layer()->updateLayerPosition();
+    if (m_style->position() != FIXED) {
+        bool needsDocSizeUpdate = true;
+        RenderObject *cb = container();
+        while (cb) {
+            if (cb->hasOverflowClip() && cb->layer()) {
+                cb->layer()->checkScrollbarsAfterLayout();
+                needsDocSizeUpdate = false;
+                break;
+            }
+            cb = cb->container();
+        }
+        if (needsDocSizeUpdate && canvas()) {
+            bool posXOffset = (xPos()-oldXPos >= 0);
+            bool posYOffset = (yPos()-oldYPos >= 0);                         
+            canvas()->updateDocSizeAfterLayerTranslation(this, posXOffset, posYOffset);
+        }
+    }
+    // success
+    return true;
+}
+
 void RenderObject::dirtyFormattingContext( bool checkContainer )
 {
     if (m_markedForRepaint && !checkContainer)
         return;
     m_markedForRepaint = true;
     if (layer() && (style()->position() == FIXED || style()->position() == ABSOLUTE))
-        return;        
-    if (m_parent && (checkContainer || style()->width().isVariable() || style()->height().isVariable() || 
-                     !(isFloating() || flowAroundFloats() || isTableCell())))
+        return;
+    if (m_parent && (checkContainer || style()->width().isVariable() || style()->height().isVariable() ||
+                    !(isFloating() || flowAroundFloats() || isTableCell())))
         m_parent->dirtyFormattingContext(false);
 }
 
@@ -1299,8 +1418,8 @@ void RenderObject::repaintDuringLayout()
 {
     if (canvas()->needsFullRepaint() || isText())
         return;
-    if (layer()) {
-        layer()->repaint( true );
+    if (layer() && !isInlineFlow()) {
+        layer()->repaint( NormalPriority, true );
     } else {
        repaint();
        canvas()->deferredRepaint( this );
@@ -1343,12 +1462,27 @@ void RenderObject::setOverhangingContents(bool p)
     }
 }
 
+void RenderObject::updateBackgroundImages(RenderStyle* oldStyle)
+{
+    // FIXME: This will be slow when a large number of images is used.  Fix by using a dict.
+    const BackgroundLayer* oldLayers = oldStyle ? oldStyle->backgroundLayers() : 0;
+    const BackgroundLayer* newLayers = m_style ? m_style->backgroundLayers() : 0;
+    for (const BackgroundLayer* currOld = oldLayers; currOld; currOld = currOld->next()) {
+        if (currOld->backgroundImage() && (!newLayers || !newLayers->containsImage(currOld->backgroundImage())))
+            currOld->backgroundImage()->deref(this);
+    }
+    for (const BackgroundLayer* currNew = newLayers; currNew; currNew = currNew->next()) {
+        if (currNew->backgroundImage() && (!oldLayers || !oldLayers->containsImage(currNew->backgroundImage())))
+            currNew->backgroundImage()->ref(this);
+    }
+}
+
 QRect RenderObject::viewRect() const
 {
     return containingBlock()->viewRect();
 }
 
-bool RenderObject::absolutePosition(int &xPos, int &yPos, bool f)
+bool RenderObject::absolutePosition(int &xPos, int &yPos, bool f) const
 {
     RenderObject* p = parent();
     if(p) {
@@ -1447,7 +1581,7 @@ RenderObject *RenderObject::container() const
         // we may not have one if we're part of an uninstalled subtree.  We'll
         // climb as high as we can though.
         o = parent();
-        while (o && o->style()->position() == STATIC && !o->isRoot() && !o->isCanvas())
+        while (o && o->style()->position() == STATIC && !o->isCanvas())
             o = o->parent();
     }
     else
@@ -1462,8 +1596,6 @@ DOM::DocumentImpl* RenderObject::document() const
 
 void RenderObject::remove()
 {
-    removeFromObjectLists();
-
     if ( parent() )
         //have parent, take care of the tree integrity
         parent()->removeChild(this);
@@ -1508,6 +1640,10 @@ void RenderObject::detach()
     deleteInlineBoxes();
     remove();
 
+    // make sure our DOM-node don't think we exist
+    if ( node() && node()->renderer() == this)
+        node()->setRenderer(0);
+
     // by default no refcounting
     arenaDelete(renderArena(), this);
 }
@@ -1631,7 +1767,9 @@ bool RenderObject::nodeAtPoint(NodeInfo& info, int _x, int _y, int _tx, int _ty,
                 (_y >= ty) && (_y < ty + height()) && (_x >= tx) && (_x < tx + width())) || isRoot() || isBody();
     bool inOverflowRect = inside;
     if ( !inOverflowRect ) {
-        QRect overflowRect( tx, ty, overflowWidth(), overflowHeight() );
+        int ol = overflowLeft();
+        int ot = overflowTop();
+        QRect overflowRect( tx+ol, ty+ot, overflowWidth()-ol, overflowHeight()-ot );
         inOverflowRect = overflowRect.contains( _x, _y );
     }
 
@@ -1650,37 +1788,32 @@ bool RenderObject::nodeAtPoint(NodeInfo& info, int _x, int _y, int _tx, int _ty,
                 inside = true;
     }
 
-    if (inside) {
-        if (!info.innerNode() && !isInline() && continuation()) {
-            // We are in the margins of block elements that are part of a continuation.  In
-            // this case we're actually still inside the enclosing inline element that was
-            // split.  Go ahead and set our inner node accordingly.
-            info.setInnerNode(continuation()->element());
-            if (!info.innerNonSharedNode())
-                info.setInnerNonSharedNode(continuation()->element());
-        }
-
-        if (info.innerNode() && info.innerNode()->renderer() &&
-            !info.innerNode()->renderer()->isInline() && element() && isInline()) {
-            // Within the same layer, inlines are ALWAYS fully above blocks.  Change inner node.
-            info.setInnerNode(element());
-
-            // Clear everything else.
-            info.setInnerNonSharedNode(0);
-            info.setURLElement(0);
-        }
-
-        if (!info.innerNode() && element())
-            info.setInnerNode(element());
-
-        if(!info.innerNonSharedNode() && element())
-            info.setInnerNonSharedNode(element());
-
-    }
+    if (inside)
+        setInnerNode(info);
 
     return inside;
 }
 
+
+void RenderObject::setInnerNode(NodeInfo& info)
+{
+    if (!info.innerNode() && !isInline() && continuation()) {
+        // We are in the margins of block elements that are part of a continuation.  In
+        // this case we're actually still inside the enclosing inline element that was
+        // split.  Go ahead and set our inner node accordingly.
+        info.setInnerNode(continuation()->element());
+        if (!info.innerNonSharedNode())
+            info.setInnerNonSharedNode(continuation()->element());
+    }
+
+    if (!info.innerNode() && element())
+        info.setInnerNode(element());
+            
+    if(!info.innerNonSharedNode() && element())
+        info.setInnerNonSharedNode(element());
+}
+
+
 short RenderObject::verticalPositionHint( bool firstLine ) const
 {
     short vpos = m_verticalPosition;
@@ -1705,7 +1838,7 @@ short RenderObject::getVerticalPosition( bool firstLine, RenderObject* ref ) con
             vpos = PositionBottom;
         } else {
             if (!ref) ref = parent();
-            bool checkParent = ref->isInline() && !ref->isReplacedBlock() && 
+            bool checkParent = ref->isInline() && !ref->isReplacedBlock() &&
                                !( ref->style()->verticalAlign() == TOP || ref->style()->verticalAlign() == BOTTOM );
             vpos = checkParent ? ref->verticalPositionHint( firstLine ) : 0;
             // don't allow elements nested inside text-top to have a different valignment.
@@ -1866,21 +1999,15 @@ void RenderObject::getTextDecorationColors(int decorations, QColor& underline, Q
         if (currDecs) {
             if (currDecs & UNDERLINE) {
                 decorations &= ~UNDERLINE;
-                underline = st->textDecorationColor().isValid()
-				? st->textDecorationColor()
-				: st->color();
+                underline = st->color();
             }
             if (currDecs & OVERLINE) {
                 decorations &= ~OVERLINE;
-                overline = st->textDecorationColor().isValid()
-				? st->textDecorationColor()
-				: st->color();
+                overline = st->color();
             }
             if (currDecs & LINE_THROUGH) {
                 decorations &= ~LINE_THROUGH;
-                linethrough = st->textDecorationColor().isValid()
-				? st->textDecorationColor()
-				: st->color();
+                linethrough = st->color();
             }
         }
         curr = curr->parent();
@@ -1893,17 +2020,11 @@ void RenderObject::getTextDecorationColors(int decorations, QColor& underline, Q
     if (decorations && curr) {
         RenderStyle *st = curr->style();
         if (decorations & UNDERLINE)
-            underline = st->textDecorationColor().isValid()
-				? st->textDecorationColor()
-				: st->color();
+            underline = st->color();
         if (decorations & OVERLINE)
-            overline = st->textDecorationColor().isValid()
-				? st->textDecorationColor()
-				: st->color();
+            overline = st->color();
         if (decorations & LINE_THROUGH)
-            linethrough = st->textDecorationColor().isValid()
-				? st->textDecorationColor()
-				: st->color();
+            linethrough = st->color();
     }
 }
 
@@ -1935,9 +2056,9 @@ bool RenderObject::usesLineWidth() const
     return (flowAroundFloats() && (style()->width().isVariable() || isHR() || (style()->htmlHacks() && !isTable())));
 }
 
-bool RenderObject::hasCounter(const DOMString& counter) const
+bool RenderObject::hasCounter(const QString& counter) const
 {
-    if (style()) {
+    if (style() && (!isText() || isCounter())) {
         if (lookupCounter(counter)) return true;
         if (style()->hasCounterReset(counter)) {
             return true;
@@ -1954,16 +2075,21 @@ bool RenderObject::hasCounter(const DOMString& counter) const
                 element()->id() == ID_MENU ||
                 element()->id() == ID_DIR))
             return true;
+    } else
+    if (counter == "-khtml-quotes" && isQuote()) {
+        return (static_cast<const RenderQuote*>(this)->quoteCount() != 0);
     }
     return false;
 }
 
-CounterNode* RenderObject::getCounter(const DOMString& counter, bool view, bool counters)
+CounterNode* RenderObject::getCounter(const QString& counter, bool view, bool counters)
 {
 //     kdDebug( 6040 ) << renderName() << " getCounter(" << counter << ")" << endl;
 
     if (!style()) return 0;
 
+    if (isText() && !isCounter()) return 0;
+
     CounterNode *i = lookupCounter(counter);
     if (i) return i;
     int val = 0;
@@ -2016,6 +2142,11 @@ CounterNode* RenderObject::getCounter(const DOMString& counter, bool view, bool
 //             kdDebug( 6040 ) << renderName() << " counter-reset: " << counter << " " << val << endl;
         }
     }
+    else if (counter == "-khtml-quotes" && isQuote()) {
+        i = new CounterNode(this);
+        val = static_cast<RenderQuote*>(this)->quoteCount();
+    }
+
     if (!i) {
         i = new CounterNode(this);
         val = 0;
@@ -2062,19 +2193,21 @@ CounterNode* RenderObject::getCounter(const DOMString& counter, bool view, bool
                 n = n->previousSibling();
             }
             if (sibling->isReset())
+            {
                 if (last != sibling)
                     sibling->insertAfter(i, last);
                 else
                     sibling->insertAfter(i, 0);
-            else
+            }
+            else if (last->parent())
                 last->parent()->insertAfter(i, last);
         }
-        else {
+        else if (parent()) {
             // Nothing found among siblings, let our parent search
             last = parent()->getCounter(counter, false);
             if (last->isReset())
                 last->insertAfter(i, 0);
-            else
+            else if (last->parent())
                 last->parent()->insertAfter(i, last);
         }
     }
@@ -2082,40 +2215,95 @@ CounterNode* RenderObject::getCounter(const DOMString& counter, bool view, bool
     return i;
 }
 
-CounterNode* RenderObject::lookupCounter(const DOMString& counter) const
+CounterNode* RenderObject::lookupCounter(const QString& counter) const
 {
-    CounterList *i = m_counterList;
-    while (i) {
-        if (i->counter == counter)
-            return i->m_counterNode;
-        i = i->next;
-    }
-    return 0;
+    QDict<khtml::CounterNode>* counters = document()->counters(this);
+    if (counters)
+        return counters->find(counter);
+    else
+        return 0;
 }
 
 void RenderObject::detachCounters()
 {
-    CounterList *j, *i = m_counterList;
-    m_counterList = 0;
-    while (i) {
-        i->m_counterNode->remove();
-        delete i->m_counterNode;
-        j = i;
-        i = i->next;
-        delete j;
+    QDict<khtml::CounterNode>* counters = document()->counters(this);
+    if (!counters) return;
+
+    QDictIterator<khtml::CounterNode> i(*counters);
+
+    while (i.current()) {
+        (*i)->remove();
+        delete (*i);
+        ++i;
+    }
+    document()->removeCounters(this);
+}
+
+void RenderObject::insertCounter(const QString& counter, CounterNode* val)
+{
+    QDict<khtml::CounterNode>* counters = document()->counters(this);
+
+    if (!counters) {
+        counters = new QDict<khtml::CounterNode>(11);
+        document()->setCounters(this, counters);
+    }
+
+    counters->insert(counter, val);
+}
+
+void RenderObject::updateWidgetMasks() {
+    for (RenderObject* curr = firstChild(); curr; curr = curr->nextSibling()) {
+        if ( curr->isWidget() && static_cast<RenderWidget*>(curr)->needsMask() ) {
+            QWidget* w = static_cast<RenderWidget*>(curr)->widget();
+            if (!w)
+                return;
+            RenderLayer* l = curr->enclosingStackingContext();
+            QRegion r = l ? l->getMask() : QRegion();
+            int x,y;
+            if (!r.isNull() && curr->absolutePosition(x,y)) {
+                int pbx = curr->borderLeft()+curr->paddingLeft();
+                int pby = curr->borderTop()+curr->paddingTop();
+                x+= pbx;
+                y+= pby;
+                r = r.intersect(QRect(x,y,
+                                  curr->width()-pbx-curr->borderRight()-curr->paddingRight(),
+                                  curr->height()-pby-curr->borderBottom()-curr->paddingBottom()));
+#ifdef MASK_DEBUG
+                QMemArray<QRect> ar = r.rects();
+                kdDebug(6040) << "|| Setting widget mask for " << curr->information() << endl;
+                for (int i = 0; i < ar.size() ; ++i) {
+                    kdDebug(6040) << "		" <<  ar[i] << endl;
+                }
+#endif
+                r.translate(-x,-y);
+                w->setMask(r);
+            } else {
+                w->clearMask();
+            }
+        }
+        else if (!curr->layer() || !curr->layer()->isStackingContext())
+            curr->updateWidgetMasks();
+
     }
 }
 
-void RenderObject::insertCounter(const DOMString& counter, CounterNode* val)
+QRegion RenderObject::visibleFlowRegion(int x, int y) const
 {
-    CounterList* i = new CounterList;
-    i->counter = counter;
-    i->m_counterNode = val;
-    i->next = m_counterList;
-    m_counterList = i;
+    QRegion r;
+    for (RenderObject* ro=firstChild();ro;ro=ro->nextSibling()) {
+        if( !ro->layer() && !ro->isInlineFlow() && ro->style()->visibility() == VISIBLE) {
+            const RenderStyle *s = ro->style();
+            if (ro->isRelPositioned())
+                static_cast<const RenderBox*>(ro)->relativePositionOffset(x,y);
+            if ( s->backgroundImage() || s->backgroundColor().isValid() || s->hasBorder() )
+                r += QRect(x + ro->effectiveXPos(),y + ro->effectiveYPos(), ro->effectiveWidth(), ro->effectiveHeight());
+            else
+                r += ro->visibleFlowRegion(x+ro->xPos(), y+ro->yPos());
+        }
+    }
+    return r;
 }
 
-
 #undef RED_LUMINOSITY
 #undef GREEN_LUMINOSITY
 #undef BLUE_LUMINOSITY
