@@ -18,11 +18,12 @@
  *
  * You should have received a copy of the GNU Library General Public License
  * along with this library; see the file COPYING.LIB.  If not, write to
- * the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
- * Boston, MA 02111-1307, USA.
+ * the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
+ * Boston, MA 02110-1301, USA.
  *
  */
 #include "render_replaced.h"
+#include "render_layer.h"
 #include "render_canvas.h"
 #include "render_line.h"
 
@@ -57,7 +58,7 @@ RenderReplaced::RenderReplaced(DOM::NodeImpl* node)
     // init RenderObject attributes
     setReplaced(true);
 
-    m_intrinsicWidth = 200;
+    m_intrinsicWidth = 300;
     m_intrinsicHeight = 150;
 }
 
@@ -70,9 +71,11 @@ void RenderReplaced::calcMinMaxWidth()
 #endif
 
     m_width = calcReplacedWidth();
-    m_width += paddingLeft() + paddingRight() + borderLeft() + borderRight();
+    m_width = calcBoxWidth( m_width );
 
-    if ( style()->width().isPercent() || style()->height().isPercent() ) {
+    if ( style()->width().isPercent() || style()->height().isPercent() || 
+		    style()->maxWidth().isPercent() || style()->maxHeight().isPercent() ||
+		    style()->minWidth().isPercent() || style()->minHeight().isPercent() ) {
         m_minWidth = 0;
         m_maxWidth = m_width;
     }
@@ -95,9 +98,12 @@ RenderWidget::RenderWidget(DOM::NodeImpl* node)
     m_widget = 0;
     // a widget doesn't support being anonymous
     assert(!isAnonymous());
-    m_view = node->getDocument()->view();
+    m_view  = node->getDocument()->view();
+    m_arena.reset(renderArena());
     m_resizePending = false;
     m_discardResizes = false;
+    m_isKHTMLWidget = false;
+    m_needsMask = false;
 
     // this is no real reference counting, its just there
     // to make sure that we're not deleted while we're recursed
@@ -151,7 +157,7 @@ void  RenderWidget::resizeWidget( int w, int h )
     w = kMin( w, 2000 );
 
     if (m_widget->width() != w || m_widget->height() != h) {
-        m_resizePending = !strcmp(m_widget->name(), "__khtml");
+        m_resizePending = isKHTMLWidget();
         ref();
         element()->ref();
         QApplication::postEvent( this, new QWidgetResizeEvent( w, h ) );
@@ -197,7 +203,8 @@ void RenderWidget::setQWidget(QWidget *widget)
         if (m_widget) {
             m_widget->removeEventFilter(this);
             disconnect( m_widget, SIGNAL( destroyed()), this, SLOT( slotWidgetDestructed()));
-            delete m_widget;
+            m_widget->hide();
+            m_widget->deleteLater(); //Might happen due to event on the widget, so be careful
             m_widget = 0;
         }
         m_widget = widget;
@@ -205,7 +212,7 @@ void RenderWidget::setQWidget(QWidget *widget)
             connect( m_widget, SIGNAL( destroyed()), this, SLOT( slotWidgetDestructed()));
             m_widget->installEventFilter(this);
 
-            if ( !strcmp(m_widget->name(), "__khtml") && !::qt_cast<QFrame*>(m_widget))
+            if ( (m_isKHTMLWidget = !strcmp(m_widget->name(), "__khtml")) && !::qt_cast<QFrame*>(m_widget))
                 m_widget->setBackgroundMode( QWidget::NoBackground );
 
             if (m_widget->focusPolicy() > QWidget::StrongFocus)
@@ -231,9 +238,26 @@ void RenderWidget::layout( )
 {
     KHTMLAssert( needsLayout() );
     KHTMLAssert( minMaxKnown() );
-    if ( m_widget )
+    if ( m_widget ) {
         resizeWidget( m_width-borderLeft()-borderRight()-paddingLeft()-paddingRight(),
                       m_height-borderTop()-borderBottom()-paddingTop()-paddingBottom() );
+        if (!isKHTMLWidget() && !isFrame() && !m_needsMask) {
+            m_needsMask = true;
+            RenderLayer* rl = enclosingStackingContext();
+            RenderLayer* el = enclosingLayer();
+            while (rl && el && el != rl) {
+                if (el->renderer()->style()->position() != STATIC) {
+                    m_needsMask = false;
+                    break;
+                }
+                el = el->parent();
+            }                                                                                                                                      
+            if (m_needsMask) {
+                if (rl) rl->setHasOverlaidWidgets();
+                canvas()->setNeedsWidgetMasks();
+            }
+        }
+    }
 
     setNeedsLayout(false);
 }
@@ -253,7 +277,7 @@ void RenderWidget::updateFromElement()
             int lowlightVal = 100 + (2*contrast_+4)*10;
 
             if (backgroundColor.isValid()) {
-                if (strcmp(widget()->name(), "__khtml"))
+                if (!isKHTMLWidget())
                     widget()->setEraseColor(backgroundColor );
                 for ( int i = 0; i < QPalette::NColorGroups; ++i ) {
                     pal.setColor( (QPalette::ColorGroup)i, QColorGroup::Background, backgroundColor );
@@ -307,7 +331,7 @@ void RenderWidget::updateFromElement()
             m_widget->unsetPalette();
         // Border:
         QFrame* frame = ::qt_cast<QFrame*>(m_widget);
-        if (frame && canHaveBorder()) {
+        if (frame) {
             if (shouldPaintBackgroundOrBorder())
             {
                 frame->setFrameShape(QFrame::NoFrame);
@@ -355,8 +379,9 @@ void RenderWidget::paint(PaintInfo& paintInfo, int _tx, int _ty)
 {
     _tx += m_x;
     _ty += m_y;
-
-    if (shouldPaintBackgroundOrBorder() && paintInfo.phase != PaintActionOutline)
+ 
+    if (shouldPaintBackgroundOrBorder() && 
+          (paintInfo.phase == PaintActionChildBackground || paintInfo.phase == PaintActionChildBackgrounds))
         paintBoxDecorations(paintInfo, _tx, _ty);
 
     if (!m_widget || !m_view || paintInfo.phase != PaintActionForeground)
@@ -370,14 +395,10 @@ void RenderWidget::paint(PaintInfo& paintInfo, int _tx, int _ty)
          (_tx + m_width <= paintInfo.r.left()) || (_tx > paintInfo.r.right()) )
         return;
 
-    // add offset for relative positioning
-    if(isRelPositioned())
-        relativePositionOffset(_tx, _ty);
-
     int xPos = _tx+borderLeft()+paddingLeft();
     int yPos = _ty+borderTop()+paddingTop();
 
-    bool khtmlw = !strcmp(m_widget->name(), "__khtml");
+    bool khtmlw = isKHTMLWidget();
     int childw = m_widget->width();
     int childh = m_widget->height();
     if ( (childw == 2000 || childh == 3072) && m_widget->inherits( "KHTMLView" ) ) {
@@ -676,6 +697,31 @@ void RenderWidget::EventPropagator::sendEvent(QEvent *e) {
     }
 }
 
+void RenderWidget::ScrollViewEventPropagator::sendEvent(QEvent *e) {
+    switch(e->type()) {
+    case QEvent::MouseButtonPress:
+        viewportMousePressEvent(static_cast<QMouseEvent *>(e));
+        break;
+    case QEvent::MouseButtonRelease:
+        viewportMouseReleaseEvent(static_cast<QMouseEvent *>(e));
+        break;
+    case QEvent::MouseButtonDblClick:
+        viewportMouseDoubleClickEvent(static_cast<QMouseEvent *>(e));
+        break;
+    case QEvent::MouseMove:
+        viewportMouseMoveEvent(static_cast<QMouseEvent *>(e));
+        break;
+    case QEvent::KeyPress:
+        keyPressEvent(static_cast<QKeyEvent *>(e));
+        break;
+    case QEvent::KeyRelease:
+        keyReleaseEvent(static_cast<QKeyEvent *>(e));
+        break;
+    default:
+        break;
+    }
+}
+
 bool RenderWidget::handleEvent(const DOM::EventImpl& ev)
 {
     bool ret = false;
@@ -683,6 +729,7 @@ bool RenderWidget::handleEvent(const DOM::EventImpl& ev)
     case EventImpl::MOUSEDOWN_EVENT:
     case EventImpl::MOUSEUP_EVENT:
     case EventImpl::MOUSEMOVE_EVENT: {
+        if (!ev.isMouseEvent()) break;
         const MouseEventImpl &me = static_cast<const MouseEventImpl &>(ev);
         QMouseEvent* const qme = me.qEvent();
 
@@ -690,8 +737,7 @@ bool RenderWidget::handleEvent(const DOM::EventImpl& ev)
         int absy = 0;
 
         absolutePosition(absx, absy);
-
-        const QPoint p(me.clientX() - absx + m_view->contentsX(),
+        QPoint p(me.clientX() - absx + m_view->contentsX(),
                  me.clientY() - absy + m_view->contentsY());
         QMouseEvent::Type type;
         int button = 0;
@@ -741,7 +787,11 @@ bool RenderWidget::handleEvent(const DOM::EventImpl& ev)
 //                   << " pos=" << p << " type=" << type
 //                   << " button=" << button << " state=" << state << endl;
         QMouseEvent e(type, p, button, state);
-        static_cast<EventPropagator *>(m_widget)->sendEvent(&e);
+        QScrollView * sc = ::qt_cast<QScrollView*>(m_widget);
+        if (sc && !::qt_cast<QListBox*>(m_widget))
+            static_cast<ScrollViewEventPropagator *>(sc)->sendEvent(&e);
+        else
+            static_cast<EventPropagator *>(m_widget)->sendEvent(&e);
         ret = e.isAccepted();
         break;
     }
@@ -749,12 +799,21 @@ bool RenderWidget::handleEvent(const DOM::EventImpl& ev)
         // do nothing; see the mapping table below
         break;
     case EventImpl::KEYUP_EVENT: {
-        QKeyEvent* const ke = static_cast<const TextEventImpl &>(ev).qKeyEvent();
+        if (!ev.isKeyRelatedEvent()) break;
+
+        const KeyEventBaseImpl& domKeyEv = static_cast<const KeyEventBaseImpl &>(ev);
+        if (domKeyEv.isSynthetic() && !acceptsSyntheticEvents()) break;
+
+        QKeyEvent* const ke = domKeyEv.qKeyEvent();
         static_cast<EventPropagator *>(m_widget)->sendEvent(ke);
         ret = ke->isAccepted();
         break;
     }
-    case EventImpl::KHTML_KEYPRESS_EVENT: {
+    case EventImpl::KEYPRESS_EVENT: {
+        if (!ev.isKeyRelatedEvent()) break;
+
+        const KeyEventBaseImpl& domKeyEv = static_cast<const KeyEventBaseImpl &>(ev);
+        if (domKeyEv.isSynthetic() && !acceptsSyntheticEvents()) break;
 
         // See KHTMLView::dispatchKeyEvent: autorepeat is just keypress in the DOM
         // but it's keyrelease+keypress in Qt. So here we do the inverse mapping as
@@ -769,7 +828,7 @@ bool RenderWidget::handleEvent(const DOM::EventImpl& ev)
         // Qt::KeyPress is sent for DOM keypress and not DOM keydown to allow
         // sites to block a key with onkeypress, #99749
 
-        QKeyEvent* const ke = static_cast<const TextEventImpl &>(ev).qKeyEvent();
+        QKeyEvent* const ke = domKeyEv.qKeyEvent();
         if (ke->isAutoRepeat()) {
             QKeyEvent releaseEv( QEvent::KeyRelease, ke->key(), ke->ascii(), ke->state(),
                                ke->text(), ke->isAutoRepeat(), ke->count() );
@@ -800,8 +859,11 @@ void RenderWidget::deref()
 {
     if (_ref) _ref--;
 //     qDebug( "deref(%p): width get count is %d", this, _ref);
-    if (!_ref)
-        arenaDelete(renderArena());
+    if (!_ref) {
+        khtml::SharedPtr<RenderArena> guard(m_arena); //Since delete on us gets called -first-,
+                                                      //before the arena free
+        arenaDelete(m_arena.get());
+    }
 }
 
 FindSelectionResult RenderReplaced::checkSelectionPoint(int _x, int _y, int _tx, int _ty, DOM::NodeImpl*& node, int &offset, SelPointState &)
