@@ -92,6 +92,99 @@ void InlineTextBox::operator delete(void* ptr, size_t sz)
     *(size_t *)ptr = sz;
 }
 
+void InlineTextBox::selectionStartEnd(int& sPos, int& ePos)
+{
+    int startPos, endPos;
+    if (object()->selectionState() == RenderObject::SelectionInside) {
+        startPos = 0;
+        endPos = renderText()->string()->l;
+    } else {
+        renderText()->selectionStartEnd(startPos, endPos);
+        if (object()->selectionState() == RenderObject::SelectionStart)
+            endPos = renderText()->string()->l;
+        else if (object()->selectionState() == RenderObject::SelectionEnd)
+            startPos = 0;
+    }
+
+    sPos = kMax(startPos - m_start, 0);
+    ePos = kMin(endPos - m_start, (int)m_len);
+}
+
+RenderObject::SelectionState InlineTextBox::selectionState()
+{
+    RenderObject::SelectionState state = object()->selectionState();
+    if (state == RenderObject::SelectionStart || state == RenderObject::SelectionEnd ||
+        state == RenderObject::SelectionBoth) {
+        int startPos, endPos;
+        renderText()->selectionStartEnd(startPos, endPos);
+        
+        bool start = (state != RenderObject::SelectionEnd && startPos >= m_start && startPos < m_start + m_len);
+        bool end = (state != RenderObject::SelectionStart && endPos > m_start && endPos <= m_start + m_len);
+        if (start && end)
+            state = RenderObject::SelectionBoth;
+        else if (start)
+            state = RenderObject::SelectionStart;
+        else if (end)
+            state = RenderObject::SelectionEnd;
+        else if ((state == RenderObject::SelectionEnd || startPos < m_start) &&
+                 (state == RenderObject::SelectionStart || endPos > m_start + m_len))
+            state = RenderObject::SelectionInside;
+    }
+    return state;
+}
+
+void InlineTextBox::paint(RenderObject::PaintInfo& i, int tx, int ty)
+{
+    if (object()->isBR() || object()->style()->visibility() != VISIBLE || i.phase == PaintActionOutline)
+        return;
+
+    if (i.phase == PaintActionSelection && object()->selectionState() == RenderObject::SelectionNone)
+        // When only painting the selection, don't bother to paint if there is none.
+        return;
+
+    int xPos = tx + m_x;
+    int w = width();
+    if ((xPos >= i.r.x() + i.r.width()) || (xPos + w <= i.r.x()))
+        return;
+
+    // Set our font.
+    RenderStyle* styleToUse = object()->style(m_firstLine);
+    int d = styleToUse->textDecorationsInEffect();
+    if (styleToUse->font() != i.p->font())
+        i.p->setFont(styleToUse->font());
+    const Font *font = &styleToUse->htmlFont();
+    bool haveSelection = selectionState() != RenderObject::SelectionNone;
+    
+    // Now calculate startPos and endPos, for painting selection.
+    // We paint selection while endPos > 0
+    int ePos = 0, sPos = 0;
+    if (haveSelection && !object()->canvas()->staticMode()) {
+        selectionStartEnd(sPos, ePos);
+    }
+    if (styleToUse->color() != i.p->pen().color())
+        i.p->setPen(styleToUse->color());
+
+    if (m_len > 0 && i.phase != PaintActionSelection) {
+        if (styleToUse->textShadow())
+            paintShadow(i.p, font, tx, ty, styleToUse->textShadow());
+        if (!haveSelection || sPos != 0 || ePos != m_len) {
+            font->drawText(i.p, m_x + tx, m_y + ty + m_baseline, renderText()->string()->s, renderText()->string()->l, m_start, m_len,
+                           m_toAdd, m_reversed ? QPainter::RTL : QPainter::LTR);
+        }
+    }
+
+    if (d != TDNONE && i.phase != PaintActionSelection && styleToUse->htmlHacks()) {
+        i.p->setPen(styleToUse->color());
+        paintDecoration(i.p, font, tx, ty, d);
+    }
+
+    if (haveSelection && i.phase == PaintActionSelection) {
+        //kdDebug(6040) << this << " paintSelection with startPos=" << sPos << " endPos=" << ePos << endl;
+        if ( sPos < ePos )
+	    paintSelection(font, renderText(), i.p, styleToUse, tx, ty, sPos, ePos, d);
+    }
+}
+
 /** returns the proper ::selection pseudo style for the given element
  * @return the style or 0 if no ::selection pseudo applies.
  */
@@ -130,20 +223,42 @@ void InlineTextBox::paintSelection(const Font *f, RenderText *text, QPainter *p,
 	// ### should be at most retrieved once per render text
 	QColor bg = khtml::retrieveBackgroundColor(text);
 	// It may happen that the contrast is -- well -- virtually non existent.
-	// In this case, simply invert the colors
-	if (!khtml::hasSufficientContrast(hbg, bg)) {
-	    hc = QColor(0xff-hc.red(),0xff-hc.green(),0xff-hc.blue());
-	    hbg = QColor(0xff-hbg.red(),0xff-hbg.green(),0xff-hbg.blue());
-	}/*end if*/
+	// In this case, simply swap the colors, thus in compliance with
+	// NN4 (win32 only), IE, and Mozilla.
+	if (!khtml::hasSufficientContrast(hbg, bg))
+	    qSwap(hc, hbg);
     }
 
     p->setPen(hc);
 
     //kdDebug( 6040 ) << "textRun::painting(" << QConstString(text->str->s + m_start, m_len).string().left(30) << ") at(" << m_x+tx << "/" << m_y+ty << ")" << endl;
+
+    const bool needClipping = startPos != 0 || endPos != m_len;
+
+    if (needClipping) {
+        p->save();
+
+        int visualSelectionStart = f->width(text->str->s, text->str->l, m_start, startPos, m_start, m_start + m_len, m_toAdd);
+        int visualSelectionEnd = f->width(text->str->s, text->str->l, m_start, endPos, m_start, m_start + m_len, m_toAdd);
+        int visualSelectionWidth = visualSelectionEnd - visualSelectionStart;
+        if (m_reversed) {
+            visualSelectionStart = f->width(text->str->s, text->str->l, m_start, m_len) - visualSelectionEnd;
+        }
+
+        QRect selectionRect(m_x + tx + visualSelectionStart, m_y + ty, visualSelectionWidth, height());
+        QRegion r(selectionRect);
+        if (p->hasClipping())
+            r &= p->clipRegion(QPainter::CoordPainter);
+        p->setClipRegion(r, QPainter::CoordPainter);
+    }
+
     f->drawText(p, m_x + tx, m_y + ty + m_baseline, text->str->s, text->str->l,
-    		m_start, m_len, m_toAdd,
-		m_reversed ? QPainter::RTL : QPainter::LTR,
-		startPos, endPos, hbg, m_y + ty, height(), deco);
+                m_start, m_len, m_toAdd,
+                m_reversed ? QPainter::RTL : QPainter::LTR,
+                needClipping ? 0 : startPos, needClipping ? m_len : endPos,
+		hbg, m_y + ty, height(), deco);
+
+    if (needClipping) p->restore();
 }
 
 void InlineTextBox::paintDecoration( QPainter *pt, const Font *f, int _tx, int _ty, int deco)
@@ -222,7 +337,7 @@ void InlineTextBox::paintShadow(QPainter *pt, const Font *f, int _tx, int _ty, c
             bmap[n] = f;
         }
 
-        float factor = 0.0; // maximal opacity-sum
+        float factor = 0.0; // maximal potential opacity-sum
         for(int n=-thickness; n<=thickness; n++)
             for(int m=-thickness; m<=thickness; m++) {
                 int d = n*n+m*m;
@@ -230,7 +345,8 @@ void InlineTextBox::paintShadow(QPainter *pt, const Font *f, int _tx, int _ty, c
                     factor += bmap[d];
             }
 
-        factor = 1.0/factor;
+        // arbitratry factor adjustment to make shadows solid.
+        factor = factor/1.333;
 
         // alpha map
         float* amap = (float*)alloca(sizeof(float)*(h*w));
@@ -261,9 +377,14 @@ void InlineTextBox::paintShadow(QPainter *pt, const Font *f, int _tx, int _ty, c
         int g = qGreen(color);
         int b = qBlue(color);
 
+        // divide by factor
+        factor = 1.0/factor;
+
         for(int j=0; j<h; j++) {
             for(int i=0; i<w; i++) {
-                res.setPixel(i,j, qRgba(r,g,b,(int)(amap[i+j*w]*factor*255.0)));
+                int a = (int)(amap[i+j*w] * factor * 255.0);
+                if (a > 255) a = 255;
+                res.setPixel(i,j, qRgba(r,g,b,a));
             }
         }
 
@@ -307,12 +428,12 @@ FindSelectionResult InlineTextBox::checkSelectionPoint(int _x, int _y, int _tx,
     }
     if ( _x > _tx + m_x + m_width ) {
 	// to the right
-	return m_reversed ? SelectionPointBeforeInLine : SelectionPointAfterInLine;
+	return  SelectionPointAfterInLine;
     }
 
     // The Y matches, check if we're on the left
     if ( _x < _tx + m_x ) {
-        return m_reversed ? SelectionPointAfterInLine : SelectionPointBeforeInLine;
+        return SelectionPointBeforeInLine;
     }
 
     // consider spacing for justified text
@@ -538,7 +659,7 @@ void RenderText::setStyle(RenderStyle *_style)
         RenderObject::setStyle( _style );
         m_lineHeight = RenderObject::lineHeight(false);
 
-        if (changedText) {
+        if (!isBR() && changedText) {
             DOM::DOMStringImpl* textToTransform = originalString();
             if (textToTransform)
                 setText(textToTransform, true);
@@ -808,17 +929,9 @@ long RenderText::maxOffset() const
   return m_lines[count - 1]->m_start + m_lines[count - 1]->m_len;
 }
 
-bool RenderText::absolutePosition(int &xPos, int &yPos, bool)
+bool RenderText::absolutePosition(int &xPos, int &yPos, bool) const
 {
     return RenderObject::absolutePosition(xPos, yPos, false);
-
-    if(parent() && parent()->absolutePosition(xPos, yPos, false)) {
-        xPos -= paddingLeft() + borderLeft();
-        yPos -= borderTop() + paddingTop();
-        return true;
-    }
-    xPos = yPos = 0;
-    return false;
 }
 
 bool RenderText::posOfChar(int chr, int &x, int &y)
@@ -840,169 +953,9 @@ bool RenderText::posOfChar(int chr, int &x, int &y)
     return false;
 }
 
-void RenderText::paint( PaintInfo& pI, int tx, int ty)
+void RenderText::paint( PaintInfo& /*pI*/, int /*tx*/, int /*ty*/)
 {
-    if (pI.phase == PaintActionSelection && selectionState() == SelectionNone)
-        // When only painting the selection, don't bother to paint if there is none.
-        return;
-    if (pI.phase != PaintActionForeground && pI.phase != PaintActionSelection
-        || style()->visibility() != VISIBLE)
-        return;
-
-    int s = m_lines.count() - 1;
-    if ( s < 0 ) return;
-
-    if ( ty + m_lines[0]->m_y > pI.r.bottom() ) return;
-    if ( ty + m_lines[s]->m_y + m_lines[s]->height() < pI.r.top() ) return;
-
-    int ow = style()->outlineWidth();
-    RenderStyle* pseudoStyle = hasFirstLine() ? style()->getPseudoStyle(RenderStyle::FIRST_LINE) : 0;
-    InlineTextBox f(0, pI.r.top()-ty);
-    int si = m_lines.findFirstMatching(&f);
-    // something matching found, find the first one to paint
-    bool isStatic = canvas()->staticMode();
-    if (isStatic && pI.phase == PaintActionSelection) return;
-
-    if(si >= 0)
-    {
-        // Move up until out of area to be painted
-        while(si > 0 && m_lines[si-1]->checkVerticalPoint(pI.r.y(), ty, pI.r.height(), m_lineHeight))
-            si--;
-
-        // Now calculate startPos and endPos, for painting selection.
-        // We paint selection while endPos > 0
-        int endPos, startPos;
-        if (!isStatic && (selectionState() != SelectionNone)) {
-            if (selectionState() == SelectionInside) {
-                //kdDebug(6040) << this << " SelectionInside -> 0 to end" << endl;
-                startPos = 0;
-                endPos = str->l;
-            } else {
-                selectionStartEnd(startPos, endPos);
-                if(selectionState() == SelectionStart)
-                    endPos = str->l;
-                else if(selectionState() == SelectionEnd)
-                    startPos = 0;
-            }
-            //kdDebug(6040) << this << " Selection from " << startPos << " to " << endPos << endl;
-        }
-
-        InlineTextBox* s;
-	const Font *font = &style()->htmlFont();
-
-        bool haveSelection = !isStatic && selectionState() != SelectionNone && startPos != endPos;
-        if (!haveSelection && pI.phase == PaintActionSelection)
-            // When only painting the selection, don't bother to paint if there
-            return;
-
-        // run until we find one that is outside the range, then we
-        // know we can stop
-        do {
-            s = m_lines[si];
-
-            RenderStyle* _style = pseudoStyle && s->m_firstLine ? pseudoStyle : style();
-            int d = _style->textDecorationsInEffect();
-
-            if(_style->font() != pI.p->font())
-                pI.p->setFont(_style->font());
-
-	    //has to be outside the above if because of small caps.
-	    font = &_style->htmlFont();
-
-            if(_style->color() != pI.p->pen().color())
-                pI.p->setPen(_style->color());
-
-#ifdef APPLE_CHANGES
-            // Set a text shadow if we have one.
-            bool setShadow = false;
-            if (_style->textShadow()) {
-                p->setShadow(_style->textShadow()->x, _style->textShadow()->y,
-                             _style->textShadow()->blur, _style->textShadow()->color);
-                setShadow = true;
-            }
-#endif
-
-            if (s->m_len > 0 && pI.phase != PaintActionSelection) {
-	        if (!haveSelection) {
-	            //kdDebug( 6040 ) << "RenderObject::paintObject(" << QConstString(str->s + s->m_start, s->m_len).string() << ") at(" << s->m_x+tx << "/" << s->m_y+ty << ")" << endl;
-#ifndef APPLE_CHANGES
-                    if (_style->textShadow())
-                        s->paintShadow(pI.p, font, tx, ty, _style->textShadow());
-#endif
-// kdDebug(6040) << QConstString(str->s + s->m_start, s->m_len).string().left(40) << endl;
-		    font->drawText(pI.p, s->m_x + tx, s->m_y + ty + s->m_baseline, str->s, str->l, s->m_start, s->m_len,
-				   s->m_toAdd, s->m_reversed ? QPainter::RTL : QPainter::LTR);
-	        }
-		else {
-                    int offset = s->m_start;
-                    int sPos = kMax( startPos - offset, 0 );
-                    int ePos = kMin( endPos - offset, int( s->m_len ) );
-// selected text is always separate in konqueror
-#ifdef APPLE_CHANGES
-                    if (paintSelectedTextSeparately) {
-#endif
-#ifndef APPLE_CHANGES
-                        if (_style->textShadow())
-                            s->paintShadow(pI.p, font, tx, ty, _style->textShadow());
-#endif
-                        if (sPos >= ePos)
-                            font->drawText(pI.p, s->m_x + tx, s->m_y + ty + s->m_baseline,
-                                           str->s, str->l, s->m_start, s->m_len,
-                                           s->m_toAdd, s->m_reversed ? QPainter::RTL : QPainter::LTR);
-                        else {
-                            if (sPos-1 >= 0)
-                                font->drawText(pI.p, s->m_x + tx, s->m_y + ty + s->m_baseline,
-                                               str->s, str->l, s->m_start, s->m_len,
-                                               s->m_toAdd, s->m_reversed ? QPainter::RTL : QPainter::LTR, 0, sPos);
-                            if (ePos < s->m_len)
-                                font->drawText(pI.p, s->m_x + tx, s->m_y + ty + s->m_baseline,
-                                               str->s, str->l, s->m_start, s->m_len,
-                                               s->m_toAdd, s->m_reversed ? QPainter::RTL : QPainter::LTR, ePos, s->m_len);
-                        }
-#ifdef APPLE_CHANGES
-                    }
-
-                    if ( sPos < ePos ) {
-                        if (selectionColor != p->pen().color())
-                            p->setPen(selectionColor);
-
-                        font->drawText(pI.p, s->m_x + tx, s->m_y + ty + s->m_baseline, str->s,
-                                       str->l, s->m_start, s->m_len,
-                                       s->m_toAdd, s->m_reversed ? QPainter::RTL : QPainter::LTR, sPos, ePos);
-                    }
-#endif
-                }
-	    }
-
-            if (d != TDNONE && pI.phase == PaintActionForeground
-                && style()->htmlHacks()) {
-                pI.p->setPen(_style->color());
-                s->paintDecoration(pI.p, font, tx, ty, d);
-            }
-
-            if (haveSelection && pI.phase == PaintActionSelection) {
-		int offset = s->m_start;
-		int sPos = kMax( startPos - offset, 0 );
-		int ePos = kMin( endPos - offset, int( s->m_len ) );
-                //kdDebug(6040) << this << " paintSelection with startPos=" << sPos << " endPos=" << ePos << endl;
-		if ( sPos < ePos )
-		    s->paintSelection(font, this, pI.p, _style, tx, ty, sPos, ePos, d);
-
-            }
-#ifdef BIDI_DEBUG
-            {
-                int h = lineHeight( false ) + paddingTop() + paddingBottom() + borderTop() + borderBottom();
-                QColor c2 = QColor("#0000ff");
-                drawBorder(pI.p, tx + s->m_x, ty + s->m_y, tx + s->m_x + 1, ty + s->m_y + h,
-                              RenderObject::BSLeft, c2, c2, SOLID, 1, 1);
-                drawBorder(pI.p, tx + s->m_x + s->m_width, ty + s->m_y, tx + s->m_x + s->m_width + 1, ty + s->m_y + h,
-                              RenderObject::BSRight, c2, c2, SOLID, 1, 1);
-            }
-#endif
-
-        } while (++si < (int)m_lines.count() && m_lines[si]->checkVerticalPoint(pI.r.y()-ow, ty, pI.r.height(), m_lineHeight));
-
-    }
+    KHTMLAssert( false );
 }
 
 void RenderText::calcMinMaxWidth()
@@ -1044,7 +997,7 @@ void RenderText::calcMinMaxWidth()
             m_hasBeginWS = true;
         if ((isSpace || isNewline) && i == len-1)
             m_hasEndWS = true;
-        
+
         if (i && c == SOFT_HYPHEN)
             continue;
 
@@ -1163,12 +1116,43 @@ void RenderText::setText(DOMStringImpl *text, bool force)
     if ( str && style() ) {
         oldstr = str;
         switch(style()->textTransform()) {
-	case CAPITALIZE:   str = str->capitalize();  break;
-	case UPPERCASE:   str = str->upper();       break;
-	case LOWERCASE:  str = str->lower();       break;
-	case NONE:
-	default:;
-	}
+        case CAPITALIZE:
+        {
+            RenderObject *o;
+            bool runOnString = false;
+
+            // find previous non-empty text renderer if one exists
+            for (o = previousRenderer(); o; o = o->previousRenderer()) {
+                if (!o->isInlineFlow()) {
+                    if (!o->isText())
+                        break;
+
+                    DOMStringImpl *prevStr = static_cast<RenderText*>(o)->string();
+                    // !prevStr can happen with css like "content:open-quote;"
+                    if (!prevStr)
+                        break;
+
+                    if (prevStr->length() == 0)
+                        continue;
+                    QChar c = (*prevStr)[prevStr->length() - 1];
+                    if (!c.isSpace())
+                        runOnString = true;
+
+                    break;
+                }
+            }
+
+            str = str->capitalize(runOnString);
+        }
+        break;
+
+		
+		
+        case UPPERCASE: str = str->upper();       break;
+        case LOWERCASE: str = str->lower();       break;
+        case NONE:
+        default:;
+    }
         str->ref();
         oldstr->deref();
     }
@@ -1290,11 +1274,11 @@ short RenderText::width() const
     return w;
 }
 
-void RenderText::repaint(bool immediate)
+void RenderText::repaint(Priority p)
 {
     RenderObject *cb = containingBlock();
     if(cb)
-        cb->repaint(immediate);
+        cb->repaint(p);
 }
 
 bool RenderText::isFixedWidthFont() const
