@@ -98,9 +98,10 @@ private:
 #define NodeImpl_IdNSMask    0xffff0000
 #define NodeImpl_IdLocalMask 0x0000ffff
 
-const Q_UINT16 noNamespace = 0;
-const Q_UINT16 anyNamespace = 0xffff;
+const Q_UINT16 defaultNamespace = 0;
 const Q_UINT16 xhtmlNamespace = 1;
+const Q_UINT16 emptyNamespace = 2;
+const Q_UINT16 anyNamespace = 0xffff;
 const Q_UINT16 anyLocalName = 0xffff;
 
 inline Q_UINT16 localNamePart(Q_UINT32 id) { return id & NodeImpl_IdLocalMask; }
@@ -130,8 +131,10 @@ public:
     // insertBefore, replaceChild and appendChild also close newChild
     // unlike the speed optimized addChild (which is used by the parser)
     virtual NodeImpl *insertBefore ( NodeImpl *newChild, NodeImpl *refChild, int &exceptioncode );
-    virtual NodeImpl *replaceChild ( NodeImpl *newChild, NodeImpl *oldChild, int &exceptioncode );
-    virtual NodeImpl *removeChild ( NodeImpl *oldChild, int &exceptioncode );
+    
+    /* These two methods may delete the old node, so make sure to reference it if you need it */
+    virtual void replaceChild ( NodeImpl *newChild, NodeImpl *oldChild, int &exceptioncode );
+    virtual void removeChild ( NodeImpl *oldChild, int &exceptioncode );
     virtual NodeImpl *appendChild ( NodeImpl *newChild, int &exceptioncode );
     virtual bool hasChildNodes (  ) const;
     virtual NodeImpl *cloneNode ( bool deep ) = 0;
@@ -219,8 +222,10 @@ public:
     virtual void sheetLoaded() {}
 
     bool hasID() const      { return m_hasId; }
+    bool hasClass() const   { return m_hasClass; }
     bool active() const     { return m_active; }
-    bool focused() const { return m_focused; }
+    bool focused() const    { return m_focused; }
+    bool hovered() const    { return m_hovered; }
     bool attached() const   { return m_attached; }
     bool closed() const     { return m_closed; }
     bool changed() const    { return m_changed; }
@@ -230,13 +235,23 @@ public:
     bool implicitNode() const { return m_implicit; }
     bool htmlCompat() const { return m_htmlCompat; }
     void setHasID(bool b=true) { m_hasId = b; }
+    void setHasClass(bool b=true) { m_hasClass = b; }
     void setHasChangedChild( bool b = true ) { m_hasChangedChild = b; }
     void setInDocument(bool b=true) { m_inDocument = b; }
     void setHTMLCompat(bool b) { m_htmlCompat = b; }
     virtual void setFocus(bool b=true) { m_focused = b; }
     virtual void setActive(bool b=true) { m_active = b; }
+    virtual void setHovered(bool b=true) { m_hovered = b; }
     virtual void setChanged(bool b=true);
 
+    // for descending restyle when ID or CLASS changes
+    bool changedAscendentAttribute() const { return m_changedAscendentAttribute; }
+    void setChangedAscendentAttribute(bool b) { m_changedAscendentAttribute = b; }
+
+    // for style selection performance: whether the element matches several CSS Classes
+    bool hasClassList() const { return m_hasClassList; }
+    void setHasClassList(bool b) { m_hasClassList = b; }
+
     unsigned short tabIndex() const { return m_tabIndex; }
     void setTabIndex(unsigned short _tabIndex) { m_tabIndex = _tabIndex; }
 
@@ -249,9 +264,10 @@ public:
     virtual void getCaret(int offset, bool override, int &_x, int &_y, int &width, int &height);
     virtual QRect getRect() const;
 
-    enum StyleChange { NoChange, NoInherit, Inherit, Force };
+    enum StyleChange { NoChange, NoInherit, Inherit, Detach, Force };
     virtual void recalcStyle( StyleChange = NoChange ) {}
-    StyleChange diff( khtml::RenderStyle *s1, khtml::RenderStyle *s2 ) const;
+    static StyleChange diff( khtml::RenderStyle *s1, khtml::RenderStyle *s2 );
+    static bool pseudoDiff( khtml::RenderStyle *s1, khtml::RenderStyle *s2, unsigned int pid);
 
     unsigned long nodeIndex() const;
     // Returns the document that this node is associated with. This is guaranteed to always be non-null, as opposed to
@@ -351,11 +367,13 @@ public:
     virtual void detach();
 
     /**
-     * Notifies the node that no more children will be added.
+     * Notifies the node that no more children will be added during parsing.
+     * After a node has been closed all changes must go through the DOM interface.
      */
     virtual void close();
 
-    void closeRenderer();
+    virtual void structureChanged() {};
+    virtual void backwardsStructureChanged() {};
 
     void createRendererIfNeeded();
     virtual khtml::RenderStyle *styleForRenderer(khtml::RenderObject *parent);
@@ -447,18 +465,18 @@ protected:
     bool m_closed : 1;
     bool m_changed : 1;
     bool m_hasChangedChild : 1;
+    bool m_changedAscendentAttribute : 1;
     bool m_inDocument : 1;
     bool m_hasAnchor : 1;
     bool m_specified : 1; // used in AttrImpl. Accessor functions there
-    
+
+    bool m_hovered : 1;
     bool m_focused : 1;
     bool m_active : 1;
     bool m_implicit : 1; // implicitely generated by the parser
-    bool m_rendererNeedsClose : 1;
     bool m_htmlCompat : 1; // true if element was created in HTML compat mode
-    bool m_unused : 1;
-    bool m_unused2 : 1;
-    bool m_unused3 : 1;    
+    bool m_hasClassList : 1;
+    bool m_hasClass : 1;   // true if element has a class property, as relevant to CSS
 };
 
 // this is the full Node Implementation with parents and children.
@@ -473,8 +491,8 @@ public:
     virtual NodeImpl *firstChild() const;
     virtual NodeImpl *lastChild() const;
     virtual NodeImpl *insertBefore ( NodeImpl *newChild, NodeImpl *refChild, int &exceptioncode );
-    virtual NodeImpl *replaceChild ( NodeImpl *newChild, NodeImpl *oldChild, int &exceptioncode );
-    virtual NodeImpl *removeChild ( NodeImpl *oldChild, int &exceptioncode );
+    virtual void replaceChild ( NodeImpl *newChild, NodeImpl *oldChild, int &exceptioncode );
+    virtual void removeChild ( NodeImpl *oldChild, int &exceptioncode );
     virtual NodeImpl *appendChild ( NodeImpl *newChild, int &exceptioncode );
     virtual bool hasChildNodes (  ) const;
 
@@ -494,6 +512,7 @@ public:
 
     virtual void setFocus(bool=true);
     virtual void setActive(bool=true);
+    virtual void setHovered(bool=true);
     virtual unsigned long childNodeCount();
     virtual NodeImpl *childNode(unsigned long index);
 
@@ -556,7 +575,7 @@ public:
     struct Cache: public khtml::Shared<Cache>
     {
         static Cache* make() { return new Cache; }
-    
+
         CacheKey key;//### We must store this in here due to QCache in Qt3 sucking
 
         unsigned int version;
@@ -568,7 +587,7 @@ public:
         unsigned int position;
         unsigned int length;
         bool         hasLength;
-        
+
         void updateNodeListInfo(DocumentImpl* doc);
 
         virtual void clear(DocumentImpl* doc);
@@ -593,7 +612,7 @@ protected:
     NodeImpl *recursiveItem    ( NodeImpl* absStart, NodeImpl *start, unsigned long &offset ) const;
     NodeImpl *recursiveItemBack( NodeImpl* absStart, NodeImpl *start, unsigned long &offset ) const;
 
-    // Override this to determine what nodes to return. Set doRecurse to 
+    // Override this to determine what nodes to return. Set doRecurse to
     // false if the children of this node do not need to be entered.
     virtual bool nodeMatches( NodeImpl *testNode, bool& doRecurse ) const = 0;
 
@@ -604,9 +623,9 @@ protected:
 class ChildNodeListImpl : public NodeListImpl
 {
 public:
- 
+
     ChildNodeListImpl( NodeImpl *n);
-    
+
 protected:
     virtual bool nodeMatches( NodeImpl *testNode, bool& doRecurse ) const;
 };
