@@ -4,7 +4,7 @@
  *           (c) 1999 Mario Weilguni <mweilguni@sime.com>
  *           (c) 2001 Lubos Lunak <l.lunak@kde.org>
  *
- * $Id: kinit.cpp 430034 2005-06-29 22:02:17Z dfaure $
+ * $Id: kinit.cpp 588539 2006-09-26 10:37:51Z lunakl $
  *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Library General Public
@@ -17,8 +17,8 @@
  *
  * You should have received a copy of the GNU Library General Public License
  * along with this library; see the file COPYING.LIB.  If not, write to
- * the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
- * Boston, MA 02111-1307, USA.
+ * the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
+ * Boston, MA 02110-1301, USA.
  */
 
 #include "config.h"
@@ -41,6 +41,7 @@
 #include <stdio.h>
 #include <stdlib.h>
 #include <string.h>
+#include <ctype.h>
 #include <unistd.h>
 #include <locale.h>
 
@@ -290,6 +291,8 @@ static int get_current_desktop( Display* disp )
     {
 	if (type_ret == XA_CARDINAL && format_ret == 32 && nitems_ret == 1)
 	    desktop = *((long *) data_ret) + 1;
+        if (data_ret)
+            XFree ((char*) data_ret);
     }
 #endif
     return desktop;
@@ -383,6 +386,38 @@ QCString execpath_avoid_loops( const QCString& exec, int envc, const char* envs,
      return execpath;
 }
 
+#ifdef KDEINIT_OOM_PROTECT
+static int oom_pipe = -1;
+
+static void oom_protect_sighandler( int ) {
+}
+
+static void reset_oom_protect() {
+   if( oom_pipe <= 0 )
+      return;
+   struct sigaction act, oldact;
+   act.sa_handler = oom_protect_sighandler;
+   act.sa_flags = 0;
+   sigemptyset( &act.sa_mask );
+   sigaction( SIGUSR1, &act, &oldact );
+   sigset_t sigs, oldsigs;
+   sigemptyset( &sigs );
+   sigaddset( &sigs, SIGUSR1 );
+   sigprocmask( SIG_BLOCK, &sigs, &oldsigs );
+   pid_t pid = getpid();
+   if( write( oom_pipe, &pid, sizeof( pid_t )) > 0 ) {
+      sigsuspend( &oldsigs ); // wait for the signal to come
+    }
+   sigprocmask( SIG_SETMASK, &oldsigs, NULL );
+   sigaction( SIGUSR1, &oldact, NULL );
+   close( oom_pipe );
+   oom_pipe = -1;
+}
+#else
+static void reset_oom_protect() {
+}
+#endif
+
 static pid_t launch(int argc, const char *_name, const char *args,
                     const char *cwd=0, int envc=0, const char *envs=0,
                     bool reset_env = false,
@@ -436,7 +471,13 @@ static pid_t launch(int argc, const char *_name, const char *args,
   if (0 > pipe(d.fd))
   {
      perror("kdeinit: pipe() failed!\n");
-     exit(255);
+     d.result = 3;
+     d.errorMsg = i18n("Unable to start new process.\n"
+                       "The system may have reached the maximum number of open files possible or the maximum number of open files that you are allowed to use has been reached.").utf8();
+     close(d.fd[0]);
+     close(d.fd[1]);
+     d.fork = 0;
+     return d.fork;
   }
 
 #if defined Q_WS_X11 && ! defined K_WS_QTONLY
@@ -452,7 +493,12 @@ static pid_t launch(int argc, const char *_name, const char *args,
   switch(d.fork) {
   case -1:
      perror("kdeinit: fork() failed!\n");
-     exit(255);
+     d.result = 3;
+     d.errorMsg = i18n("Unable to create new process.\n"
+                       "The system may have reached the maximum number of processes possible or the maximum number of processes that you are allowed to use has been reached.").utf8();
+     close(d.fd[0]);
+     close(d.fd[1]);
+     d.fork = 0;
      break;
   case 0:
      /** Child **/
@@ -471,6 +517,7 @@ static pid_t launch(int argc, const char *_name, const char *args,
         }
         close( d.launcher[0] );
      }
+     reset_oom_protect();
 
      if (cwd && *cwd)
         chdir(cwd);
@@ -1086,11 +1133,12 @@ static void handle_launcher_request(int sock = -1)
    {
       d.launcher_ok = true;
    }
-   else if ((request_header.cmd == LAUNCHER_EXEC) ||
+   else if (request_header.arg_length && 
+      ((request_header.cmd == LAUNCHER_EXEC) ||
        (request_header.cmd == LAUNCHER_EXT_EXEC) ||
        (request_header.cmd == LAUNCHER_SHELL ) ||
        (request_header.cmd == LAUNCHER_KWRAPPER) ||
-       (request_header.cmd == LAUNCHER_EXEC_NEW))
+       (request_header.cmd == LAUNCHER_EXEC_NEW)))
    {
       pid_t pid;
       klauncher_header response_header;
@@ -1216,7 +1264,7 @@ static void handle_launcher_request(int sock = -1)
       }
       d.debug_wait = false;
    }
-   else if (request_header.cmd == LAUNCHER_SETENV)
+   else if (request_header.arg_length && request_header.cmd == LAUNCHER_SETENV)
    {
       const char *env_name;
       const char *env_value;
@@ -1354,12 +1402,13 @@ static void handle_requests(pid_t waitForPid)
          if (sock >= 0)
          {
 #if defined(KDEINIT_USE_XFT) && defined(KDEINIT_USE_FONTCONFIG)
-            if( !FcConfigUptoDate(NULL))
+            if( FcGetVersion() < 20390 && !FcConfigUptoDate(NULL))
                FcInitReinitialize();
 #endif
             if (fork() == 0)
             {
                 close_fds();
+                reset_oom_protect();
                 handle_launcher_request(sock);
                 exit(255); /* Terminate process. */
             }
@@ -1374,12 +1423,13 @@ static void handle_requests(pid_t waitForPid)
          if (sock >= 0)
          {
 #if defined(KDEINIT_USE_XFT) && defined(KDEINIT_USE_FONTCONFIG)
-            if( !FcConfigUptoDate(NULL))
+            if( FcGetVersion() < 20390 && !FcConfigUptoDate(NULL))
                FcInitReinitialize();
 #endif
             if (fork() == 0)
             {
                 close_fds();
+                reset_oom_protect();
                 handle_launcher_request(sock);
                 exit(255); /* Terminate process. */
             }
@@ -1638,6 +1688,7 @@ int main(int argc, char **argv, char **envp)
    int launch_klauncher = 1;
    int launch_kded = 1;
    int keep_running = 1;
+   int new_startup = 0;
    d.suicide = false;
 
    /** Save arguments first... **/
@@ -1655,6 +1706,12 @@ int main(int argc, char **argv, char **envp)
          d.suicide = true;
       if (strcmp(safe_argv[i], "--exit") == 0)
          keep_running = 0;
+      if (strcmp(safe_argv[i], "--new-startup") == 0)
+         new_startup = 1;
+#ifdef KDEINIT_OOM_PROTECT
+      if (strcmp(safe_argv[i], "--oom-pipe") == 0 && i+1<argc)
+         oom_pipe = atol(argv[i+1]);
+#endif
       if (strcmp(safe_argv[i], "--help") == 0)
       {
         printf("Usage: kdeinit [options]\n");
@@ -1695,13 +1752,15 @@ int main(int argc, char **argv, char **envp)
 
    /** Create our instance **/
    s_instance = new KInstance("kdeinit");
-   // Don't make it the global instance
-   KGlobal::_instance = 0L;
 
    /** Prepare to change process name **/
    kdeinit_initsetproctitle(argc, argv, envp);
    kdeinit_setproctitle("kdeinit Starting up...");
    kdeinit_library_path();
+   // Don't make our instance the global instance
+   // (do it only after kdeinit_library_path, that one indirectly uses KConfig,
+   // which seems to be buggy and always use KGlobal instead of the maching KInstance)
+   KGlobal::_instance = 0L;
    // don't change envvars before kdeinit_initsetproctitle()
    unsetenv("LD_BIND_NOW");
    unsetenv("DYLD_BIND_AT_LAUNCH");
@@ -1745,44 +1804,26 @@ int main(int argc, char **argv, char **envp)
          exit(1);
       }
    }
-
+#ifndef __CYGWIN__
    if (!d.suicide && !getenv("KDE_IS_PRELINKED"))
    {
       QString konq = locate("lib", "libkonq.la", s_instance);
       if (!konq.isEmpty())
 	  (void) lt_dlopen(QFile::encodeName(konq).data());
    }
-
+#endif 
    if (launch_klauncher)
    {
-      pid = launch( 1, "klauncher", 0 );
+      if( new_startup )
+         pid = launch( 2, "klauncher", "--new-startup" );
+      else
+         pid = launch( 1, "klauncher", 0 );
 #ifndef NDEBUG
       fprintf(stderr, "kdeinit: Launched KLauncher, pid = %ld result = %d\n", (long) pid, d.result);
 #endif
       handle_requests(pid); // Wait for klauncher to be ready
    }
-
-   if ( !d.suicide )
-   {
-      pid = launch( 1, "krandrinithack", 0 );
-      if( pid > 0 && d.result == 0 )
-      {
-#ifndef NDEBUG
-         fprintf(stderr, "kdeinit: Launched KRandrInitHack, pid = %ld result = %d\n", (long) pid, d.result);
-#endif
-         WaitPid(pid);
-      }
-   }
-
-   if (launch_kded)
-   {
-      pid = launch( 1, "kded", 0 );
-#ifndef NDEBUG
-      fprintf(stderr, "kdeinit: Launched KDED, pid = %ld result = %d\n", (long) pid, d.result);
-#endif
-      handle_requests(pid);
-   }
-
+   
 #if defined Q_WS_X11 && ! defined K_WS_QTONLY
 //#ifdef Q_WS_X11
    X11fd = initXconnection();
@@ -1790,8 +1831,11 @@ int main(int argc, char **argv, char **envp)
 
    {
 #if defined(KDEINIT_USE_XFT) && defined(KDEINIT_USE_FONTCONFIG)
-      XftInit(0);
-      XftInitFtLibrary();
+      if( FcGetVersion() < 20390 )
+      {
+        XftInit(0);
+        XftInitFtLibrary();
+      }
 #endif
       QFont::initialize();
       setlocale (LC_ALL, "");
@@ -1806,6 +1850,18 @@ int main(int argc, char **argv, char **envp)
 #endif
    }
 
+   if (launch_kded)
+   {
+      if( new_startup )
+         pid = launch( 2, "kded", "--new-startup" );
+      else
+         pid = launch( 1, "kded", 0 );
+#ifndef NDEBUG
+      fprintf(stderr, "kdeinit: Launched KDED, pid = %ld result = %d\n", (long) pid, d.result);
+#endif
+      handle_requests(pid);
+   }
+
    for(i = 1; i < argc; i++)
    {
       if (safe_argv[i][0] == '+')
@@ -1816,7 +1872,11 @@ int main(int argc, char **argv, char **envp)
 #endif
          handle_requests(pid);
       }
-      else if (safe_argv[i][0] == '-')
+      else if (safe_argv[i][0] == '-'
+#ifdef KDEINIT_OOM_PROTECT
+          || isdigit(safe_argv[i][0])
+#endif
+          )
       {
          // Ignore
       }
