@@ -13,7 +13,7 @@
 
   You should have received a copy of the GNU Library General Public License
   along with this library,  If not, write to the Free Software Foundation,
-  Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
+  Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
 */
 
 #include <dcopclient.h>
@@ -35,6 +35,7 @@
 #include <knotifydialog.h>
 #include <kstandarddirs.h>
 #include <kurlrequester.h>
+#include <kio/netaccess.h>
 
 #include <qcheckbox.h>
 #include <qgroupbox.h>
@@ -262,7 +263,7 @@ KNotifyWidget::KNotifyWidget( QWidget *parent, const char *name,
 
     d->toolTip = new KNotifyToolTip( header );
 
-    m_playButton->setPixmap( SmallIcon( "player_play" ) );
+    m_playButton->setIconSet( SmallIconSet( "player_play" ) );
     connect( m_playButton, SIGNAL( clicked() ), SLOT( playSound() ));
 
     connect( m_listview, SIGNAL( currentChanged( QListViewItem * ) ),
@@ -335,7 +336,7 @@ void KNotifyWidget::showAdvanced( bool show )
 {
     if ( show )
     {
-        m_extension->setText( i18n("Fewer Op&tions") );
+        m_extension->setText( i18n("Advanced <<") );
         QToolTip::add( m_extension, i18n("Hide advanced options") );
 
         m_logToFile->show();
@@ -347,11 +348,12 @@ void KNotifyWidget::showAdvanced( bool show )
         m_stderr->show();
         m_taskbar->show();
 
+	m_passivePopup->setEnabled( m_messageBox->isChecked() );
         m_actionsBoxLayout->setSpacing( KDialog::spacingHint() );
     }
     else
     {
-        m_extension->setText( i18n("More Op&tions") );
+        m_extension->setText( i18n("Advanced >>") );
         QToolTip::add( m_extension, i18n("Show advanced options") );
 
         m_logToFile->hide();
@@ -912,6 +914,33 @@ void KNotifyWidget::openExecDialog( KURLRequester *requester )
 
 void KNotifyWidget::playSound()
 {
+    if (!KIO::NetAccess::exists( m_soundPath->url(), true, 0 )) {        
+        bool foundSound=false;
+
+        // find the first "sound"-resource that contains files
+        const Application *app = currentEvent()->application();
+        QStringList soundDirs = KGlobal::dirs()->findDirs("data", app->appName() + "/sounds");
+        soundDirs += KGlobal::dirs()->resourceDirs( "sound" );
+
+        if ( !soundDirs.isEmpty() ) {
+           QDir dir;
+           dir.setFilter( QDir::Files | QDir::Readable );
+           QStringList::ConstIterator it = soundDirs.begin();
+           while ( it != soundDirs.end() ) {
+               dir = *it;
+               if ( dir.isReadable() && dir.count() > 2 && 
+	            KIO::NetAccess::exists( *it + m_soundPath->url(), true, 0 )) {
+                       foundSound=true;
+                       break;
+               }
+               ++it;
+           }
+        }
+        if ( !foundSound ) {
+            KMessageBox::sorry(this, i18n("The specified file does not exist." ));
+            return;
+        }
+    }
     KAudioPlayer::play( m_soundPath->url() );
 }
 
@@ -1056,11 +1085,11 @@ void Application::reloadEvents( bool revertToDefaults )
             e->description = kc->readEntry( comment );
             e->dontShow = kc->readNumEntry("nopresentation", 0 );
             e->configGroup = *it;
-
-            if ( e->name.isEmpty() || e->description.isEmpty() )
+            if ( e->name.isEmpty() && e->description.isEmpty() )
                 delete e;
-
             else { // load the event
+                if( !e->name.isEmpty() && e->description.isEmpty() )
+                        e->description = e->name;
                 // default to passive popups over plain messageboxes
                 int default_rep = kc->readNumEntry("default_presentation",
                                                    0 | KNotifyClient::PassivePopup);
