@@ -1,8 +1,8 @@
 // -*- c-basic-offset: 2 -*-
 /*
  *  This file is part of the KDE libraries
- *  Copyright (C) 1999-2000 Harri Porten (porten@kde.org)
- *  Copyright (C) 2003 Apple Computer, Inc.
+ *  Copyright (C) 1999-2005 Harri Porten (porten@kde.org)
+ *  Copyright (C) 2004 Apple Computer, Inc.
  *
  *  This library is free software; you can redistribute it and/or
  *  modify it under the terms of the GNU Lesser General Public
@@ -16,7 +16,7 @@
  *
  *  You should have received a copy of the GNU Lesser General Public
  *  License along with this library; if not, write to the Free Software
- *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+ *  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
  *
  */
 
@@ -54,6 +54,7 @@
 #include <locale.h>
 #include <ctype.h>
 #include <assert.h>
+#include <limits.h>
 
 #include "date_object.h"
 #include "error_object.h"
@@ -61,6 +62,10 @@
 
 #include "date_object.lut.h"
 
+#ifdef _MSC_VER
+#  define strncasecmp(a,b,c) _strnicmp(a,b,c)
+#endif
+
 using namespace KJS;
 
 // come constants
@@ -95,17 +100,27 @@ static UString formatDateUTCVariant(struct tm &tm)
 
 static UString formatTime(struct tm &tm)
 {
+    int tz;
     char buffer[100];
-    if (tm.tm_gmtoff == 0) {
+#if defined BSD || defined(__linux__) || defined(__APPLE__)
+    tz = tm.tm_gmtoff;
+#else
+#  if defined(__BORLANDC__) || defined (__CYGWIN__)
+    tz = - _timezone;
+#  else
+    tz = - timezone;
+#  endif
+#endif
+    if (tz == 0) {
         snprintf(buffer, sizeof(buffer), "%02d:%02d:%02d GMT", tm.tm_hour, tm.tm_min, tm.tm_sec);
     } else {
-        int offset = tm.tm_gmtoff;
+        int offset = tz;
         if (offset < 0) {
             offset = -offset;
         }
         snprintf(buffer, sizeof(buffer), "%02d:%02d:%02d GMT%c%02d%02d",
                 tm.tm_hour, tm.tm_min, tm.tm_sec,
-                tm.tm_gmtoff < 0 ? '-' : '+', offset / (60*60), (offset / 60) % 60);
+                tz < 0 ? '-' : '+', offset / (60*60), (offset / 60) % 60);
     }
     return UString(buffer);
 }
@@ -175,9 +190,11 @@ static double timeZoneOffset(const struct tm *t)
 #if defined BSD || defined(__linux__) || defined(__APPLE__)
   return -(t->tm_gmtoff / 60);
 #else
-#  if defined(__BORLANDC__)
+#  if defined(__BORLANDC__) || defined(__CYGWIN__)
 // FIXME consider non one-hour DST change
+#if !defined(__CYGWIN__)
 #error please add daylight savings offset here!
+#endif
   return _timezone / 60 - (t->tm_isdst > 0 ? 60 : 0);
 #  else
   return timezone / 60 - (t->tm_isdst > 0 ? 60 : 0 );
@@ -268,7 +285,7 @@ DateInstanceImp::DateInstanceImp(ObjectImp *proto)
 
 // ------------------------------ DatePrototypeImp -----------------------------
 
-const ClassInfo DatePrototypeImp::info = {"Date", 0, &dateTable, 0};
+const ClassInfo DatePrototypeImp::info = {"Date", &DateInstanceImp::info, &dateTable, 0};
 
 /* Source for date_object.lut.h
    We use a negative ID to denote the "UTC" variant.
@@ -340,7 +357,7 @@ Value DatePrototypeImp::get(ExecState *exec, const Identifier &propertyName) con
 
 DateProtoFuncImp::DateProtoFuncImp(ExecState *exec, int i, int len)
   : InternalFunctionImp(
-    static_cast<FunctionPrototypeImp*>(exec->interpreter()->builtinFunctionPrototype().imp())
+    static_cast<FunctionPrototypeImp*>(exec->lexicalInterpreter()->builtinFunctionPrototype().imp())
     ), id(abs(i)), utc(i<0)
   // We use a negative ID to denote the "UTC" variant.
 {
@@ -400,6 +417,14 @@ Value DateProtoFuncImp::call(ExecState *exec, Object &thisObj, const List &args)
     case GetSeconds:
     case GetMilliSeconds:
     case GetTimezoneOffset:
+    case SetTime:
+    case SetMilliSeconds:
+    case SetSeconds:
+    case SetMinutes:
+    case SetHours:
+    case SetDate:
+    case SetMonth:
+    case SetFullYear:
       return Number(NaN);
     }
   }
@@ -481,7 +506,7 @@ Value DateProtoFuncImp::call(ExecState *exec, Object &thisObj, const List &args)
     break;
   case GetYear:
     // IE returns the full year even in getYear.
-    if ( exec->interpreter()->compatMode() != Interpreter::IECompat )
+    if ( exec->dynamicInterpreter()->compatMode() != Interpreter::IECompat )
       result = Number(t->tm_year);
     else
       result = Number(1900 + t->tm_year);
@@ -648,7 +673,7 @@ Object DateObjectImp::construct(ExecState *exec, const List &args)
     }
   }
 
-  Object proto = exec->interpreter()->builtinDatePrototype();
+  Object proto = exec->lexicalInterpreter()->builtinDatePrototype();
   Object ret(new DateInstanceImp(proto.imp()));
   ret.setInternalValue(Number(timeClip(value)));
   return ret;
@@ -750,7 +775,7 @@ static double ymdhms_to_seconds(int year, int mon, int day, int hour, int minute
 
 // we follow the recommendation of rfc2822 to consider all
 // obsolete time zones not listed here equivalent to "-0000"
-static const struct {
+static const struct KnownZone {
 #ifdef _WIN32
     char tzName[4];
 #else
@@ -767,8 +792,7 @@ static const struct {
     { "MST", -420 },
     { "MDT", -360 },
     { "PST", -480 },
-    { "PDT", -420 },
-    { { 0, 0, 0, 0 }, 0 }
+    { "PDT", -420 }
 };
 
 double KJS::makeTime(struct tm *t, double ms, bool utc)
@@ -783,7 +807,7 @@ double KJS::makeTime(struct tm *t, double ms, bool utc)
         t->tm_isdst = t3.tm_isdst;
 #else
         (void)localtime(&zero);
-#  if defined(__BORLANDC__)
+#  if defined(__BORLANDC__) || defined(__CYGWIN__)
         utcOffset = - _timezone;
 #  else
         utcOffset = - timezone;
@@ -842,6 +866,13 @@ static int findMonth(const char *monthStr)
   return -1;
 }
 
+// maybe this should be more often than just isspace() but beware of
+// conflicts with : in time strings.
+static bool isSpaceLike(char c)
+{
+    return isspace(c) || c == ',' || c == ':' || c == '-';
+}
+
 double KJS::KRFCDate_parseDate(const UString &_date)
 {
      // This parse a date in the form:
@@ -872,17 +903,17 @@ double KJS::KRFCDate_parseDate(const UString &_date)
      bool have_time = false;
 
      // Skip leading space
-     while(*dateString && isspace(*dateString))
+     while(*dateString && isSpaceLike(*dateString))
      	dateString++;
 
      const char *wordStart = dateString;
      // Check contents of first words if not number
      while(*dateString && !isdigit(*dateString))
      {
-        if ( isspace(*dateString) && dateString - wordStart >= 3 )
+        if (isSpaceLike(*dateString) && dateString - wordStart >= 3)
         {
           month = findMonth(wordStart);
-          while(*dateString && isspace(*dateString))
+          while(*dateString && isSpaceLike(*dateString))
              dateString++;
           wordStart = dateString;
         }
@@ -895,7 +926,7 @@ double KJS::KRFCDate_parseDate(const UString &_date)
        // TODO: emit warning about dubious format found
      }
 
-     while(*dateString && isspace(*dateString))
+     while(*dateString && isSpaceLike(*dateString))
      	dateString++;
 
      if (!*dateString)
@@ -911,11 +942,11 @@ double KJS::KRFCDate_parseDate(const UString &_date)
      if (!*dateString)
      	return invalidDate;
 
-     if (day < 1)
+     if (day < 0)
        return invalidDate;
      if (day > 31) {
        // ### where is the boundary and what happens below?
-       if (*dateString == '/' && day >= 1000) {
+       if (*dateString == '/') {
          // looks like a YYYY/MM/DD date
          if (!*++dateString)
            return invalidDate;
@@ -933,7 +964,7 @@ double KJS::KRFCDate_parseDate(const UString &_date)
        } else {
          return invalidDate;
        }
-     } else if (*dateString == '/' && day <= 12 && month == -1)
+     } else if (*dateString == '/' && month == -1)
      {
      	dateString++;
         // This looks like a MM/DD/YYYY date, not an RFC date.....
@@ -953,7 +984,7 @@ double KJS::KRFCDate_parseDate(const UString &_date)
        if (*dateString == '-')
          dateString++;
 
-       while(*dateString && isspace(*dateString))
+       while(*dateString && isSpaceLike(*dateString))
          dateString++;
 
        if (*dateString == ',')
@@ -965,7 +996,7 @@ double KJS::KRFCDate_parseDate(const UString &_date)
          if (month == -1)
            return invalidDate;
 
-         while(*dateString && (*dateString != '-') && !isspace(*dateString))
+         while(*dateString && (*dateString != '-') && !isSpaceLike(*dateString))
            dateString++;
 
          if (!*dateString)
@@ -992,13 +1023,12 @@ double KJS::KRFCDate_parseDate(const UString &_date)
      if (*newPosStr)
      {
         // ' 23:12:40 GMT'
-        if (!isspace(*newPosStr)) {
-           if ( *newPosStr == ':' ) // Ah, so there was no year, but the number was the hour
-               year = -1;
-           else
-               return invalidDate;
-        } else // in the normal case (we parsed the year), advance to the next number
+	if (*newPosStr == ':') // Ah, so there was no year, but the number was the hour
+	    year = -1;
+        else if (isSpaceLike(*newPosStr)) // we parsed the year
             dateString = ++newPosStr;
+	else
+	    return invalidDate;
 
         hour = strtol(dateString, &newPosStr, 10);
 
@@ -1043,10 +1073,32 @@ double KJS::KRFCDate_parseDate(const UString &_date)
 
             if ((second < 0) || (second > 59))
               return invalidDate;
+
+	    // disallow trailing colon seconds
+	    if (*dateString == ':')
+		return invalidDate;
           }
 
           while(*dateString && isspace(*dateString))
             dateString++;
+
+	  if (strncasecmp(dateString, "AM", 2) == 0) {
+	    if (hour > 12)
+	      return invalidDate;
+	    if (hour == 12)
+	      hour = 0;
+	    dateString += 2;
+	    while (isspace(*dateString))
+	      dateString++;
+	  } else if (strncasecmp(dateString, "PM", 2) == 0) {
+	    if (hour > 12)
+	      return invalidDate;
+	    if (hour != 12)
+	      hour += 12;
+	    dateString += 2;
+	    while (isspace(*dateString))
+	      dateString++;
+	  }
         }
      } else {
        dateString = newPosStr;
@@ -1056,8 +1108,8 @@ double KJS::KRFCDate_parseDate(const UString &_date)
      // broken mail-/news-clients omit the time zone
      if (*dateString) {
 
-       if ( (dateString[0] == 'G' && dateString[1] == 'M' && dateString[2] == 'T')
-            || (dateString[0] == 'U' && dateString[1] == 'T' && dateString[2] == 'C') )
+       if (strncasecmp(dateString, "GMT", 3) == 0 ||
+	   strncasecmp(dateString, "UTC", 3) == 0)
        {
          dateString += 3;
          have_tz = true;
@@ -1091,7 +1143,7 @@ double KJS::KRFCDate_parseDate(const UString &_date)
            offset = ((offset / 100)*60 + (offset % 100))*sgn;
          have_tz = true;
        } else {
-         for (int i=0; known_zones[i].tzName != 0; i++) {
+         for (int i=0; i < int(sizeof(known_zones)/sizeof(KnownZone)); i++) {
            if (0 == strncasecmp(dateString, known_zones[i].tzName, strlen(known_zones[i].tzName))) {
              offset = known_zones[i].tzOffset;
              dateString += strlen(known_zones[i].tzName);
