@@ -15,8 +15,8 @@
  *
  *  You should have received a copy of the GNU Library General Public License
  *  along with this library; see the file COPYING.LIB.  If not, write to
- *  the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
- *  Boston, MA 02111-1307, USA.
+ *  the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
+ *  Boston, MA 02110-1301, USA.
  *
  */
 
@@ -39,6 +39,7 @@
 #include "identifier.h"
 #include "lookup.h"
 #include "internal.h"
+#include "dtoa.h"
 
 // we can't specify the namespace in yacc's C output, so do it here
 using namespace KJS;
@@ -453,7 +454,7 @@ int Lexer::lex()
 
   long double dval = 0;
   if (state == Number) {
-    dval = strtod(buffer8, 0L);
+    dval = kjs_strtod(buffer8, 0L);
   } else if (state == Hex) { // scan hex numbers
     dval = 0;
     if (buffer8[0] == '0' && (buffer8[1] == 'x' || buffer8[1] == 'X')) {
@@ -507,8 +508,8 @@ int Lexer::lex()
 
   restrKeyword = false;
   delimited = false;
-  yylloc.first_line = yylineno; // ???
-  yylloc.last_line = yylineno;
+  kjsyylloc.first_line = yylineno; // ???
+  kjsyylloc.last_line = yylineno;
 
   switch (state) {
   case Eof:
